package gov.va.shared.util;

import gov.va.shared.model.lookup.LookupType;
import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.model.lookup.sds.VAFacilityType;

public final class StationsUtil {

	/*
	 * This may seem like overkill but it gives us a way to reference where we
	 * are doing this and also we can do add'l logic in the future if SDS
	 * structure changes - CPB
	 */
	public static String getThreeDigitStationNumber(String s) {
		return s.substring(0, 3);
	}

	public static boolean isThreeDigitStation(String s) {
		String threeDigitStationNumber = getThreeDigitStationNumber(s);
		return threeDigitStationNumber.equals(s);
	}

	public static final boolean isValidFacility(VAFacility facility) {
		boolean isValidFacilityType = false;
		boolean isMedicallyTreating = false;

		VAFacilityType facType = facility.getFacilityType();
		if (facType != null) {
			isMedicallyTreating = facType.isMedicalTreating();
			isValidFacilityType = LookupUtil.getById(ValidFacilityTypesEnum.class, facType.getId()) != null;
		}

		boolean isActive = facility.isActive();
		return (isValidFacilityType && isActive && isMedicallyTreating);
	}

	public static final boolean isTestedStationSameOr3DParent(String testedStation, String station) {
		if (station == null || testedStation == null)
			return false;
		return (testedStation.equalsIgnoreCase(station)
				|| StationsUtil.getThreeDigitStationNumber(station).equals(testedStation));
	}

	// --------------------------------- Inner Classes

	public static enum ValidFacilityTypesEnum implements LookupType {
		CBOC(1009148, "CBOC", "COMMUNITY BASED OUTPATIENT CLINIC"), //
		DENT(1009160, "Dent", "DENTAL CLINIC"), //
		DOM(1009158, "Dom", "DOMICILIARY"), //
		MROC(1009174, "M&ROC", "MEDICAL AND REGIONAL OFFFICE CENTER"), //
		NHC(1009185, "NHC", "NURSING HOME CARE"), //
		OC(1009189, "OC", "OUTPATIENT CLINIC (INDEPENDENT)"), //
		OPC(1009197, "OPC", "OUT PATIENT CLINIC"), //
		ORC(1009198, "ORC", "OUTREACH CLINIC"), //
		VAMC(1009231, "VAMC", "VA MEDICAL CENTER"), //
		ROOC(1009209, "RO-OC", "REGIONAL OFFICE - OUTPATIENT CLINIC");

		private long id;
		private String code;
		private String name;

		private ValidFacilityTypesEnum(long id, String code, String name) {
			this.id = id;
			this.code = code;
			this.name = name;
		}

		public long getId() {
			return id;
		}

		public String getCode() {
			return code;
		}

		public String getName() {
			return name;
		}
	}

}
