package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.AppUser;
import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.shared.model.lookup.sds.VAFacility;

/**
 * Represents a vista info for app user at specific vista instance
 * 
 * @author serveruchitr
 * 
 */
@Entity
@Table(name = "APP_USER_VISTA_INFO", schema = "CORE")
@AttributeOverrides({ @AttributeOverride(name = "id", column = @Column(name = "APP_USER_VISTA_INFO_ID") ) })
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = NONE, isGetterVisibility = NONE)
public class AppUserVistaInfo extends AbstractAuditedVersionedPersistent<AppUserVistaInfo> {
	private static final long serialVersionUID = -1207845227302313398L;

	// ---------------------------------------- Fields

	private VAFacility vaFacility;

	private AppUser user;
	private String vistaId;

	// ---------------------------------------- Business Methods

	// ---------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(AppUserVistaInfo oo) {
		return new EqualsBuilder().append(nullSafeGetId(getVAFacility()), nullSafeGetId(oo.getVAFacility()))
				.append(nullSafeGetId(getUser()), nullSafeGetId(oo.getUser())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getVAFacility())).append(nullSafeGetId(getUser()))
				.toHashCode();
	}

	// ---------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VA_INSTITUTION_ID")
	@BatchSize(size=500)
	public VAFacility getVAFacility() {
		return vaFacility;
	}

	public void setVAFacility(VAFacility vaFacility) {
		this.vaFacility = vaFacility;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "APP_USER_ID")
	@BatchSize(size=500)
	public AppUser getUser() {
		return user;
	}

	public void setUser(AppUser user) {
		this.user = user;
	}

	@Column(name = "VISTA_ID", length = 20)
	public String getVistaId() {
		return vistaId;
	}

	public void setVistaId(String vistaId) {
		this.vistaId = vistaId;
	}

}
