package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Convert;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.persistence.conversion.OrganizationScopeTypeConverter;

@Entity
@Table(name = "VtkBenefitingServices", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class BenefitingService extends AbstractAuditedVersionedPersistent<BenefitingService>
		implements Comparable<BenefitingService> {
	private static final long serialVersionUID = 6904844123870655771L;

	// -------------------------------------- Fields

	@NotNull
	@Length(max = 4)
	private String code;
	@NotBlank
	@Length(max = 35)
	private String name;
	@NotBlank
	@Length(max = 30)
	private String subdivision;
	private String abbreviation;
	@NotNull
	private BenefitingServiceScopeType scope;

	private VtkInstitution vtkInstitution;

	// -------------------------------------- Business Methods

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(BenefitingService oo) {
		/*
		 * doubt all three of these are necessary but don't have data to confirm
		 * yet - CPB
		 */
		return new EqualsBuilder().append(code, oo.getCode())
				.append(nullSafeGetId(vtkInstitution), nullSafeGetId(oo.getVtkInstitution())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(code).append(nullSafeGetId(getVtkInstitution())).toHashCode();
	}

	@Override
	public int compareTo(BenefitingService o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(nullSafeLowercase(name), nullSafeLowercase(o.getName())).toComparison() > 0
				? 1 : -1;
	}

	public String toString() {
		return getCode() + " (station " + getVtkInstitution().getStationNumber() + ", name '" + name + "')";
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "StationFK", nullable = false)
	public VtkInstitution getVtkInstitution() {
		return vtkInstitution;
	}

	public void setVtkInstitution(VtkInstitution vtkInstitution) {
		this.vtkInstitution = vtkInstitution;
	}

	@Column(length = 4, nullable = false)
	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	@Column(name = "ServiceName", length = 35, nullable = false)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(length = 30, nullable = false)
	public String getSubdivision() {
		return subdivision;
	}

	public void setSubdivision(String subdivision) {
		this.subdivision = subdivision;
	}

	@Column(length = 7)
	public String getAbbreviation() {
		return abbreviation;
	}

	public void setAbbreviation(String abbreviation) {
		this.abbreviation = abbreviation;
	}

	@Convert(converter = OrganizationScopeTypeConverter.class)
	public BenefitingServiceScopeType getScope() {
		return scope;
	}

	public void setScope(BenefitingServiceScopeType scope) {
		this.scope = scope;
	}

}
