package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.VtkInstitution;

@Entity
@Table(name = "VtkDonReference", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class DonationReference extends AbstractAuditedVersionedPersistent<DonationReference> {

	private static final long serialVersionUID = 3900607532835213411L;
	// -------------------------------------- Fields

	@NotNull
	private VtkInstitution vtkInstitution;
	@Size(max = 50)
	private String donationReference;

	// -------------------------------------- Constructors

	public DonationReference() {
	}

	public DonationReference(String donationReference) {
		this.donationReference = donationReference;
	}

	// -------------------------------------- Business Methods

	// ---------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(DonationReference oo) {
		return new EqualsBuilder().append(nullSafeGetId(getVtkInstitution()), nullSafeGetId(oo.getVtkInstitution()))
				.append(getDonationReference(), oo.getDonationReference()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getVtkInstitution())).append(getDonationReference())
				.toHashCode();
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "InstitutionFK", nullable = false)
	@JsonIgnore
	public VtkInstitution getVtkInstitution() {
		return vtkInstitution;
	}

	public void setVtkInstitution(VtkInstitution vtkInstitution) {
		this.vtkInstitution = vtkInstitution;
	}

	@Column(name = "DonationReference", length = 50)
	public String getDonationReference() {
		return donationReference;
	}

	public void setDonationReference(String serviceTitle) {
		this.donationReference = serviceTitle;
	}

}
