package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;

@Entity
@Table(name = "VtkEvents", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class Event extends AbstractAuditedVersionedPersistent<Event> {
	private static final long serialVersionUID = 6904844123870655771L;

	// -------------------------------------- Fields

	@NotNull
	private Volunteer volunteer;
	private ShirtSize shirtSize;
	private boolean uniform;
	private Integer numberOfShirts;

	// -------------------------------------- Business Methods

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(Event oo) {
		/*
		 * doubt all three of these are necessary but don't have data to confirm
		 * yet - CPB
		 */
		return new EqualsBuilder().append(nullSafeGetId(volunteer), nullSafeGetId(oo.getVolunteer()))
				.append(nullSafeGetId(shirtSize), nullSafeGetId(oo.getShirtSize())).append(uniform, oo.isUniform())
				.append(numberOfShirts, oo.getNumberOfShirts()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(volunteer)).append(nullSafeGetId(shirtSize)).append(uniform)
				.append(numberOfShirts).toHashCode();
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VolunteerFK", nullable = false)
	public Volunteer getVolunteer() {
		return volunteer;
	}

	public void setVolunteer(Volunteer volunteer) {
		this.volunteer = volunteer;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ShirtSizesFK")
	public ShirtSize getShirtSize() {
		return shirtSize;
	}

	public void setShirtSize(ShirtSize shirtSize) {
		this.shirtSize = shirtSize;
	}

	@Column(name = "UniformFlag", nullable = false)
	public boolean isUniform() {
		return uniform;
	}

	public void setUniform(boolean uniform) {
		this.uniform = uniform;
	}

	public Integer getNumberOfShirts() {
		return numberOfShirts;
	}

	public void setNumberOfShirts(Integer numberOfShirts) {
		this.numberOfShirts = numberOfShirts;
	}

}
