package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.VtkInstitution;

@Entity
@Immutable
@Table(name = "VtkStaffTitles", schema = "dbo")
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
@AttributeOverride(name = "name", column = @Column(name = "title") )
public class VtkStaffTitle extends AbstractAuditedVersionedPersistent<VtkStaffTitle>
		implements Comparable<VtkStaffTitle> {

	private static final long serialVersionUID = 9103380479149326407L;

	private String title;
	private boolean volChiefSupervisor;
	private String description;
	private String scope;
	private VtkInstitution vtkInstitution;
	private boolean chief;

	// ---------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(VtkStaffTitle oo) {
		return new EqualsBuilder().append(getTitle(), oo.getTitle()).append(getScope(), oo.getScope())
				.append(nullSafeGetId(getVtkInstitution()), nullSafeGetId(oo.getVtkInstitution())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getTitle()).append(getScope()).append(nullSafeGetId(getVtkInstitution()))
				.toHashCode();
	}

	@Override
	public int compareTo(VtkStaffTitle o) {
		if (equals(o))
			return 0;
		return new CompareToBuilder().append(getScope(), o.getScope()).append(getTitle(), o.getTitle())
				.toComparison() > 0 ? 1 : -1;
	}

	// -------------------------------------- Accessor Methods

	@Column(name = "TITLE", nullable = false)
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	@Column(name = "ISVOLCHIEFSUPERVISOR", nullable = false)
	public boolean isVolChiefSupervisor() {
		return volChiefSupervisor;
	}

	public void setVolChiefSupervisor(boolean volChiefSupervisor) {
		this.volChiefSupervisor = volChiefSupervisor;
	}

	@Column(name = "DESCRIPTION")
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	@Column(name = "SCOPE")
	public String getScope() {
		return scope;
	}

	public void setScope(String scope) {
		this.scope = scope;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "InstitutionFK", unique = true)
	@JsonIgnore
	public VtkInstitution getVtkInstitution() {
		return vtkInstitution;
	}

	public void setVtkInstitution(VtkInstitution vtkInstitution) {
		this.vtkInstitution = vtkInstitution;
	}

	@Column(name = "ISCHIEF", nullable = false)
	public boolean isChief() {
		return chief;
	}

	public void setChief(boolean chief) {
		this.chief = chief;
	}
	
	

}
