package gov.va.vss.persistence.impl;

import javax.persistence.NoResultException;

import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Repository;

import gov.va.shared.util.StationsUtil;
import gov.va.vss.model.AppUserVistaInfo;
import gov.va.vss.persistence.dao.AppUserVistaInfoDAO;
import gov.va.vss.util.cache.CacheNames;

@Repository
public class AppUserVistaInfoDAOImpl extends GenericHibernateDAOImpl<AppUserVistaInfo>implements AppUserVistaInfoDAO {

	@Cacheable(value = CacheNames.QUERIES_APP_USER_VISTA_INFO_DAO)
	@Override
	public AppUserVistaInfo findByAppUserIdAndStationNumber(Long appUserId, String stationNumber) {
		try {
			return (AppUserVistaInfo) query("from " + AppUserVistaInfo.class.getName()
					+ " where institution.stationNumber = :stationNumber" + " and user.id = :appUserId")
							.setParameter("stationNumber", StationsUtil.getThreeDigitStationNumber(stationNumber))
							.setParameter("appUserId", appUserId).getSingleResult();
		} catch (NoResultException e) {
			return null;
		}
	}

	@Override
	@Cacheable(value = CacheNames.QUERIES_APP_USER_VISTA_INFO_DAO)
	public AppUserVistaInfo findByActiveDirectoryNameAndStationNumber(String activeDirectoryName,
			String stationNumber) {
		try {
			return (AppUserVistaInfo) query(
					"from " + AppUserVistaInfo.class.getName() + " where institution.stationNumber = :stationNumber"
							+ " and upper(user.userCredentials.userID) = :activeDirectoryName")
									.setParameter("stationNumber",
											StationsUtil.getThreeDigitStationNumber(stationNumber))
									.setParameter("activeDirectoryName", activeDirectoryName).getSingleResult();
		} catch (NoResultException e) {
			return null;
		}
	}

}
