package gov.va.vss.service.batch;

import java.io.IOException;
import java.io.Serializable;
import java.util.Map;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.Session;

import org.apache.commons.lang3.Validate;
import org.springframework.jms.core.MessageCreator;

import gov.va.vss.service.messaging.listeners.BatchProcessListener;
import gov.va.vss.util.SerializationUtil;

public class BatchProcessMessageCreator implements MessageCreator {

	BatchProcessType jobType;
	String requesterActiveDirectoryName;
	Map<String, Serializable> jobParamMap;

	public BatchProcessMessageCreator(BatchProcessType jobType, String requesterActiveDirectoryName,
			Map<String, Serializable> jobParamMap) {
		Validate.notNull(jobType, "jobType is required");
		this.jobType = jobType;

		this.requesterActiveDirectoryName = requesterActiveDirectoryName;
		this.jobParamMap = jobParamMap;
	}

	@Override
	public Message createMessage(Session session) throws JMSException {
		MapMessage mapMessage = session.createMapMessage();
		mapMessage.setString(BatchProcessListener.JOB_NAME, jobType.getJobName());
		if (requesterActiveDirectoryName != null) {
			mapMessage.setString(BatchProcessListener.USER_ID, requesterActiveDirectoryName);
		}
		try {
			if (jobParamMap != null) {
				byte[] serializedMap = SerializationUtil.serializeObject((Serializable) jobParamMap);
				mapMessage.setBytes(BatchProcessListener.JOB_PARAM_MAP, serializedMap);
			}
			return mapMessage;
		} catch (IOException e) {
			throw new JMSException("Error serializing map");
		}
	}
}