package gov.va.vss.service.impl;

import java.time.ZonedDateTime;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.model.lookup.JobExecutionType.JobExecutionTypeValue;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;
import gov.va.vss.persistence.queryCustomization.fieldTypes.JobExecutionResultAssociationFieldType;
import gov.va.vss.service.batch.JobExecutionResultItemService;
import gov.va.vss.service.batch.JobExecutionResultService;

@Service
@Profile({ "attended", "unattended" })
public class JobExecutionResultServiceImpl extends AbstractServiceImpl implements JobExecutionResultService {

	@Autowired
	private JobExecutionResultItemService jobExecutionResultItemService;

	@Override
	public JobExecutionResult saveOrUpdate(JobExecutionResult result) {
		return jobExecutionResultDAO.saveOrUpdate(result);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRES_NEW)
	public JobExecutionResult saveOrUpdateInNewTransaction(JobExecutionResult result) {
		return jobExecutionResultDAO.saveOrUpdate(result);
	}

	@Override
	public int deleteByCriteria(Long jobExecutionResultId, ZonedDateTime olderThanDate, JobExecutionTypeValue type,
			Long referenceJobExecutionId, String initiatorAppUserName) {
		jobExecutionResultItemService.deleteByCriteria(jobExecutionResultId, olderThanDate, type,
				referenceJobExecutionId);
		int numDeleted = jobExecutionResultDAO.deleteByCriteria(jobExecutionResultId, olderThanDate, type,
				referenceJobExecutionId, initiatorAppUserName);
		return numDeleted;
	}

	@Override
	public void refresh(JobExecutionResult result) {
		jobExecutionResultDAO.refresh(result);
	}

	@Override
	public void setEndDateOrRemoveIfEmpty(long jobExecutionResultId) {
		JobExecutionResult result = jobExecutionResultDAO.findRequiredByPrimaryKey(jobExecutionResultId,
				new QueryCustomization(JobExecutionResultAssociationFieldType.RESULT_ITEMS));

		if (result.getJobExecutionResultItems().isEmpty()) {
			deleteByCriteria(jobExecutionResultId, null, null, null, null);
		} else {
			result.setEndTime(ZonedDateTime.now());
			result = saveOrUpdate(result);
		}
	}

}
