package gov.va.vss.service.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import gov.va.shared.model.AppUser;
import gov.va.shared.service.AppUserService;
import gov.va.shared.service.CreateOrRetrieveUserCallback;
import gov.va.vss.model.VtkUser;
import gov.va.vss.service.AppUserAndVtkUser;
import gov.va.vss.service.VtkUserService;
import gov.va.vss.service.validation.ServiceValidationException;

@Service
public class VtkUserServiceImpl extends AbstractServiceImpl implements VtkUserService {
	private static final Logger log = LoggerFactory.getLogger(VtkUserServiceImpl.class);

	@Autowired
	private AppUserService appUserService;

	@Override
	public VtkUser saveOrUpdate(VtkUser vtkUser) throws ServiceValidationException {
		return vtkUserDAO.saveOrUpdate(vtkUser);
	}

	@Override
	public void delete(long vtkUserId) {
		vtkUserDAO.delete(vtkUserId);
	}

	@Override
	public VtkUser createOrRetrieveVtkUser(String username) {
		VtkUser user = vtkUserDAO.findByAppUser(username);
		if (user == null) {
			AppUser appUser = appUserDAO.findRequiredByUsername(username, false);
			user = new VtkUser(appUser);
			user = vtkUserDAO.saveOrUpdate(user);
		}

		return user;
	}

	@Override
	public AppUserAndVtkUser createOrRetrieveAppUser(String username) {
		MyCreateOrRetrieveUserCallback callback = new MyCreateOrRetrieveUserCallback();
		AppUser appUser = appUserService.createOrRetrieveUser(username, callback);
		return new AppUserAndVtkUser(appUser, callback.vtkUser);
	}

	class MyCreateOrRetrieveUserCallback implements CreateOrRetrieveUserCallback {
		VtkUser vtkUser;

		@Override
		public AppUser userRetrieved(AppUser u) {
			vtkUser = createOrRetrieveVtkUser(u.getUsername());
			return u;
		}

		@Override
		public AppUser userCreated(AppUser u) {
			vtkUser = createOrRetrieveVtkUser(u.getUsername());
			return u;
		}
	}

}
