package gov.va.vss.web.reports;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import gov.va.shared.model.AppUserFacility;
import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.vss.model.Organization;
import gov.va.vss.web.AbstractAppController;

@Controller
public class ReportParametersController extends AbstractAppController {

	@RequestMapping(value = "/assignedStations", method = RequestMethod.GET)
	public @ResponseBody List<VAFacility> getAppUser() {

		if (getCurrentUser().isNationalAdmin()) {
			return vaFacilityDAO.findAll();
		} else {
			List<VAFacility> assignedFacilities = new ArrayList<VAFacility>();
			for (AppUserFacility auFac : getCurrentUser().getFacilities()) {
				assignedFacilities.add(auFac.getFacility());
			}
			return assignedFacilities;
		}
	}

	@RequestMapping(value = "/volunteerOrgs", method = RequestMethod.GET)
	public @ResponseBody List<Organization> getVolunteerOrgs() {

		boolean includeNational = false;
		if (getCurrentUser().isNationalAdmin()) {
			includeNational = true;
		}

		VAFacility loggedInSite = getSiteContext();
		List<Long> stationIds = new ArrayList<Long>();
		stationIds.add(loggedInSite.getId());
		List<Organization> volunteerOrgs = organizationDAO.findByCriteria(null, true, stationIds, includeNational);
		return volunteerOrgs;

	}

	@RequestMapping(value = "/monthsOfYear", method = RequestMethod.GET)
	public @ResponseBody Set<MultiSelectGenericCommand<Integer, String>> getMonthsOfYear() {

		Set<MultiSelectGenericCommand<Integer, String>> months = new TreeSet<MultiSelectGenericCommand<Integer, String>>();
		Calendar cal = Calendar.getInstance();
		Map<String, Integer> monthMap = cal.getDisplayNames(Calendar.MONTH, Calendar.LONG, Locale.ENGLISH);
		for (Map.Entry<String, Integer> entry : monthMap.entrySet()) {
			String monthName = entry.getKey();
			Integer monthInt = entry.getValue() + 1;
			months.add(new MultiSelectGenericCommand<Integer, String>(monthInt, monthName));
		}
		return months;
	}

}