package gov.va.vss;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.security.auth.login.LoginContext;
import javax.sql.DataSource;

import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.MessageSource;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.transaction.PlatformTransactionManager;

import gov.va.shared.model.core.IdentifiedPersistent;
import gov.va.shared.persistence.AppUserDAO;
import gov.va.shared.persistence.lookup.sds.GenderDAO;
import gov.va.shared.persistence.lookup.sds.StateDAO;
import gov.va.shared.persistence.lookup.sds.VAFacilityDAO;
import gov.va.shared.persistence.lookup.sds.VAFacilityTypeDAO;
import gov.va.shared.service.AppUserTestService;
import gov.va.shared.service.VAFacilityService;
import gov.va.shared.test.AbstractDAOTest;
import gov.va.vss.config.AppConfig;
import gov.va.vss.config.testOnly.AppTestConfig;
import gov.va.vss.persistence.dao.NationalOfficialDAO;
import gov.va.vss.persistence.dao.OrganizationDAO;
import gov.va.vss.persistence.dao.VistaStationDataDAO;
import gov.va.vss.persistence.dao.VoluntaryServiceStaffDAO;
import gov.va.vss.persistence.dao.VtkUserDAO;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionResultDAO;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionResultItemDAO;
import gov.va.vss.persistence.dao.lookup.ActiveStatusDAO;
import gov.va.vss.persistence.dao.lookup.JobExecutionTypeDAO;
import gov.va.vss.persistence.dao.lookup.StdVAVSTitleDAO;
import gov.va.vss.persistence.dao.lookup.VtkInstitutionDAO;
import gov.va.vss.persistence.dao.lookup.VtkInstitutionTypeDAO;
import gov.va.vss.persistence.dao.lookup.VtkStateDAO;
import gov.va.vss.persistence.dao.views.FacilityAndVisnDAO;
import gov.va.vss.service.VistaStationDataService;
import gov.va.vss.service.VtkService;
import gov.va.vss.test.util.TestObjectFactory;

@ContextConfiguration(classes = { AppConfig.class, AppTestConfig.class })
public abstract class AbstractWebDAOTest<T extends IdentifiedPersistent> extends AbstractDAOTest<T> {

	protected LoginContext vistaLoginContext = null;

	@Autowired
	protected ActiveStatusDAO activeStatusDAO;
	@Autowired
	protected AppUserDAO appUserDAO;
	@Autowired
	protected FacilityAndVisnDAO facilityAndVISNDAO;
	@Autowired
	protected GenderDAO genderDAO;
	@Autowired
	protected JobExecutionResultDAO jobExecutionResultDAO;
	@Autowired
	protected JobExecutionResultItemDAO jobExecutionResultItemDAO;
	@Autowired
	protected JobExecutionTypeDAO jobExecutionTypeDAO;
	@Autowired
	protected NationalOfficialDAO nationalOfficalDAO;	
	@Autowired
	protected OrganizationDAO organizationDAO;
	@Autowired
	protected StateDAO stateDAO;
	@Autowired
	protected StdVAVSTitleDAO stdVAVSTitleDAO;
	
	@Autowired
	protected VAFacilityDAO vaFacilityDAO;
	@Autowired
	protected VAFacilityTypeDAO vaFacilityTypeDAO;
	@Autowired
	protected VistaStationDataDAO vistaStationDataDAO;
	@Autowired
	protected VtkInstitutionDAO vtkInstitutionDAO;
	@Autowired
	protected VtkInstitutionTypeDAO vtkInstitutionTypeDAO;
	@Autowired
	protected VoluntaryServiceStaffDAO voluntaryServiceStaffDAO;
	@Autowired
	protected VtkStateDAO vtkStateDAO;
	@Autowired
	protected VtkUserDAO vtkUserDAO;
	@Autowired
	protected AppUserTestService appUserService;
	@Autowired
	protected VAFacilityService institutionsService;
	@Autowired
	protected VistaStationDataService vistaStationDataService;
	@Autowired
	protected VtkService vtkService;


	@Autowired
	protected JobExplorer jobExplorer;

	@Autowired
	protected DataSource dataSource;
	@Autowired
	protected TestObjectFactory testObjectFactory;

	@Autowired
	protected MessageSource messageSource;

	@PersistenceContext
	protected EntityManager em;

	@Autowired
	@Qualifier("transactionManager")
	protected PlatformTransactionManager tm;

}
