package gov.va.vss.persistence;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import gov.va.vss.AbstractTransactionalWebAppTest;

public class TestBasicDataSourceClusterStore extends AbstractTransactionalWebAppTest {

	private BasicDataSourceClusterStore<String, String> store = new BasicDataSourceClusterStore<String, String>();

	private String testKey1 = "unittest_@!#$*(#ERW";
	private String testVal1 = "123";
	private String testKey2 = "unittest_@WEFWO*#@RW";
	private String testVal2 = "456";

	@Before
	public void prepTable() {
		store.setDataSource(jdbcTemplate.getDataSource());
		store.setTableName("CORE.HZC_STORE_PATIENT_INBOUND_MSGS");
		deleteFromTables(store.tableName);
	}

	@Test
	public void testLoad() {
		Object o = store.load(testKey1);
		Assert.assertNull(o);
		store.store(testKey1, "123");
		o = store.load(testKey1);
		Assert.assertEquals(o, "123");
	}

	@Test
	public void testLoadAll() {
		Object o = store.load(testKey1);
		Assert.assertNull(o);
		o = store.load(testKey2);
		Assert.assertNull(o);

		store.store(testKey1, testVal1);
		store.store(testKey2, testVal2);
		Map<String, String> x = store.loadAll(Arrays.asList(testKey1, testKey2));
		Assert.assertEquals(x.size(), 2);
		Assert.assertEquals(x.get(testKey1), testVal1);
		Assert.assertEquals(x.get(testKey2), testVal2);
	}

	@Test
	public void testLoadAllKeys() {
		Object o = store.load(testKey1);
		Assert.assertNull(o);
		o = store.load(testKey2);
		Assert.assertNull(o);

		store.store(testKey1, testVal1);
		store.store(testKey2, testVal2);
		Set<String> allKeys = store.loadAllKeys();

		Assert.assertEquals(allKeys.size(), 2);
		Assert.assertTrue(allKeys.contains(testKey1));
		Assert.assertTrue(allKeys.contains(testKey2));
	}

	@Test
	public void testStore() {
		Object o = store.load(testKey1);
		Assert.assertNull(o);

		store.store(testKey1, testVal1);
		o = store.load(testKey1);
		Assert.assertEquals(o, testVal1);
	}

	@Test
	public void testStoreAll() {
		Object o = store.load(testKey1);
		Assert.assertNull(o);
		o = store.load(testKey2);
		Assert.assertNull(o);

		Map<String, String> all = new HashMap<>();
		all.put(testKey1, testVal1);
		all.put(testKey2, testVal2);
		store.storeAll(all);
		o = store.load(testKey1);
		Assert.assertEquals(o, testVal1);
		o = store.load(testKey2);
		Assert.assertEquals(o, testVal2);
	}
}
