package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;

import java.time.ZoneId;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.shared.web.conversion.ZoneIdSerializer;

/**
 * Represents a specific Vista instance and various fields the app uses to
 * interact with it.
 * 
 * @author vhaisdbarryc
 */
@Entity
@Table(name = "VISTA_INSTANCE", schema = "CORE")
@AttributeOverrides({ @AttributeOverride(name = "id", column = @Column(name = "VISTA_INSTANCE_ID") ) })
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = NONE, isGetterVisibility = NONE)
public class VistaStationData extends AbstractAuditedVersionedPersistent<VistaStationData> {
	private static final long serialVersionUID = -1207845227302313398L;

	// ---------------------------------------- Fields

	private String stationNumber;
	private ZoneId timeZone;

	// ---------------------------------------- Business Methods

	// ---------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(VistaStationData oo) {
		return new EqualsBuilder().append(getStationNumber(), oo.getStationNumber()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getStationNumber()).toHashCode();
	}

	// ---------------------------------------- Accessor Methods

	@Column(name = "STATION_NUMBER", length = 7)
	@JsonProperty
	public String getStationNumber() {
		return stationNumber;
	}

	public void setStationNumber(String stationNumber) {
		this.stationNumber = stationNumber;
	}

	@Column(name = "TIME_ZONE")
	@JsonSerialize(using = ZoneIdSerializer.class)
	public ZoneId getTimeZone() {
		return timeZone;
	}

	public void setTimeZone(ZoneId timeZone) {
		this.timeZone = timeZone;
	}

}
