package gov.va.vss.model.lookup;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonProperty;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.shared.model.lookup.LookupType;

/* Legacy "Ver" version column ignored - CPB */
@Entity
@Immutable
@Table(name = "VtkState", schema = "dbo")
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = NONE, isGetterVisibility = NONE)
public class VtkState extends AbstractAuditedVersionedPersistent<VtkState> implements Comparable<VtkState> {
	private static final long serialVersionUID = -8678395783438462990L;

	private String abbreviation;
	private String vaCode;
	private String name;

	public VtkState() {
	}

	public VtkState(gov.va.shared.model.lookup.sds.State state) {
		this.abbreviation = state.getPostalName();
		this.vaCode = state.getFipsCode();
		this.name = state.getName();
	}

	@Override
	protected boolean requiredEquals(VtkState oo) {
		return new EqualsBuilder().append(getVaCode(), oo.getVaCode()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getVaCode()).toHashCode();
	}

	@Override
	public int compareTo(VtkState o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(abbreviation, o.getAbbreviation()).toComparison() > 0 ? 1 : -1;
	}

	@Column(name = "StateName", length = 50)
	@JsonProperty
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "Abbreviation", length = 2)
	@JsonProperty
	public String getAbbreviation() {
		return abbreviation;
	}

	public void setAbbreviation(String abbreviation) {
		this.abbreviation = abbreviation;
	}

	@Column(name = "VACode", length = 2)
	@JsonProperty
	public String getVaCode() {
		return vaCode;
	}

	public void setVaCode(String vaCode) {
		this.vaCode = vaCode;
	}

	public static enum VtkStateType implements LookupType {
		COLORADO(13);

		private long id;

		private VtkStateType(long id) {
			this.id = id;
		}

		public long getId() {
			return id;
		}

	}

}
