package gov.va.vss.persistence.impl.jobExecution;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.NoResultException;
import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.jobExecution.JobExecutionMetadata;
import gov.va.vss.model.lookup.JobExecutionType;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionMetadataDAO;
import gov.va.vss.persistence.impl.GenericHibernateDAOImpl;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class JobExecutionMetadataDAOImpl extends GenericHibernateDAOImpl<JobExecutionMetadata>
		implements JobExecutionMetadataDAO {

	@Override
	public JobExecutionMetadata getWithName(JobExecutionType jobExecutionType, String name,
			QueryCustomization... customization) {
		QueryCustomization c = ArrayUtils.isNotEmpty(customization) ? customization[0] : null;

		StringBuilder sb = new StringBuilder("select t from ").append(JobExecutionMetadata.class.getName())
				.append(" t");

		if (c != null)
			c.appendRemainingJoins(sb, "t");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (jobExecutionType != null) {
			whereClauseItems.add("t.type.id = :jobExecutionTypeId");
			params.put("jobExecutionTypeId", jobExecutionType.getId());
		}

		if (name != null) {
			whereClauseItems.add("t.name = :name");
			params.put("name", name);
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, customization);

		try {
			return (JobExecutionMetadata) q.getSingleResult();
		} catch (NoResultException e) {
			return null;
		}
	}

}
