package gov.va.vss.service.impl;

import java.io.Serializable;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.core.JobExecutionException;
import org.springframework.batch.core.JobParameter;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.configuration.JobLocator;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.stereotype.Service;

import gov.va.vss.service.batch.BatchProcessMessageCreator;
import gov.va.vss.service.batch.BatchProcessService;
import gov.va.vss.service.batch.BatchProcessType;

@Service
public class BatchProcessServiceImpl implements BatchProcessService {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(BatchProcessServiceImpl.class);

	public static final String ATTRIBUTE_TARGET_PATIENT_LIST = "patientIdsSelected";
	public static final String BATCH_ACTIONS_ACTION_PARAMS_PREFIX = "actionParams.";
	public static final String BATCH_ACTIONS_REQUEST_PARAM_PREFIX = "batchActions.";

	@Autowired
	@Qualifier("jmsTemplateBatchProcessEvents")
	private JmsTemplate batchProcessJmsTemplate;

	@Autowired
	private JobLauncher jobLauncher;
	@Autowired
	private JobLocator jobLocator;

	public void performBulkOperations(Map<String, Serializable> opParamsMap, String requesterActiveDirectoryName)
			throws Exception {
		batchProcessJmsTemplate.send(new BatchProcessMessageCreator(BatchProcessType.SOME_BATCH_PROCESS,
				requesterActiveDirectoryName, opParamsMap));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * gov.va.vss.service.impl.BatchProcessService#executeJob(java.lang.String,
	 * java.util.Map)
	 */
	@Override
	public void executeJob(String jobName, Map<String, JobParameter> batchJobParameterMap)
			throws JobExecutionException {
		JobParameters jobParameters = new JobParameters(batchJobParameterMap);
		jobLauncher.run(jobLocator.getJob(jobName), jobParameters);
	}

}
