package gov.va.vss.service.impl;

import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import gov.va.shared.model.AppUser;
import gov.va.shared.service.AppUserService;
import gov.va.vss.model.VtkUser;
import gov.va.vss.service.AppUserAndVtkUser;
import gov.va.vss.service.VtkUserService;
import gov.va.vss.service.validation.ServiceValidationException;

@Service
public class VtkUserServiceImpl extends AbstractServiceImpl implements VtkUserService {
	private static final Logger log = LoggerFactory.getLogger(VtkUserServiceImpl.class);

	@Autowired
	private AppUserService appUserService;

	@Override
	public VtkUser saveOrUpdate(VtkUser vtkUser) throws ServiceValidationException {
		return vtkUserDAO.saveOrUpdate(vtkUser);
	}

	@Override
	public int delete(long vtkUserId) {
		vtkUserDAO.delete(vtkUserId);
		return 1;
	}

	@Override
	public VtkUser createOrRetrieveVtkUser(String username) {
		VtkUser user = vtkUserDAO.findByAppUser(username);
		if (user == null) {
			AppUser appUser = appUserDAO.findRequiredByUsername(username, false);
			user = new VtkUser(appUser);
			user = vtkUserDAO.saveOrUpdate(user);
		}

		return user;
	}

	@Override
	public AppUserAndVtkUser createOrRetrieveAppUser(String username) {
		Map<String, Object> userAdminCustomizationsModel = new HashMap<>();

		AppUser appUser = appUserService.createOrRetrieveUser(username, userAdminCustomizationsModel);
		return new AppUserAndVtkUser(appUser, (VtkUser) userAdminCustomizationsModel.get("vtkUser"));
	}

	@Override
	public int deleteByAppUserId(long appUserId) {
		return vtkUserDAO.deleteByAppUserId(appUserId);
	}

}
