package gov.va.vss.service.scheduledJobs;

import java.util.List;
import java.util.concurrent.locks.Lock;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Profile;
import org.springframework.jmx.export.annotation.ManagedOperation;
import org.springframework.jmx.export.annotation.ManagedResource;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import gov.va.shared.model.AppUser;

@Service
@Profile(value = { "default", "unattended" })
@ManagedResource
public class AppUserUpdateJob extends AbstractScheduledJob {
	private static final Logger log = LoggerFactory.getLogger(AppUserUpdateJob.class);

	@Scheduled(cron = "${scheduledJobs.appUserUpdate.cron}")
	@ManagedOperation
	public void updateAllAppUsers() {
		log.info("{}: Running scheduled job {}", Thread.currentThread().toString(),
				AppUserUpdateJob.class.getSimpleName());

		String lockName = getClass().getName();
		Lock lock = clusterOpsUnattendedOnly.getLock(lockName);
		if (!lock.tryLock())
			return;

		try {
			List<AppUser> allUsers = appUserDAO.findAll();
			for (AppUser appUser : allUsers) {
				appUserService.updateUserFromLDAP(appUser.getId(), false);
			}
		} catch (Throwable t) {
			log.error("Error updating users", t);
		} finally {
			lock.unlock();
		}
	}
}
