package gov.va.vss.web.jobResults;

import static gov.va.vss.persistence.queryCustomization.fieldTypes.JobExecutionResultAssociationFieldType.INITIATOR;
import static gov.va.vss.persistence.queryCustomization.fieldTypes.JobExecutionResultAssociationFieldType.REFERENCE_INSTITUTION;
import static gov.va.vss.persistence.queryCustomization.fieldTypes.JobExecutionResultAssociationFieldType.TYPE;

import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedSet;

import javax.servlet.http.HttpServletResponse;

import org.hibernate.service.spi.ServiceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;

import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.util.SecurityUtil;
import gov.va.shared.util.SessionUtil;
import gov.va.shared.util.StationsUtil;
import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.breadcrumbs.Breadcrumb;

@Controller
public class JobResultsController extends AbstractAppController {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(JobResultsController.class);

	private static final String VIEW_NAME = "jobResultsSearch";
	private static final String JOB_RESULTS_SEARCH_FORM = "jobResultsSearchForm";

	@RequestMapping("job-results-search-form-show")
	@Breadcrumb("Background Job Log")
	public String showJobResultsSearchForm(ModelMap model) throws ServiceException {
		JobResultsSearchForm form = new JobResultsSearchForm();
		ZoneId userTimeZone = SecurityUtil.getCurrentUser().getTimeZone();
		form.setStartDate(ZonedDateTime.now().withZoneSameInstant(userTimeZone).minusHours(1));
		form.setIncludeStationlessJobs(true);
		form.setIncludeStationSpecificJobs(true);

		VAFacility sta6n = SessionUtil.getSiteContext();
		String stationNumber = sta6n.getStationNumber();
		VAFacility sta3n = vaFacilityDAO.findByStationNumber(StationsUtil.getThreeDigitStationNumber(stationNumber));
		form.setStationCode(sta3n);
		populateModelFromForm(form, model);
		return VIEW_NAME;
	}

	@RequestMapping("job-results-search-form-redisplay")
	public String redisplayJobResultsSearchForm(@ModelAttribute JobResultsSearchForm form, ModelMap model)
			throws ServiceException {
		populateModelFromForm(form, model);
		return VIEW_NAME;
	}

	public void populateModelFromForm(JobResultsSearchForm form, ModelMap model) {
		Set<Long> validStations = new HashSet<>();
		if (form.isIncludeStationlessJobs())
			validStations.add(null);
		if (form.isIncludeStationSpecificJobs() && form.getStationCode() != null)
			validStations.add(form.getStationCode().getId());

		List<JobExecutionResult> list = jobExecutionResultDAO.findByCriteria(null, validStations,
				form.getJobExecutionType(), form.getStartDate(), form.getEndDate(),
				new QueryCustomization(INITIATOR, TYPE, REFERENCE_INSTITUTION));
		model.put("jobResults", list);

		model.put(JOB_RESULTS_SEARCH_FORM, form);
		SortedSet<VAFacility> institutions = vaFacilityDAO.findAllThreeDigitStationsSorted();
		model.put("vamcsList", institutions);
		model.put("allJobExecutionTypes", jobExecutionTypeDAO.findAllSorted(true));
	}

	@RequestMapping("job-results-search-form-submit")
	public String processJobResultsCreateFormSubmit(@ModelAttribute JobResultsSearchForm patientSearchForm,
			HttpServletResponse response, ModelMap model) {
		model.put(JOB_RESULTS_SEARCH_FORM, patientSearchForm);
		return "redirect:/job-results-search-form-redisplay";
	}

}
