<%@ include file="../shared/inc_header.jsp"%>

<tiles:importAttribute name="ssrsReportBasename" scope="page" />

<style>
div#ssrsFrameWrapper {
	text-align: center;
	width: 98vw;
	display: block;
}

#ssrsEmbedFrame {
	width: 98vw;
	height: calc(100vh - 390px);
}
</style>


<script type="text/javascript" src="${jsHome}/widget.js"></script>

<%--
	Override the getReportPathExtension method in concrete report class JSPs, if some parameter
	changes the name of the SSRS report generated (e.g. if there are two separate SSRS reports,
	report_visn and report_facility, and a dropdown for the user for "type" with values "visn"
	and "facility", this method would return "_" + the value of that dropdown. CPB
 --%>
<script type="text/javascript">
	var ssrsReportBasename = "<c:out value="${ssrsReportBasename}" />"
	
	/* Default (invalid) implementation for this method to force callers to override this - CPB */
	function getReportPathExtension() {
		alert('Please implement getReportPathExtension() in your report file (return "" if no extension needed)')
	}
	
	/* Map of <widgetId, widget> */
	var allParamWidgets = new Object()
	function getWidget(id) {
		return allParamWidgets[id]
	}
	
	/* Will be redefined by each param widget as they are added to the page */
	var latestReportParameterBaseConfig = {}
	
	function registerWidget(widgetConfig) {
		var widget = new Widget($.extend({}, latestReportParameterBaseConfig, widgetConfig))
		
		if (allParamWidgets[widget.id]) {
			alert('Duplicate widgets detected with ID "' + widget.id)
			return
		}
		
		allParamWidgets[widget.id] = widget
		return widget
	}
	
	/* List of methods to call when all widgets have completed initializing */
	var widgetsInitializedCallback = null
	function setWidgetsInitializedCallback(callback) {
		widgetsInitializedCallback = callback
	}
	
	var preValidationsCallback = null
	/* A function taking a single parameter representing the final parameters to send to the report */
	var preSubmitCallback = null
	var reportSubmitted = false
	
	$(function() {
		var totalInitialized = 0
		var totalToInitialize = Object.keys(allParamWidgets).length
		
		$.each(allParamWidgets, function(widgetId, widget) {
			widget.initialize(function() {
				console.log('Report parameter widget "' + widgetId + '" initialized.')
				totalInitialized++
				
				if (totalInitialized == totalToInitialize) {
					ensureUniqueParametersAcrossWidgets()
					
					if (widgetsInitializedCallback != null)
						widgetsInitializedCallback()
					
					createParamsDialog()
				}
			})
			
			widget.registerChangeEventSelectors()
		})
		
		var showParamsDialog = function() {
			$("#reportParametersWrapperDiv").dialog('open')
		}
		$("#showParamsDialog").click(showParamsDialog)
		
		hidePageTitle()
		adjustIframeSizingAndLoading()
	})
	
	function createParamsDialog() {
		$("#reportParametersWrapperDiv").dialog({
			autoOpen : true,
			modal : true,
			width : 800,
			height : 500,
			closeOnEscape : true,
			draggable : true,
			resizable : true,
			hide: 'fold',
			show: 'fold',
			title: "<tiles:getAsString name="pageTitle" />",
			buttons : {
				'Cancel' : function() {
					$(this).dialog('close')
				},
				'Submit' : function() {
					if (submitReport()) {
						reportSubmitted = true
						$(this).dialog('close')
					}
				}
			}
		})
	}
	
	function adjustIframeSizingAndLoading() {
		$("#ssrsEmbedFrame").load(hideSpinner)
		
		setInterval(function() {
				if (!reportSubmitted) return
				
				var iFrame = $('#ssrsEmbedFrame')[0]
				try {
				var theVal = iFrame.contentWindow.document.body.offsetHeight
				if (!theVal || theVal < 400)
					theVal = 400
				iFrame.style.height = theVal + 'px';
			} catch (ignored) {}
		}, 1000)
	}
	
	/* Common sense check - CPB */
	function ensureUniqueParametersAcrossWidgets() {
		var allParamsTest = new Object()
		$.each(allParamWidgets, function(widgetId, widget) {
			if (!widget.submitAsReportParam) return
			
			$.each(widget.getParameters(), function(index, item) {
				if (typeof allParamsTest[item.paramName] !== 'undefined') {
					alert('Duplicate report parameter detected: ' + item.paramName + ' in widget ' + widget.id)
				}
				
				allParamsTest[item.paramName] = item.paramValue
			})
		})
	}
	
	function submitReport() {
		if (preValidationsCallback) {
			var result = preValidationsCallback()
			if (result != null) {
				displayAttentionDialog(result)
				return false
			}
		}
		
		/* retrieve all widget parameters & add DOM parameter children to the form
		 as hidden elements - CPB */
		var allParams = new Object()
		var validationFailed = false
		
		$.each(allParamWidgets, function(widgetId, widget) {
			var params = widget.getParameters()
			
			/* If the widget's visible, ensure all non-optional parameters have been set by the user - CPB */
			if (widget.visible) {
				var missingParam = false
				$.each(params, function(index, item) {
					if (item.paramValue == null || (!$.isArray(item.paramValue) && $.trim(item.paramValue) == '')) {
						displayAttentionDialog('Please enter a value for parameter "' + item.displayName + '"')
						missingParam = true
						return false
					}
				})
				if (missingParam) {
					validationFailed = true
					return false
				}
			}
			
			if (!widget.submitAsReportParam || widget.bypassAsReportParam) return
		
			$.each(params, function(index, item) {
				allParams[item.paramName] = item.paramValue
			})
		})

		if (validationFailed) return false
		
		if (preSubmitCallback) {
			var result = preSubmitCallback(allParams)
			if (result != null) {
				displayAttentionDialog(result)
				return false
			}
		}
		
		var formAction = '${reportBaseURL}/' + ssrsReportBasename + getReportPathExtension() + '&rc:Parameters=false'
		$('#frmRender').attr('action', formAction)

		// remove previously submitted params, if any
		$('#frmRender .ssrsParametersSubmit').remove()
		
		$.each(allParams, function(key, value) {
			$.each($.isArray(value) ? value : [value], function(index, item) {
				$('<input />').attr('type', 'hidden').attr(
						'name', key).attr('value', item).attr(
						'class', 'ssrsParametersSubmit')
						.appendTo('#frmRender')
			}) 
		})

		showSpinner()
		$('#frmRender').submit()
		return true
	}
</script>

<div>
	<div align="center">
		<a class="buttonAnchor" id="showParamsDialog">Edit Parameters</a>
	</div>

	<div align="center" id="reportParametersWrapperDiv"
		<%-- style="display: none"--%>
		>
		<tiles:insertAttribute name="reportParameters" />
		<form id="frmRender" action="willBeReplaced" method="POST"
			target="ssrsEmbedFrame">
			<input type="hidden" name="rs:Command" value="Render">
		</form>
	</div>

	<div id="ssrsFrameWrapper">
		<iframe id="ssrsEmbedFrame" frameborder="0"
			name="ssrsEmbedFrame"></iframe>
	</div>
</div>