package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Convert;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;
import org.springframework.format.annotation.DateTimeFormat;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.StdCreditCardType;
import gov.va.vss.model.lookup.VtkDonationType;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.model.lookup.VtkState;
import gov.va.vss.util.DateUtil;
import gov.va.vss.web.conversion.VtkDonationTypeConverter;

@Entity
@Table(name = "VssDonationSummary", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class DonationSummary extends AbstractAuditedVersionedPersistent<DonationSummary>
		implements Comparable<DonationSummary> {
	private static final long serialVersionUID = 2676191929993376944L;

	// -------------------------------------- Fields

	private VtkInstitution vtkInstitution;
	private VtkDonationType vtkDonationType;
	private Organization organization;
	private Donor donor;
	private VtkState familyContactState;
	private DonationReference donReference;
	private String ackOverrideNamePrefix;
	private String ackOverrideFirstName;
	private String ackOverrideLastName;
	private String ackOverrideAddress1;
	private String ackOverrideAddress2;
	private String ackOverrideCity;
	private String ackOverrideZip;
	private VtkState ackOverrideState;
	private boolean monetary;
	private String checkNumber;
	private LocalDate checkDate;
	private StdCreditCardType stdCreditCardType;
	private String creditCardTransactionId;
	private String epayTrackingID;
	private LocalDate donationDate;
	private LocalDate acknowledgementDate;
	private String donationDescription;
	private String additionalComments;
	private String fieldServiceReceipt;
	private String designation;
	private String inMemoryOf;
	private String salutation;
	private String cc1;
	private String cc2;
	private String cc3;
	private String cc4;
	private String cc5;
	private String familyContact;
	private String familyContactAddress;
	private String familyContactCity;
	private String familyContactZip;

	private List<DonationDetail> donationDetails;

	// -------------------------------------- Business Methods

	// ---------------------------------------- Common Methods

	@Override
	public int compareTo(DonationSummary oo) {
		if (equals(oo))
			return 0;

		return new CompareToBuilder().append(getDonationDate(), oo.getDonationDate()).toComparison() > 0 ? 1 : -1;
	}

	@Override
	protected boolean requiredEquals(DonationSummary oo) {
		return new EqualsBuilder().append(nullSafeGetId(getDonor()), nullSafeGetId(oo.getDonor()))
				.append(getDonationDate(), oo.getDonationDate()).append(getVtkDonationType(), oo.getVtkDonationType())
				.append(nullSafeGetId(getVtkInstitution()), nullSafeGetId(oo.getVtkInstitution()))
				.append(getDesignation(), oo.getDesignation())
				.append(getDonationDescription(), oo.getDonationDescription()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getDonor())).append(getDonationDate())
				.append(getVtkDonationType()).append(nullSafeGetId(getVtkInstitution())).append(getDesignation())
				.append(getDonationDescription()).toHashCode();
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "InstitutionFK", unique = true)
	@JsonIgnore
	@NotNull
	public VtkInstitution getVtkInstitution() {
		return vtkInstitution;
	}

	public void setVtkInstitution(VtkInstitution vtkInstitution) {
		this.vtkInstitution = vtkInstitution;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "OrganizationFK", unique = true)
	@JsonIgnore
	public Organization getOrganization() {
		return organization;
	}

	public void setOrganization(Organization organization) {
		this.organization = organization;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "DonationTypeFK", unique = true)
	@JsonIgnore
	@NotNull
	public VtkDonationType getVtkDonationType() {
		return vtkDonationType;
	}

	public void setVtkDonationType(VtkDonationType vtkDonationType) {
		this.vtkDonationType = vtkDonationType;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "DonorFK", unique = true)
	@JsonIgnore
	@NotNull
	public Donor getDonor() {
		return donor;
	}

	public void setDonor(Donor donor) {
		this.donor = donor;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "FamilyContactStateFK")
	@JsonIgnore
	public VtkState getFamilyContactState() {
		return familyContactState;
	}

	public void setFamilyContactState(VtkState familyContactState) {
		this.familyContactState = familyContactState;
	}

	@Column(name = "AckOverrideNamePrefix", length = 10)
	public String getAckOverrideNamePrefix() {
		return ackOverrideNamePrefix;
	}

	public void setAckOverrideNamePrefix(String ackOverrideNamePrefix) {
		this.ackOverrideNamePrefix = ackOverrideNamePrefix;
	}

	@Column(name = "AckOverrideFirstName", length = 30)
	public String getAckOverrideFirstName() {
		return ackOverrideFirstName;
	}

	public void setAckOverrideFirstName(String ackOverrideFirstName) {
		this.ackOverrideFirstName = ackOverrideFirstName;
	}

	@Column(name = "ackOverrideLastName", length = 30)
	public String getAckOverrideLastName() {
		return ackOverrideLastName;
	}

	public void setAckOverrideLastName(String ackOverrideLastName) {
		this.ackOverrideLastName = ackOverrideLastName;
	}

	@Column(name = "AckOverrideAddress1", length = 35)
	public String getAckOverrideAddress1() {
		return ackOverrideAddress1;
	}

	public void setAckOverrideAddress1(String ackOverrideAddress1) {
		this.ackOverrideAddress1 = ackOverrideAddress1;
	}

	@Column(name = "AckOverrideAddress2", length = 35)
	public String getAckOverrideAddress2() {
		return ackOverrideAddress2;
	}

	public void setAckOverrideAddress2(String ackOverrideAddress2) {
		this.ackOverrideAddress2 = ackOverrideAddress2;
	}

	@Column(name = "AckOverrideCity", length = 30)
	public String getAckOverrideCity() {
		return ackOverrideCity;
	}

	public void setAckOverrideCity(String ackOverrideCity) {
		this.ackOverrideCity = ackOverrideCity;
	}

	@Column(name = "AckOverrideZip", length = 10)
	public String getAckOverrideZip() {
		return ackOverrideZip;
	}

	public void setAckOverrideZip(String ackOverrideZip) {
		this.ackOverrideZip = ackOverrideZip;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "AckOverrideStateFK")
	@JsonIgnore
	public VtkState getAckOverrideState() {
		return ackOverrideState;
	}

	public void setAckOverrideState(VtkState ackOverrideState) {
		this.ackOverrideState = ackOverrideState;
	}

	@Column(name = "VSS_STD_CreditCardTypeFK", length = 20)
	public StdCreditCardType getStdCreditCardType() {
		return stdCreditCardType;
	}

	public void setStdCreditCardType(StdCreditCardType stdCreditCardType) {
		this.stdCreditCardType = stdCreditCardType;
	}

	@Column(name = "CreditCardTransactionID", length = 20)
	public String getCreditCardTransactionId() {
		return creditCardTransactionId;
	}

	public void setCreditCardTransactionId(String creditCardTransactionId) {
		this.creditCardTransactionId = creditCardTransactionId;
	}

	@Column(name = "EpayTrackingID", length = 15)
	public String getEpayTrackingID() {
		return epayTrackingID;
	}

	public void setEpayTrackingID(String epayTrackingID) {
		this.epayTrackingID = epayTrackingID;
	}

	@Column(name = "IsMonetary")
	@NotNull
	public boolean isMonetary() {
		return monetary;
	}

	public void setMonetary(boolean monetary) {
		this.monetary = monetary;
	}

	@Column(name = "CheckNumber", length = 15)
	public String getCheckNumber() {
		return checkNumber;
	}

	public void setCheckNumber(String checkNumber) {
		this.checkNumber = checkNumber;
	}

	@Column(name = "CheckDate")
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	public LocalDate getCheckDate() {
		return checkDate;
	}

	public void setCheckDate(LocalDate checkDate) {
		this.checkDate = checkDate;
	}

	@Column(name = "DonationDate")
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	public LocalDate getDonationDate() {
		return donationDate;
	}

	public void setDonationDate(LocalDate donationDate) {
		this.donationDate = donationDate;
	}

	@Column(name = "AcknowledgementDate")
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	public LocalDate getAcknowledgementDate() {
		return acknowledgementDate;
	}

	public void setAcknowledgementDate(LocalDate acknowledgementDate) {
		this.acknowledgementDate = acknowledgementDate;
	}

	@Column(name = "DonationDescription", length = 250)
	public String getDonationDescription() {
		return donationDescription;
	}

	public void setDonationDescription(String donationDescription) {
		this.donationDescription = donationDescription;
	}
	
	@Column(name = "AdditionalComments", length = 250)
	public String getAdditionalComments() {
		return additionalComments;
	}

	public void setAdditionalComments(String additionalComments) {
		this.additionalComments = additionalComments;
	}

	@Column(name = "FieldServiceReceipt", length = 12)
	public String getFieldServiceReceipt() {
		return fieldServiceReceipt;
	}

	public void setFieldServiceReceipt(String fieldServiceReceipt) {
		this.fieldServiceReceipt = fieldServiceReceipt;
	}

	@Column(name = "Designation", length = 50)
	public String getDesignation() {
		return designation;
	}

	public void setDesignation(String designation) {
		this.designation = designation;
	}

	@Column(name = "InMemoryOf", length = 250)
	public String getInMemoryOf() {
		return inMemoryOf;
	}

	public void setInMemoryOf(String inMemoryOf) {
		this.inMemoryOf = inMemoryOf;
	}

	@Column(name = "Salutation", length = 50)
	public String getSalutation() {
		return salutation;
	}

	public void setSalutation(String salutation) {
		this.salutation = salutation;
	}

	@Column(name = "CC1", length = 50)
	public String getCc1() {
		return cc1;
	}

	public void setCc1(String cc1) {
		this.cc1 = cc1;
	}

	@Column(name = "CC2", length = 50)
	public String getCc2() {
		return cc2;
	}

	public void setCc2(String cc2) {
		this.cc2 = cc2;
	}

	@Column(name = "CC3", length = 50)
	public String getCc3() {
		return cc3;
	}

	public void setCc3(String cc3) {
		this.cc3 = cc3;
	}

	@Column(name = "CC4", length = 50)
	public String getCc4() {
		return cc4;
	}

	public void setCc4(String cc4) {
		this.cc4 = cc4;
	}

	@Column(name = "CC5", length = 50)
	public String getCc5() {
		return cc5;
	}

	public void setCc5(String cc5) {
		this.cc5 = cc5;
	}

	@Column(name = "FamilyContact", length = 50)
	public String getFamilyContact() {
		return familyContact;
	}

	public void setFamilyContact(String familyContact) {
		this.familyContact = familyContact;
	}

	@Column(name = "FamilyContactAddress", length = 35)
	public String getFamilyContactAddress() {
		return familyContactAddress;
	}

	public void setFamilyContactAddress(String familyContactAddress) {
		this.familyContactAddress = familyContactAddress;
	}

	@Column(name = "FamilyContactCity", length = 30)
	public String getFamilyContactCity() {
		return familyContactCity;
	}

	public void setFamilyContactCity(String familyContactCity) {
		this.familyContactCity = familyContactCity;
	}

	@Column(name = "FamilyContactZipCode", length = 10)
	public String getFamilyContactZip() {
		return familyContactZip;
	}

	public void setFamilyContactZip(String familyContactZip) {
		this.familyContactZip = familyContactZip;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "DonationReferenceFK", unique = true)
	@JsonIgnore
	@NotNull
	public DonationReference getDonReference() {
		return donReference;
	}

	public void setDonReference(DonationReference donReference) {
		this.donReference = donReference;
	}

	@OneToMany(fetch = FetchType.LAZY, mappedBy = "donationSummary", cascade = CascadeType.ALL)
	// @JsonView({ VolunteerView.Extended.class, VolunteerView.Search.class })
	// __JC_TODO__ ???
	@BatchSize(size = 500)
	public List<DonationDetail> getDonationDetails() {
		if (donationDetails == null)
			donationDetails = new ArrayList<>();
		return donationDetails;
	}

	public void setDonationDetails(List<DonationDetail> donationDetails) {
		this.donationDetails = donationDetails;
	}

}
