package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.core.AbstractIdentifiedPersistent;
import gov.va.shared.util.StringUtil;

@Entity
@Table(name = "ExcludedEntity", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class ExcludedEntity extends AbstractIdentifiedPersistent<ExcludedEntity> implements Comparable<ExcludedEntity> {
	private static final long serialVersionUID = 6904844123870655771L;

	// -------------------------------------- Fields

	private String lastName;
	private String firstName;
	private String middleName;
	private String businessName;
	private String general;
	private String specialty;
	private String upin;
	private String npi;
	private String dob;
	private String address;
	private String city;
	private String state;
	private String zip;
	private String exclusionType;
	private String exclusionDate;
	private String reinstateDate;
	private String waiverDate;
	private String waiverState;

	// -------------------------------------- Business Methods

	public ExcludedEntity() {
	}

	public ExcludedEntity(Object[] vals) {
		if (vals.length < 18)
			throw new IllegalArgumentException("The specified vals array must have at least 18 fields");

		this.lastName = nullOrStr(vals[0]);
		this.firstName = nullOrStr(vals[1]);
		this.middleName = nullOrStr(vals[2]);
		this.businessName = nullOrStr(vals[3]);
		this.general = nullOrStr(vals[4]);
		this.specialty = nullOrStr(vals[5]);
		this.upin = nullOrStr(vals[6]);
		this.npi = nullOrStr(vals[7]);
		this.dob = nullOrStr(vals[8]);
		this.address = nullOrStr(vals[9]);
		this.city = nullOrStr(vals[10]);
		this.state = nullOrStr(vals[11]);
		this.zip = nullOrStr(vals[12]);
		this.exclusionType = nullOrStr(vals[13]);
		this.exclusionDate = nullOrStr(vals[14]);
		this.reinstateDate = nullOrStr(vals[15]);
		this.waiverDate = nullOrStr(vals[16]);
		this.waiverState = nullOrStr(vals[17]);
	}

	// -------------------------------------- Business Methods

	private String nullOrStr(Object o) {
		return o == null ? null : String.valueOf(o).trim();
	}

	@Transient
	public String getDisplayName() {
		if (StringUtils.isNotEmpty(businessName))
			return businessName;
		return StringUtil.getDisplayName(true, firstName, middleName, lastName, null);
	}

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(ExcludedEntity o) {
		/*
		 * Ugly but they don't provide a unique key and I see some very close
		 * similarities in the data - CPB
		 */
		return new EqualsBuilder().append(exclusionDate, o.getExclusionDate()).append(lastName, o.getLastName())
				.append(businessName, o.getBusinessName()).append(firstName, o.getFirstName())
				.append(middleName, o.getMiddleName()).append(general, o.getGeneral())
				.append(specialty, o.getSpecialty()).append(dob, o.getDob()).append(address, o.getAddress())
				.append(city, o.getCity()).append(state, o.getState()).append(zip, o.getZip()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(exclusionDate).append(lastName).append(businessName).append(firstName)
				.append(middleName).append(general).append(specialty).append(dob).append(address).append(city)
				.append(state).append(zip).toHashCode();
	}

	@Override
	public int compareTo(ExcludedEntity o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(getDisplayName(), o.getDisplayName()).toComparison() > 0 ? 1 : -1;
	}

	// -------------------------------------- Accessor Methods

	@Column(length = 20)
	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	@Column(length = 15)
	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	@Column(length = 15)
	public String getMiddleName() {
		return middleName;
	}

	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}

	@Column(length = 30)
	public String getBusinessName() {
		return businessName;
	}

	public void setBusinessName(String businessName) {
		this.businessName = businessName;
	}

	@Column(length = 20)
	public String getGeneral() {
		return general;
	}

	public void setGeneral(String general) {
		this.general = general;
	}

	@Column(length = 20)
	public String getSpecialty() {
		return specialty;
	}

	public void setSpecialty(String specialty) {
		this.specialty = specialty;
	}

	@Column(length = 6)
	public String getUpin() {
		return upin;
	}

	public void setUpin(String upin) {
		this.upin = upin;
	}

	@Column(length = 10)
	public String getNpi() {
		return npi;
	}

	public void setNpi(String npi) {
		this.npi = npi;
	}

	@Column(length = 8)
	public String getDob() {
		return dob;
	}

	public void setDob(String dob) {
		this.dob = dob;
	}

	@Column(length = 30)
	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	@Column(length = 20)
	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	@Column(length = 2)
	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	@Column(length = 5)
	public String getZip() {
		return zip;
	}

	public void setZip(String zip) {
		this.zip = zip;
	}

	@Column(length = 9)
	public String getExclusionType() {
		return exclusionType;
	}

	public void setExclusionType(String exclusionType) {
		this.exclusionType = exclusionType;
	}

	@Column(length = 8)
	public String getExclusionDate() {
		return exclusionDate;
	}

	public void setExclusionDate(String exclusionDate) {
		this.exclusionDate = exclusionDate;
	}

	@Column(length = 8)
	public String getReinstateDate() {
		return reinstateDate;
	}

	public void setReinstateDate(String reinstateDate) {
		this.reinstateDate = reinstateDate;
	}

	@Column(length = 8)
	public String getWaiverDate() {
		return waiverDate;
	}

	public void setWaiverDate(String waiverDate) {
		this.waiverDate = waiverDate;
	}

	@Column(length = 2)
	public String getWaiverState() {
		return waiverState;
	}

	public void setWaiverState(String waiverState) {
		this.waiverState = waiverState;
	}

}
