package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.FacilityAssignment.FacilityAssignmentView.SearchForBenefitingServices;
import gov.va.vss.model.FacilityAssignment.FacilityAssignmentView.SearchForFacilityAssignments;
import gov.va.vss.model.lookup.VtkInstitution;

@Entity
@Table(name = "VssFacilityAssignments", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class FacilityAssignment extends AbstractAuditedVersionedPersistent<FacilityAssignment>
		implements Comparable<FacilityAssignment> {
	private static final long serialVersionUID = 6904844123870655771L;

	/**
	 * See @JsonView documentation - CPB
	 */
	public static class FacilityAssignmentView {
		public interface Basic {
		}

		public interface SearchForFacilityAssignments extends Basic {
		}

		public interface SearchForBenefitingServices extends Basic {
		}
	}

	// -------------------------------------- Fields

	private BenefitingService benefitingService;
	private BenefitingServiceRole benefitingServiceRole;
	private VtkInstitution facility;
	private boolean inactive;

	private List<VolunteerFacilityAssignment> volunteerFacilityAssignments;

	// -------------------------------------- Business Methods

	@Transient
	@JsonView(FacilityAssignmentView.Basic.class)
	public boolean isActive() {
		return !inactive;
	}

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(FacilityAssignment oo) {
		return new EqualsBuilder().append(nullSafeGetId(getBenefitingService()), oo.getBenefitingService())
				.append(nullSafeGetId(getFacility()), nullSafeGetId(oo.getFacility())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getBenefitingService())).append(nullSafeGetId(getFacility()))
				.toHashCode();
	}

	@Override
	public int compareTo(FacilityAssignment o) {
		if (equals(o))
			return 0;
		return new CompareToBuilder().append(getFacility(), o.getFacility())
				.append(getBenefitingService(), o.getBenefitingService()).toComparison() > 0 ? 1 : -1;
	}

	// -------------------------------------- Accessor Methods

	@Column(name = "IsInactive", nullable = false)
	@JsonIgnore
	public boolean isInactive() {
		return inactive;
	}

	public void setInactive(boolean inactive) {
		this.inactive = inactive;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "FacilityFK", nullable = false)
	@JsonView(SearchForFacilityAssignments.class)
	public VtkInstitution getFacility() {
		return facility;
	}

	public void setFacility(VtkInstitution facility) {
		this.facility = facility;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssBenefitingServicesFK", nullable = false)
	@JsonView(SearchForFacilityAssignments.class)
	public BenefitingService getBenefitingService() {
		return benefitingService;
	}

	public void setBenefitingService(BenefitingService benefitingService) {
		this.benefitingService = benefitingService;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssBenefitingServiceRolesFK", nullable = false)
	@JsonIgnore
	public BenefitingServiceRole getBenefitingServiceRole() {
		return benefitingServiceRole;
	}

	public void setBenefitingServiceRole(BenefitingServiceRole benefitingServiceRole) {
		this.benefitingServiceRole = benefitingServiceRole;
	}

	@OneToMany(mappedBy = "facilityAssignment", fetch = FetchType.LAZY)
	@BatchSize(size = 500)
	@JsonIgnore
	public List<VolunteerFacilityAssignment> getVolunteerFacilityAssignments() {
		if (volunteerFacilityAssignments == null)
			volunteerFacilityAssignments = new ArrayList<>();
		return volunteerFacilityAssignments;
	}

	public void setVolunteerFacilityAssignments(List<VolunteerFacilityAssignment> volunteerFacilityAssignments) {
		this.volunteerFacilityAssignments = volunteerFacilityAssignments;
	}

}
