package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.Pattern;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.Length;
import org.hibernate.validator.constraints.NotBlank;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.lookup.VtkState;
import gov.va.vss.util.ValidationUtil;

@Entity
@Table(name = "VssBranchOrganizations", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class OrganizationBranch extends AbstractAuditedVersionedPersistent<OrganizationBranch>
		implements Comparable<OrganizationBranch> {
	private static final long serialVersionUID = 6904844123870655771L;

	/**
	 * See @JsonView documentation - CPB
	 */
	public static class OrganizationBranchView {
		public interface Basic {
		}

		public interface Extended extends Basic {
		}
	}

	// -------------------------------------- Fields

	@NotBlank(message = "Display Name is required.")
	@Length(max = 35)
	private String name;

	// private boolean inactive;
	@Length(max = 35)
	private String addressLine1;
	@Length(max = 35)
	private String addressLine2;
	@Length(max = 30)
	private String city;
	private VtkState vtkState;
	@Length(max = 10)
	private String zip;
	@Length(max = 250)
	@Email(message = "Please enter a valid email in the format 'user@domain.tld'.")
	private String email;
	@Length(max = 30)
	@Pattern(regexp = ValidationUtil.PHONE_REGEX, message = "Please enter a valid phone number.")
	private String phone;

	private Organization organization;

	// -------------------------------------- Business Methods

	// @Transient
	// @JsonView(OrganizationBranchView.Extended.class)
	// public boolean isActive() {
	// return !inactive;
	// }

	@Transient
	@JsonView({ OrganizationBranchView.Basic.class })
	public String getDisplayName() {
		return getName();
	}

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(OrganizationBranch oo) {
		return new EqualsBuilder().append(getName(), oo.getName())
				.append(nullSafeGetId(getOrganization()), nullSafeGetId(oo.getOrganization())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getName()).append(nullSafeGetId(getOrganization())).toHashCode();
	}

	@Override
	public int compareTo(OrganizationBranch o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(nullSafeLowercase(getName()), nullSafeLowercase(o.getName()))
				.append(nullSafeGetId(getOrganization()), nullSafeGetId(o.getOrganization())).toComparison() > 0 ? 1
						: -1;
	}

	public String toString() {
		return getName() + " (organization ID " + nullSafeGetId(getOrganization()) + ")";
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssOrganizationsFK")
	@JsonIgnore
	public Organization getOrganization() {
		return organization;
	}

	public void setOrganization(Organization organization) {
		this.organization = organization;
	}

	@Column(length = 50)
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "Address1", length = 35)
	public String getAddressLine1() {
		return addressLine1;
	}

	public void setAddressLine1(String address1) {
		this.addressLine1 = address1;
	}

	@Column(name = "Address2", length = 35)
	public String getAddressLine2() {
		return addressLine2;
	}

	public void setAddressLine2(String address2) {
		this.addressLine2 = address2;
	}

	@Column(length = 30)
	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "STD_StateFK", unique = true)
	@JsonIgnore
	public VtkState getVtkState() {
		return vtkState;
	}

	public void setVtkState(VtkState vtkState) {
		this.vtkState = vtkState;
	}

	@Column(length = 10)
	public String getZip() {
		return zip;
	}

	public void setZip(String zip) {
		this.zip = zip;
	}

	@Column(length = 250)
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	@Column(length = 30)
	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

}
