package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.time.LocalDate;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.util.DateUtil;

@Entity
@Table(name = "VssHours", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class WorkEntry extends AbstractAuditedVersionedPersistent<WorkEntry> implements Comparable<WorkEntry> {
	private static final long serialVersionUID = 6904844123870655771L;

	public static class WorkEntryView {
		public interface Basic {
		}

		public interface Extended extends Basic {
		}
	}

	// -------------------------------------- Fields

	private Organization organization;
	private VolunteerFacilityAssignment volunteerAssignment;
	
	private LocalDate dateWorked;
	private double hoursWorked;
	private String recordStatus;

	// -------------------------------------- Business Methods

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(WorkEntry oo) {
		return new EqualsBuilder()
				.append(nullSafeGetId(getVolunteerAssignment()), nullSafeGetId(oo.getVolunteerAssignment()))
				.append(nullSafeGetId(getOrganization()), nullSafeGetId(oo.getOrganization()))
				.append(getDateWorked(), getDateWorked()).append(getHoursWorked(), getHoursWorked())
				.append(getRecordStatus(), getRecordStatus()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getVolunteerAssignment()))
				.append(nullSafeGetId(getOrganization())).append(getDateWorked()).append(getHoursWorked())
				.append(getRecordStatus()).toHashCode();
	}

	@Override
	public int compareTo(WorkEntry o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(getDateWorked(), o.getDateWorked()).toComparison() > 0 ? 1 : -1;
	}

	public String toString() {
		return "{" + getHoursWorked() + " hours on " + getDateWorked().format(DateUtil.DATE_ONLY_FORMAT) + ")";
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssOrganizationsFK")
	@JsonView(WorkEntryView.Extended.class)
	public Organization getOrganization() {
		return organization;
	}

	public void setOrganization(Organization organization) {
		this.organization = organization;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssVolunteerAssignmentsFK")
	@JsonView(WorkEntryView.Extended.class)
	public VolunteerFacilityAssignment getVolunteerAssignment() {
		return volunteerAssignment;
	}

	public void setVolunteerAssignment(VolunteerFacilityAssignment volunteerAssignment) {
		this.volunteerAssignment = volunteerAssignment;
	}

	@JsonView(WorkEntryView.Basic.class)
	@Column(nullable = false)
	public LocalDate getDateWorked() {
		return dateWorked;
	}

	public void setDateWorked(LocalDate dateWorked) {
		this.dateWorked = dateWorked;
	}

	@JsonView(WorkEntryView.Basic.class)
	@Column(nullable = false, scale = 4, precision = 2)
	public double getHoursWorked() {
		return hoursWorked;
	}

	public void setHoursWorked(double hoursWorked) {
		this.hoursWorked = hoursWorked;
	}

	@Column(length = 1, nullable = false)
	public String getRecordStatus() {
		return recordStatus;
	}

	public void setRecordStatus(String recordStatus) {
		this.recordStatus = recordStatus;
	}

}
