package gov.va.vss.model.jobExecution;

import java.time.ZonedDateTime;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.hibernate.annotations.Type;

import gov.va.shared.model.core.AbstractIdentifiedPersistent;
import gov.va.vss.model.BinaryObject;

@Entity
@Table(name = "JOB_EXECUTION_RESULT_ITEM", schema = "CORE")
public class JobExecutionResultItem extends AbstractIdentifiedPersistent<JobExecutionResultItem>
		implements Comparable<JobExecutionResultItem> {
	private static final long serialVersionUID = 4416728108665979313L;

	// ------------------------------------- Fields

	@NotNull
	private JobExecutionResult jobExecutionResult;

	@NotNull
	private ZonedDateTime startTime;
	private ZonedDateTime endTime;
	private boolean itemSuccessful;
	private String itemStatusMessageStr;

	private BinaryObject statusException;

	// private Patient referencePatient;

	// ------------------------------------- Constructors

	public JobExecutionResultItem() {
	}

	public JobExecutionResultItem(ZonedDateTime startTime) {
		setStartTime(startTime);
	}

	// ------------------------------------- Business Methods

	@Transient
	public String getStatusExceptionTrace() {
		Exception e = getStatusException();
		return e == null ? null : ExceptionUtils.getFullStackTrace(e);
	}

	@Transient
	public Exception getStatusException() {
		BinaryObject o = getStatusExceptionObj();
		return o == null ? null : (Exception) o.getData();
	}

	public void setStatusException(Exception e) {
		if (e == null) {
			setStatusExceptionObj(null);
		} else {
			BinaryObject o = getStatusExceptionObj();
			if (o == null) {
				setStatusExceptionObj(new BinaryObject(e));
			} else {
				o.setData(e);
			}
		}
	}

	// ------------------------------------- Common Methods

	@Override
	public int compareTo(JobExecutionResultItem u) {
		if (equals(u))
			return 0;

		return new CompareToBuilder().append(getStartTime(), u.getStartTime()).append(getEndTime(), u.getEndTime())
				.append(isItemSuccessful(), u.isItemSuccessful())
				.append(getItemStatusMessageStr(), u.getItemStatusMessageStr())
				.append(nullSafeGetId(getStatusExceptionObj()), nullSafeGetId(u.getStatusExceptionObj()))
				// .append(nullSafeGetId(getReferencePatient()),
				// nullSafeGetId(u.getReferencePatient()))
				.toComparison() > 0 ? 1 : -1;
	}

	@Override
	protected boolean requiredEquals(JobExecutionResultItem u) {
		return new EqualsBuilder().append(getStartTime(), u.getStartTime()).append(getEndTime(), u.getEndTime())
				.append(isItemSuccessful(), u.isItemSuccessful())
				.append(getItemStatusMessageStr(), u.getItemStatusMessageStr())
				// .append(nullSafeGetId(getReferencePatient()),
				// nullSafeGetId(u.getReferencePatient()))
				.isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getStartTime()).append(getEndTime()).append(isItemSuccessful())
				.append(getItemStatusMessageStr())
				// .append(nullSafeGetId(getReferencePatient()))
				.toHashCode();
	}

	// ------------------------------------- Accessor Methods

	@Column(name = "STATUS_MESSAGE")
	public String getItemStatusMessageStr() {
		return itemStatusMessageStr;
	}

	public void setItemStatusMessageStr(String itemStatusMessageStr) {
		this.itemStatusMessageStr = itemStatusMessageStr;
	}

	@ManyToOne(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	@JoinColumn(name = "STATUS_EXCEPTION_BINARY_OBJECTS_ID")
	@OnDelete(action = OnDeleteAction.CASCADE)
	public BinaryObject getStatusExceptionObj() {
		return statusException;
	}

	public void setStatusExceptionObj(BinaryObject statusException) {
		this.statusException = statusException;
	}

	@Column(name = "START_TIME")
	public ZonedDateTime getStartTime() {
		return startTime;
	}

	public void setStartTime(ZonedDateTime startTime) {
		this.startTime = startTime;
	}

	@Column(name = "END_TIME")
	public ZonedDateTime getEndTime() {
		return endTime;
	}

	public void setEndTime(ZonedDateTime endTime) {
		this.endTime = endTime;
	}

	@Type(type = "yes_no")
	@Column(name = "JOB_SUCCESS_IND")
	public boolean isItemSuccessful() {
		return itemSuccessful;
	}

	public void setItemSuccessful(boolean itemSuccessful) {
		this.itemSuccessful = itemSuccessful;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "JOB_EXECUTION_RESULT_ID")
	public JobExecutionResult getJobExecutionResult() {
		return jobExecutionResult;
	}

	public void setJobExecutionResult(JobExecutionResult jobExecutionResult) {
		this.jobExecutionResult = jobExecutionResult;
	}

}
