package gov.va.vss.model.lookup;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.util.Arrays;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;

import gov.va.shared.model.core.AbstractIdentifiedPersistent;
import gov.va.shared.model.lookup.LookupType;

@Entity
@Table(name = "VtkDonorType", schema = "dbo")
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class VtkDonorType extends AbstractIdentifiedPersistent<VtkDonorType> {

	private static final long serialVersionUID = -5062512168142239943L;

	private String donorType;

	@Transient
	@JsonIgnore
	public VtkDonorTypeValue getLookupType() {
		return Arrays.stream(VtkDonorTypeValue.class.getEnumConstants()).filter(x -> x.getId() == getId()).findAny()
				.orElse(null);
	}

	@Override
	protected boolean requiredEquals(VtkDonorType oo) {
		return new EqualsBuilder().append(getDonorType(), oo.getDonorType()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getDonorType()).toHashCode();
	}

	@Column(length = 50)
	public String getDonorType() {
		return donorType;
	}

	public void setDonorType(String donorType) {
		this.donorType = donorType;
	}

	public static enum VtkDonorTypeValue implements LookupType {
		INDIVIDUAL(1), //
		ORG_AND_INDIVIDUAL(2), //
		OTHER_AND_INDIVIDUAL(3), ORGANIZATION(4), OTHER_GROUPS(5);

		private long id;

		private VtkDonorTypeValue(long id) {
			this.id = id;
		}

		@Override
		public long getId() {
			return id;
		}

	}

}
