package gov.va.vss.persistence.dao;

import java.time.LocalDate;
import java.util.Collection;
import java.util.List;
import java.util.SortedSet;

import gov.va.vss.model.Volunteer;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

public interface VolunteerDAO extends CustomizableSortedDAO<Volunteer> {

	public static class TimeSummary {
		LocalDate mostRecentWorkEntryDate;
		double priorYearHours;
		double currentYearHours;
		double totalHours;
		double adjustedHours;

		public TimeSummary(LocalDate mostRecentWorkEntryDate, double priorYearHours, double currentYearHours,
				double totalHours, double adjustedHours) {
			this.mostRecentWorkEntryDate = mostRecentWorkEntryDate;
			this.priorYearHours = priorYearHours;
			this.currentYearHours = currentYearHours;
			this.totalHours = totalHours;
			this.adjustedHours = adjustedHours;
		}

		public double getPriorYearHours() {
			return priorYearHours;
		}

		public double getCurrentYearHours() {
			return currentYearHours;
		}

		public double getTotalHours() {
			return totalHours;
		}

		public double getAdjustedHours() {
			return adjustedHours;
		}

		public LocalDate getMostRecentWorkEntryDate() {
			return mostRecentWorkEntryDate;
		}

	}

	/**
	 * 
	 * @param middleName
	 *            TODO
	 * @param code
	 *            TODO
	 * @param dateOfBirth
	 *            TODO
	 * @param addressStreet
	 *            TODO
	 * @param city
	 *            TODO
	 * @param state
	 *            TODO
	 * @param zip
	 *            TODO
	 * @param phone
	 *            TODO
	 * @param email
	 *            TODO
	 * @param activeStatus
	 *            TODO
	 * @param vaFacilityIds
	 *            The IDs of the VAFacilities that are associated with the
	 *            Volunteer. These are assumed to all be VAMC (3-digit station)
	 *            IDs; if any ID is passed in which belongs to a non-VAMC
	 *            station (either a 5+ digit CBOC or a VISN) it will be ignored
	 *            since Volunteers can only be associated to VAMCs.
	 * @param vtkInstitutionIds
	 *            TODO
	 * @param name
	 *            All or part of the first name (case-insensitive)
	 * @param wildcardName
	 *            All or part of the last name (case-insensitive)
	 * @param sta3ns
	 *            The 3-digit station numbers associated with the volunteer. If
	 *            any of these are greater than three digits, an
	 *            IllegalArgumentException is thrown.
	 * @return The list of Volunteers matching the above criteria
	 */
	List<Volunteer> findByCriteria(String firstName, String middleName, String lastName, String code,
			LocalDate dateOfBirth, String addressStreet, String city, String state, String zip, String phone,
			String email, Boolean activeStatus, Collection<Long> vaFacilityIds, Collection<String> vaFacilitySta3ns,
			Collection<Long> vtkInstitutionIds, QueryCustomization... customization);

	List<String> findZipCodesAtFacilities(List<Long> facilityIds, Boolean volunteerActive,
			Boolean facilityAssignmentActive);

	SortedSet<VtkInstitution> findFacilitiesForVolunteer(long volunteerId);

	int getYearsVolunteering(long volunteerId);

	void updateFieldsWithoutVersionIncrement(long volunteerId, Long primaryOrganizationId, Long primaryFacilityId);

	TimeSummary getTimeSummary(long volunteerId);

}
