package gov.va.vss.persistence.impl;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Repository;

import com.hexiong.jdbf.DBFReader;
import com.hexiong.jdbf.JDBFException;

import gov.va.vss.model.ExcludedEntity;
import gov.va.vss.persistence.dao.ExcludedEntityDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class ExcludedEntityDAOImpl extends GenericHibernateSortedDAOImpl<ExcludedEntity> implements ExcludedEntityDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(ExcludedEntityDAOImpl.class);

	@Value("${excludedEntityURL}")
	private String url;

	@SuppressWarnings("unchecked")
	@Override
	public List<ExcludedEntity> findByCriteria(String lastName, QueryCustomization... customization) {
		StringBuilder sb = new StringBuilder("select ee from ").append(ExcludedEntity.class.getName()).append(" ee");

		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "ee");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (StringUtils.isNotBlank(lastName)) {
			whereClauseItems.add("LOWER(ee.lastName) like :lastName");
			params.put("lastName", "%" + lastName.toLowerCase() + "%");
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		return q.getResultList();
	}

	@Override
	public int importData(ImportDataCallback callback) throws IOException {
		HttpGet httpget = new HttpGet(url);

		try (CloseableHttpClient httpclient = HttpClients.createDefault();
				CloseableHttpResponse response = httpclient.execute(httpget)) {
			HttpEntity entity = response.getEntity();
			if (entity == null)
				throw new IOException("No file found at the URL " + url);

			try (InputStream instream = entity.getContent()) {
				ZipInputStream zis = new ZipInputStream(instream);
				ZipEntry e = zis.getNextEntry();
				System.out.println(e.getName() + "; " + e.getSize());

				DBFReader dbfreader = new DBFReader(zis);
				Charset defaultCharset = Charset.defaultCharset();
				int i;
				for (i = 0; dbfreader.hasNextRecord(); i++) {
					Object aobj[] = dbfreader.nextRecord(defaultCharset);
					ExcludedEntity ee = new ExcludedEntity(aobj);
					callback.processRecord(ee);
				}

				return i;
			} catch (JDBFException e) {
				throw new IOException("Couldn't read DBF file", e);
			}
		}
	}

}
