package gov.va.vss.persistence.impl;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import gov.va.shared.model.lookup.sds.VAFacility.VAFacilityValue;
import gov.va.vss.model.LetterTemplate;
import gov.va.vss.model.LetterType;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.persistence.dao.LetterTemplateDAO;
import gov.va.vss.persistence.dao.lookup.VtkInstitutionDAO;

@Repository
public class LetterTemplateDAOImpl extends GenericHibernateSortedDAOImpl<LetterTemplate> implements LetterTemplateDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(LetterTemplateDAOImpl.class);

	@Autowired
	private VtkInstitutionDAO vtkInstitutionDAO;

	@Override
	public LetterTemplate findByCriteria(LetterType type, long facilityId) {
		@SuppressWarnings("unchecked")
		List<LetterTemplate> list = query(
				"from " + LetterTemplate.class.getName() + " c where c.type = :type and c.facility.id = :facilityId")
						.setParameter("type", type).setParameter("facilityId", facilityId).getResultList();
		return list.isEmpty() ? null : list.get(0);
	}

	@Override
	public LetterTemplate findRequiredDefault(LetterType type) {
		VtkInstitution co = vtkInstitutionDAO.findByStationNumber(VAFacilityValue.CENTRAL_OFFICE.getSta3n());
		LetterTemplate t = findByCriteria(type, co.getId());
		if (t == null)
			throw new IllegalStateException("Missing national (central office) letter template for type " + type);
		return t;
	}

}
