package gov.va.vss.persistence.impl.lookup;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Repository;

import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.util.cache.CacheNames;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.persistence.dao.lookup.VtkInstitutionDAO;
import gov.va.vss.persistence.impl.GenericHibernateSortedDAOImpl;

@Repository
public class VtkInstitutionDAOImpl extends GenericHibernateSortedDAOImpl<VtkInstitution> implements VtkInstitutionDAO {

	// @Cacheable(value = CacheNames.QUERIES_VTK_INSTITUTION_DAO)
	public VtkInstitution findByStationNumber(String stationNumber) {
		@SuppressWarnings("unchecked")
		List<VtkInstitution> results = query(
				"from " + VtkInstitution.class.getName() + " where stationNumber = :stationNumber")
						.setParameter("stationNumber", stationNumber).getResultList();
		return results.isEmpty() ? null : results.get(0);
	}

	@Override
	public Map<Long, VtkInstitution> findBySDSFacilities(Collection<Long> vaFacilityIds) {
		Map<Long, VtkInstitution> results = new HashMap<>();

		if (CollectionUtils.isEmpty(vaFacilityIds))
			return results;

		@SuppressWarnings("unchecked")
		List<VtkInstitution> queryResults = query("select i from " + VtkInstitution.class.getName()
				+ " i left join fetch i.vaFacility f where f.id in (:vaFacilityIds)")
						.setParameter("vaFacilityIds", vaFacilityIds).getResultList();
		for (VtkInstitution r : queryResults)
			results.put(r.getVaFacility().getId(), r);
		return results;
	}

	@Override
	// @Cacheable(value = CacheNames.QUERIES_VTK_INSTITUTION_DAO)
	public SortedSet<VAFacility> findVAFacilitiesWithLinkToVtkInstitution() {
		@SuppressWarnings("unchecked")
		List<VAFacility> results = query(
				"select i.vaFacility from " + VtkInstitution.class.getName() + " i where i.vaFacility is not null")
						.getResultList();
		return new TreeSet<>(results);
	}

	@Override
	// @Cacheable(value = CacheNames.QUERIES_VTK_INSTITUTION_DAO)
	public SortedSet<VtkInstitution> findWithLinkToVAFacility() {
		@SuppressWarnings("unchecked")
		List<VtkInstitution> results = query(
				"select i from " + VtkInstitution.class.getName() + " i where vaFacility is not null").getResultList();
		return new TreeSet<>(results);
	}

	@Override
	// @Cacheable(value = CacheNames.QUERIES_VTK_INSTITUTION_DAO)
	public VtkInstitution findBySDSFacility(long vaFacilityId) {
		return findBySDSFacilities(Arrays.asList(vaFacilityId)).get(vaFacilityId);
	}

	@Override
	// @CacheEvict(value = CacheNames.QUERIES_VTK_INSTITUTION_DAO,
	// inefficient but we shouldn't need to save these very often - CPB
	// allEntries = true)
	public VtkInstitution saveOrUpdate(VtkInstitution item) {
		return super.saveOrUpdate(item);
	}

}
