package gov.va.vss.service.impl;

import java.time.LocalDate;

import org.apache.commons.lang.WordUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.util.SessionUtil;
import gov.va.vss.model.Volunteer;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.service.VolunteerService;
import gov.va.vss.service.validation.ServiceValidationException;

@Service
public class VolunteerServiceImpl extends AbstractServiceImpl implements VolunteerService {
	private static final Logger log = LoggerFactory.getLogger(VolunteerServiceImpl.class);

	@Override
	public Volunteer saveOrUpdate(Volunteer volunteer) throws ServiceValidationException {
		if (!volunteer.isPersistent()) {
			volunteer.setEntryDate(LocalDate.now());

			VAFacility f = SessionUtil.getSiteContext();
			if (f != null) {
				VtkInstitution i = vtkInstitutionDAO.findBySDSFacility(f.getId());
				volunteer.setPrimaryFacility(i);
			}
		}
		volunteer.setFirstName(WordUtils.capitalizeFully(volunteer.getFirstName()));
		volunteer.setMiddleName(WordUtils.capitalizeFully(volunteer.getMiddleName()));
		volunteer.setLastName(WordUtils.capitalizeFully(volunteer.getLastName()));
		volunteer.setSuffix(WordUtils.capitalizeFully(volunteer.getSuffix()));

		return volunteerDAO.saveOrUpdate(volunteer);
	}

	@Override
	public void delete(long volunteerId) {
		volunteerDAO.delete(volunteerId);
	}

	@Override
	public void setPrimaryOrganization(long volunteerId, long organizationId) {
		volunteerDAO.updateFieldsWithoutVersionIncrement(volunteerId, organizationId, null);
	}

	@Override
	public void setPrimaryFacility(long volunteerId, long facilityId) {
		volunteerDAO.updateFieldsWithoutVersionIncrement(volunteerId, null, facilityId);
	}

}
