package gov.va.vss.service.scheduledJobs;

import java.time.ZonedDateTime;
import java.util.concurrent.locks.Lock;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Profile;
import org.springframework.jmx.export.annotation.ManagedOperation;
import org.springframework.jmx.export.annotation.ManagedResource;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

@Service
@Profile(value = { "default", "unattended" })
@ManagedResource
public class PurgeJobExecutionResultsJob extends AbstractScheduledJob {
	private static final Logger log = LoggerFactory.getLogger(PurgeJobExecutionResultsJob.class);

	@Value("${scheduledJobs.purgeJobExecutionResults.purgeOlderThanMinutes}")
	private int purgeOlderThanMinutes;

	@Scheduled(cron = "${scheduledJobs.purgeJobExecutionResults.cron}")
	// or if it's on a delay, use this config:
	// @Scheduled(initialDelayString = "${scheduledJobs.startupDelayMillis}", //
	// fixedDelayString = "${scheduledJobs.cprsHeaderSync.fixedDelay}")
	@ManagedOperation
	public void purgeJobExecutionResults() {
		log.info(Thread.currentThread().toString() + ": Running scheduled job {}",
				PurgeJobExecutionResultsJob.class.getSimpleName());

		String lockName = getClass().getName();
		Lock lock = clusterOpsUnattendedOnly.getLock(lockName);
		if (!lock.tryLock())
			return;

		try {
			ZonedDateTime cutoff = ZonedDateTime.now().minusMinutes(purgeOlderThanMinutes);
			jobExecutionResultService.deleteByCriteria(null, cutoff, null, null, null);
		} catch (Throwable t) {
			log.error("Caught throwable in purgeJobExecutionResults()", t);
		} finally {
			lock.unlock();
		}
	}
}
