package gov.va.vss.web.batch;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;

import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.breadcrumbs.Breadcrumb;

@Controller
@SessionAttributes(value = AbstractAppController.DEFAULT_COMMAND_NAME)
public class BatchController extends AbstractAppController {
	private static final Logger log = LoggerFactory.getLogger(BatchController.class);

	public static final String PARAM_OBJECT_IDS_SELECTED = "patientIdsSelected";
	public static final String PARAM_PATIENT_IDS_PROCESSED = "patientIdsProcessed";

	@RequestMapping("jobexec-item-results-show")
	@Breadcrumb("Job Execution Results")
	public String showJobExecutionResults(Model model, @RequestParam long jobExecutionResultId) {
		JobExecutionResult result = jobExecutionResultDAO.findRequiredByPrimaryKey(jobExecutionResultId);
		model.addAttribute("result", result);
		return "jobExecutionItemResults";

	}

	/**
	 * Action phase of the submission of the batch operation launch page
	 * 
	 * @param status
	 * @param response
	 * @param model
	 * @param request
	 */
	@RequestMapping("patient-batch-operation-request-start")
	public String processExampleBatchOperationLauncherSubmitRequest(HttpServletResponse response,
			HttpServletRequest request, @RequestParam String serializedIdsList) {
		try {
			/* Launch the batch process */
			batchProcessServiceImpl.executeJob("someSpringBean", new HashMap<>());

			// Redirect to status page for the batch process for success
			userNotifier.notifyUserOnceWithMessage(request, "A batch operation has been launched.");

			return "redirect:/patient-batch-operation-launcher-submit-success.htm";
		} catch (Exception ex) {
			log.error("An exception was encountered during a batch process", ex);
			userNotifier.notifyUserOnceWithMessage(request,
					"The batch operation launch failed due to an unexpected exception: " + ex + ": " + ex.getMessage());
			return "patient-batch-operation-launcher-submit-failure";
		}
	}

	@RequestMapping("patient-batch-operation-launcher-submit-failure")
	public String showPatientBatchOperationLaunchSubmitFailureMessage(ModelMap model, HttpServletRequest request) {
		createBatchOperationsReferenceData(model);
		return "batchOperationsLaunchedStatus";
	}

	/**
	 * Render phase of submission of batch operation launcher
	 * 
	 * @param model
	 * @param request
	 * @return
	 */
	@RequestMapping("patient-batch-operation-launcher-submit-success")
	public String showPatientBatchOperationLaunchSubmitSuccessMessage(ModelMap model, HttpServletRequest request) {
		createBatchOperationsReferenceData(model);
		return "batchOperationsLaunchedStatus";
	}

	/**
	 * Submission of the form where a list of patients has been selected to
	 * start the launch of a batch operation
	 */
	@RequestMapping("patient-batch-operation-request")
	public String processPatientBatchOperationLauncherDisplayRequest(HttpServletResponse response, ModelMap model,
			HttpServletRequest request, @RequestParam(required = false) String objectsSerializedList) {
		String[] objectIdsSelected = objectsSerializedList == null ? new String[0] : objectsSerializedList.split(",");
		// populate ref data here
		return "patient-batch-operation-launcher-display";
	}

	/**
	 * Submission of the form where a list of patients has been selected to
	 * start the launch of a batch operation
	 */
	@RequestMapping("patient-batch-operation-launcher-display")
	@Breadcrumb("Patient Batch Job")
	public String showPatientBatchOperationLauncher(ModelMap model, HttpServletRequest request) {
		createBatchOperationsReferenceData(model);
		return "batchOperationsLauncher";
	}

	private void createBatchOperationsReferenceData(ModelMap model) {
	}
}
