package gov.va.vss.web.donation;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

import gov.va.shared.model.lookup.sds.State;
import gov.va.shared.util.StringUtil;
import gov.va.vss.model.DonationDetail;
import gov.va.vss.model.DonationSummary;
import gov.va.vss.model.Donor;

public class DonationCommand {

	// -------------------------------- Fields

	private DonationSummary donationSummary;
	private Donor donor;
	private boolean isVolunteer = false;
	private String displayName;
	private String donorCategory;
	private DonationDetail donationDetail1;
	private DonationDetail donationDetail2;
	private DonationDetail donationDetail3;
	private State state;
	private LocalDate currentFiscalYearStartDate;
	private LocalDate currentFiscalYearEndDate;

	// -------------------------------- Constructors

	public DonationCommand() {
	}

	public DonationCommand(DonationSummary donationSummary, State state) {
		this.donationSummary = donationSummary;
		this.donor = donationSummary.getDonor();
		this.state = state;

	}

	// -------------------------------- Accessor Methods

	public DonationSummary getDonationSummary() {
		if (donationSummary != null && donationSummary.getDonationDate() == null)
			donationSummary.setDonationDate(LocalDate.now());
		return donationSummary;
	}

	public void setDonationSummary(DonationSummary donationSummary) {
		this.donationSummary = donationSummary;
	}

	public Donor getDonor() {
		return donor;
	}

	public void setDonor(Donor donor) {
		this.donor = donor;
	}

	public boolean isVolunteer() {
		if (donor.getVolunteer() != null)
			isVolunteer = true;
		return isVolunteer;
	}

	public void setVolunteer(boolean isVolunteer) {
		this.isVolunteer = isVolunteer;
	}

	public String getDisplayName() {
		displayName = StringUtil.getDisplayName(true, donor.getFirstName(), donor.getMiddleName(), donor.getLastName(),
				donor.getSuffix());
		return displayName;
	}

	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}

	public String getDonorCategory() {
		if (donor.getVolunteer() != null)
			donorCategory = "Volunteer";
		else if (donor.getOrganization() != null)
			donorCategory = "Organization";
		else
			donorCategory = "Donor";
		return donorCategory;
	}

	public void setDonorCategory(String donorCategory) {
		this.donorCategory = donorCategory;
	}

	public DonationDetail getDonationDetail1() {
		return donationDetail1;
	}

	public void setDonationDetail1(DonationDetail donationDetail1) {
		this.donationDetail1 = donationDetail1;
	}

	public DonationDetail getDonationDetail2() {
		return donationDetail2;
	}

	public void setDonationDetail2(DonationDetail donationDetail2) {
		this.donationDetail2 = donationDetail2;
	}

	public DonationDetail getDonationDetail3() {
		return donationDetail3;
	}

	public void setDonationDetail3(DonationDetail donationDetail3) {
		this.donationDetail3 = donationDetail3;
	}

	public State getState() {
		return state;
	}

	public void setState(State state) {
		this.state = state;
	}

	public LocalDate getCurrentFiscalYearStartDate() {
		return currentFiscalYearStartDate;
	}

	public void setCurrentFiscalYearStartDate(LocalDate currentFiscalYearStartDate) {
		this.currentFiscalYearStartDate = currentFiscalYearStartDate;
	}

	public LocalDate getCurrentFiscalYearEndDate() {
		return currentFiscalYearEndDate;
	}

	public void setCurrentFiscalYearEndDate(LocalDate currentFiscalYearEndDate) {
		this.currentFiscalYearEndDate = currentFiscalYearEndDate;
	}

	

}
