package gov.va.vss.web.donation;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;

import gov.va.shared.model.Permission;
import gov.va.shared.model.lookup.sds.State;
import gov.va.vss.model.DonGenPostFund;
import gov.va.vss.model.DonationDetail;
import gov.va.vss.model.DonationReference;
import gov.va.vss.model.DonationSummary;
import gov.va.vss.model.Donor;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.model.lookup.VtkState;
import gov.va.vss.service.validation.ServiceValidationException;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.ValidationException;
import gov.va.vss.web.breadcrumbs.Breadcrumb;
import gov.va.vss.web.volunteer.VolunteerValidator;

@Controller
@SessionAttributes(value = { AbstractAppController.DEFAULT_COMMAND_NAME })
public class DonationController extends AbstractAppController {
	@Autowired
	private DonationValidator donationValidator;

	@RequestMapping("/donationEdit.htm")
	@Breadcrumb("Edit Donation")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_READ + "')")
	public String editDonation(@RequestParam long id, ModelMap model, HttpServletRequest request) {
		DonationSummary donation = donationSummaryDAO.findRequiredByPrimaryKey(id);

		VtkState vtkState = donation.getFamilyContactState();
		DonationCommand command = new DonationCommand(donation,
				vtkState == null ? null : stateDAO.findStateByPostalCode(vtkState.getAbbreviation()));
		command.setCurrentFiscalYearStartDate(dateUtil.getCurrentFiscalYearStartDate());
		command.setCurrentFiscalYearEndDate(dateUtil.getCurrentFiscalYearEndDate());

		model.addAttribute(DEFAULT_COMMAND_NAME, command);
		createReferenceData(model, command);
		model.addAttribute("newDonationSummary", false);
		return "editDonation";
	}

	@RequestMapping("/donationCreate.htm")
	@Breadcrumb("Create Donation")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String createDonation(@RequestParam long donorId, ModelMap model, HttpServletRequest request) {
		DonationSummary donation = new DonationSummary();
		VtkInstitution vtkInstitution = vtkService.getOrCreateInstitutionForSDSFacility(getSiteContext().getId());
		donation.setVtkInstitution(vtkInstitution);

		Donor donor = donorDAO.findRequiredByPrimaryKey(donorId);
		donation.setDonor(donor);

		DonationReference donReference = donationReferenceDAO
				.findByInstitutionAndRef(getSiteContext().getStationNumber(), "none");

		if (donation.getDonReference() == null)
			donation.setDonReference(donReference);

		VtkState vtkState = donor.getVtkState();
		DonationCommand command = new DonationCommand(donation,
				vtkState == null ? null : stateDAO.findStateByPostalCode(vtkState.getAbbreviation()));
		command.setCurrentFiscalYearStartDate(dateUtil.getCurrentFiscalYearStartDate());
		command.setCurrentFiscalYearEndDate(dateUtil.getCurrentFiscalYearEndDate());


		model.addAttribute(DEFAULT_COMMAND_NAME, command);
		createReferenceData(model, command);
		return "createDonation";
	}

	@RequestMapping("/donationDetails")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_READ + "')")
	public @ResponseBody List<DonationDetail> getDonationDetailsList(@RequestParam long donationSummaryId) {
		if (donationSummaryId == 0)
			return new ArrayList<DonationDetail>();
		List<DonationDetail> donationDetailList = donationDetailDAO.findByCriteria(donationSummaryId);
		return donationDetailList;
	}

	@RequestMapping("/donationDelete.htm")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String deleteDonationSummary(@RequestParam long donationSummaryId, @RequestParam long donorId)
			throws ServiceValidationException {
		donationService.deleteDonationSummary(donationSummaryId);

		return "redirect:/donorEdit.htm?id=" + donorId;
	}

	@RequestMapping("/deleteDonationDetail")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public @ResponseBody boolean deleteDonationDetail(@RequestParam long donationDetailId)
			throws ServiceValidationException {
		donationService.deleteDonationDetail(donationDetailId);
		return true;
	}

	@RequestMapping("/donationSummarySubmit.htm")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String submitDonationSummary(@ModelAttribute(DEFAULT_COMMAND_NAME) DonationCommand command,
			BindingResult result, SessionStatus status, ModelMap model, HttpServletRequest request)
					throws ValidationException {
		DonationSummary donationSummary = command.getDonationSummary();

		/*
		 * Perform any remaining binding steps from Command obj -> Voluntary
		 * Service Parameters here
		 */

		/* Validation step (JSR303, other custom logic in the validator) */
		
		 donationValidator.validate(command, result, false, "donationSummary");
		 boolean hasErrors = result.hasErrors();

		if (!hasErrors) {
			try {
				State state = command.getState();
				donationSummary
						.setFamilyContactState(state != null ? vtkService.getOrCreateForSDSState(state.getId()) : null);

				donationSummary = donationService.saveOrUpdateDonationSummary(donationSummary);
				userNotifier.notifyUserOnceWithMessage(request, getMessage("donationSummary.update.success"));
			} catch (ServiceValidationException e) {
				webValidationService.handle(e, result);
				hasErrors = true;
			}
		}

		if (hasErrors) {
			createReferenceData(model, command);
			return "editDonation";
		} else {
			status.setComplete();
			return "redirect:/donationEdit.htm?id=" + donationSummary.getId();
		}
	}

	@RequestMapping("/donationDetailCreateOrUpdate")
	@PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public @ResponseBody DonationDetail createOrUpdateDonationDetail(@RequestParam Long donationSummaryId,
			@RequestParam(required = false) Long donationDetailId, @RequestParam DonGenPostFund generalPostFund,
			@RequestParam BigDecimal donationValue) throws ServiceValidationException {

		DonationDetail donationDetail = null;

		if (donationDetailId == null) {
			donationDetail = new DonationDetail();
			DonationSummary donationSummary = donationSummaryDAO.findRequiredByPrimaryKey(donationSummaryId);
			donationDetail.setDonationSummary(donationSummary);
		} else {
			donationDetail = donationDetailDAO.findRequiredByPrimaryKey(donationDetailId.longValue());
		}

		donationDetail.setDonGenPostFund(generalPostFund);
		donationDetail.setDonationValue(donationValue);
		donationDetail = donationService.saveOrUpdateDonationDetail(donationDetail);

		return donationDetail;
	}

	private void createReferenceData(ModelMap model, DonationCommand donationCommand) {
		model.put("allDonationTypes", vtkDonationTypeDAO.findAllSorted());
		VtkInstitution vtkInstitution = vtkService.getOrCreateInstitutionForSDSFacility(getSiteContext().getId());
		model.put("allDonationReferences", vtkInstitution.getDonationReferences());
		model.put("allStates",  stateDAO.findListOfStatesInUSA());
		model.put("allDonGenPostFunds", new TreeSet<DonGenPostFund>(vtkInstitution.getDonGenPostFunds()));
		if (donationCommand.getDonationSummary() != null
				&& donationCommand.getDonationSummary().getDonationDate() != null) {
			boolean isReadOnly = dateUtil.getCurrentFiscalYearStartDate()
					.isAfter(donationCommand.getDonationSummary().getDonationDate());
			setFormAsReadOnly(model, isReadOnly);
		}
	}

}
