package gov.va.vss.web.letterTemplate;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;

import gov.va.shared.util.WebUtil;
import gov.va.vss.model.LetterTemplate;
import gov.va.vss.model.LetterType;
import gov.va.vss.model.lookup.VtkInstitution;
import gov.va.vss.service.validation.ServiceValidationException;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.ValidationException;
import gov.va.vss.web.breadcrumbs.Breadcrumb;

@Controller
@SessionAttributes(value = { AbstractAppController.DEFAULT_COMMAND_NAME })
public class LetterTemplateController extends AbstractAppController {

	@Autowired
	private LetterTemplateValidator validator;

	@RequestMapping("/letterTemplateEdit.htm")
	@Breadcrumb("Edit Letter Template")
	// @PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String editTemplate(@RequestParam(required = false) Long id, ModelMap model, HttpServletRequest request) {
		LetterTemplateCommand command = new LetterTemplateCommand();

		if (id != null) {
			LetterTemplate t = letterTemplateDAO.findRequiredByPrimaryKey(id);
			command = new LetterTemplateCommand(t);
		}

		model.addAttribute(DEFAULT_COMMAND_NAME, command);
		createReferenceData(model);
		return "letterTemplateEdit";
	}

	private void createReferenceData(ModelMap model) {
		WebUtil.addEnumToModel(LetterType.class, model);
	}

	@RequestMapping("/letterTemplateSubmit.htm")
	// @PreAuthorize("hasAuthority('" + Permission.DONATION_CREATE + "')")
	public String submitLetterTemplate(@ModelAttribute(DEFAULT_COMMAND_NAME) LetterTemplateCommand command,
			BindingResult result, SessionStatus status, ModelMap model, HttpServletRequest request)
					throws ValidationException {
		LetterTemplate letterTemplate = command.getLetterTemplate();

		/* If we don't have a letter template yet in the command */
		if (letterTemplate == null) {
			LetterType type = command.getType();
			VtkInstitution i = command.getFacility();

			/*
			 * Check to see if we clicked "Search" after choosing both the type
			 * and facility
			 */
			if (type != null && i != null) {
				letterTemplate = letterTemplateDAO.findByCriteria(type, i.getId());

				/*
				 * If we did, but don't have a letter template for this
				 * facility/type yet, create one with the default text from the
				 * CO for this type. CPB
				 */
				if (letterTemplate == null) {
					LetterTemplate defaultTemplate = letterTemplateDAO.findRequiredDefault(type);

					letterTemplate = new LetterTemplate();
					letterTemplate.setFacility(i);
					letterTemplate.setType(type);
					letterTemplate.setBody(defaultTemplate.getBody());
					command.setLetterTemplate(letterTemplate);
					createReferenceData(model);
					return "letterTemplateEdit";
				}
			} else {
				/*
				 * If we clicked search but didn't specify both the type and
				 * facility, send user back to UI with an error - CPB
				 */
				if (type == null)
					result.rejectValue("type", null, "Please select a letter type.");
				if (i == null)
					result.rejectValue("facility", null, "Please select a facility.");

				createReferenceData(model);
				return "letterTemplateEdit";
			}
		}

		/*
		 * User clicked submit and we have a letter template in the command,
		 * which may or may not be persistent. Save the body and letter template.
		 */
		boolean isEdit = letterTemplate.isPersistent();

		/* Validation step (JSR303, other custom logic in the validator) */
		validator.validate(command, result, false, "letterTemplate");
		boolean hasErrors = result.hasErrors();

		if (!hasErrors) {
			try {
				letterTemplate = letterTemplateService.saveOrUpdate(letterTemplate);
				userNotifier.notifyUserOnceWithMessage(request,
						getMessage(isEdit ? "letterTemplate.update.success" : "letterTemplate.create.success"));
			} catch (ServiceValidationException e) {
				webValidationService.handle(e, result);
				hasErrors = true;
			}
		}

		if (hasErrors) {
			createReferenceData(model);
			return "letterTemplateEdit";
		} else {
			status.setComplete();
			return "redirect:/letterTemplateEdit.htm?id=" + letterTemplate.getId();
		}
	}

}
