package gov.va.vss.web.reports;

import static gov.va.vss.persistence.dao.BenefitingServiceDAO.BENEFITING_SERVICE_PREFIX_GAMES;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.AppUserFacility;
import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.util.PersistenceUtil;
import gov.va.vss.model.BenefitingService;
import gov.va.vss.model.BenefitingService.BenefitingServiceView;
import gov.va.vss.model.Organization;
import gov.va.vss.model.Organization.OrganizationView;
import gov.va.vss.model.Volunteer;
import gov.va.vss.model.Volunteer.VolunteerView;
import gov.va.vss.model.lookup.VtkAdministrativeUnit;
import gov.va.vss.model.views.CombinedInstitution;
import gov.va.vss.web.AbstractAppController;

@Controller
public class ReportParametersController extends AbstractAppController {

	@RequestMapping(value = "/assignedVAFacilities", method = RequestMethod.GET)
	public @ResponseBody Collection<VAFacility> getAssignedVAFacilities() {
		SortedSet<VAFacility> allLinkedFacilities = vtkInstitutionDAO.findVAFacilitiesWithLinkToVtkInstitution();
		if (getCurrentUser().isNationalAdmin())
			return allLinkedFacilities;

		List<VAFacility> assignedFacilities = new ArrayList<VAFacility>();
		for (AppUserFacility auFac : getCurrentUser().getFacilities()) {
			VAFacility facility = auFac.getFacility();
			if (allLinkedFacilities.contains(facility))
				assignedFacilities.add(facility);
		}

		return assignedFacilities;
	}

	@RequestMapping(value = "/assignedVtkInstitutions", method = RequestMethod.GET)
	public @ResponseBody Collection<CombinedInstitution> getAssignedVtkInstitutions(
			@RequestParam boolean showAllFacilities) {
		SortedSet<CombinedInstitution> linkedFacilities = combinedInstitutionDAO.findWithLinkToVAFacility();
		if (getCurrentUser().isNationalAdmin() || showAllFacilities)
			return linkedFacilities;

		Set<Long> vaFacilityIds = PersistenceUtil.translateObjectsToIds(getCurrentUser().getBasicFacilities());

		for (Iterator<CombinedInstitution> it = linkedFacilities.iterator(); it.hasNext();) {
			CombinedInstitution vi = it.next();
			if (!vaFacilityIds.contains(vi.getVaFacility().getId()))
				it.remove();
		}

		return linkedFacilities;
	}

	/**
	 * Powers the jQuery organization selection table - CPB
	 * 
	 * @param draw
	 * @param start
	 * @param length
	 * @param searchValue
	 * @param searchIsRegex
	 * @param allParams
	 * @return
	 */
	@RequestMapping("/organization/quickSearch")
	public @ResponseBody Map<String, Object> findOrganizationsByName(@RequestParam int draw, @RequestParam int start,
			@RequestParam int length, @RequestParam(name = "search[value]") String searchValue,
			@RequestParam(name = "search[regex]") boolean searchIsRegex) {
		Map<String, Object> resultMap = new HashMap<>();

		SortedSet<OrgQuickSearchResult> results = null;
		if (StringUtils.isNotBlank(searchValue)) {
			results = new TreeSet<>(organizationDAO.quickSearch(searchValue, length));
		} else {
			results = new TreeSet<>();
		}
		resultMap.put("data", results);
		resultMap.put("draw", draw);
		return resultMap;
	}

	@RequestMapping("/organization")
	@JsonView(OrganizationView.Basic.class)
	public @ResponseBody SortedSet<Organization> findOrganizations(@RequestParam boolean includeLocal,
			@RequestParam boolean includeNational, @RequestParam(required = false) Long stationId) {
		return new TreeSet<>(organizationDAO.findByCriteria(null, includeNational, includeLocal, null,
				stationId != null ? Arrays.asList(stationId) : null, true, null, null));
	}

	@RequestMapping("/benefitingService")
	@JsonView(BenefitingServiceView.Basic.class)
	public @ResponseBody SortedSet<BenefitingService> findBenefitingServices(@RequestParam boolean includeLocal,
			@RequestParam boolean includeNational, @RequestParam boolean excludeGames, @RequestParam long stationId) {
		return new TreeSet<>(benefitingServiceDAO.findByCriteria(null, null, null, null, includeNational, includeLocal,
				null, Arrays.asList(stationId), excludeGames ? Arrays.asList(BENEFITING_SERVICE_PREFIX_GAMES) : null));
	}

	@RequestMapping(value = "/assignedVISNs", method = RequestMethod.GET)
	public @ResponseBody Collection<VtkAdministrativeUnit> getAssignedVISNs() {
		// if (getCurrentUser().isNationalAdmin()) {
		return vtkAdministrativeUnitDAO.findAllSorted();
		// } else {
		// List<VAFacility> assignedFacilities = new ArrayList<VAFacility>();
		// for (AppUserFacility auFac : getCurrentUser().getFacilities()) {
		// assignedFacilities.add(auFac.getFacility());
		// }
		// return assignedFacilities;
		// }
	}

	@RequestMapping(value = "/volunteersAtFacility", method = RequestMethod.GET)
	@JsonView(VolunteerView.Basic.class)
	public @ResponseBody List<Volunteer> getVolunteersAtFacility(@RequestParam long stationId) {
		List<Volunteer> volunteerOrgs = volunteerDAO.findByCriteria(null, null, null, null, null, null, null, null,
				null, null, null, true, null, null, Arrays.asList(stationId));
		return volunteerOrgs;
	}

	@RequestMapping(value = "/volunteerZipCodesAtFacility", method = RequestMethod.GET)
	public @ResponseBody List<String> getVolunteerZipCodesAtFacility(@RequestParam long stationId) {
		List<String> zipCodes = volunteerDAO.findZipCodesAtFacilities(Arrays.asList(stationId), true, true);
		return zipCodes;

	}

	@RequestMapping(value = "/monthsOfYear", method = RequestMethod.GET)
	public @ResponseBody Set<MultiSelectGenericCommand<Integer, String>> getMonthsOfYear() {

		Set<MultiSelectGenericCommand<Integer, String>> months = new TreeSet<MultiSelectGenericCommand<Integer, String>>();
		Calendar cal = Calendar.getInstance();
		Map<String, Integer> monthMap = cal.getDisplayNames(Calendar.MONTH, Calendar.LONG, Locale.ENGLISH);
		for (Map.Entry<String, Integer> entry : monthMap.entrySet()) {
			String monthName = entry.getKey();
			Integer monthInt = entry.getValue() + 1;
			months.add(new MultiSelectGenericCommand<Integer, String>(monthInt, monthName));
		}
		return months;
	}

}