package gov.va.vss.web.volunteer;

import java.time.LocalDate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;

import gov.va.vss.model.Volunteer;
import gov.va.vss.model.lookup.VtkState;
import gov.va.vss.util.DateUtil;
import gov.va.vss.web.AbstractAppValidator;

@Component
public class VolunteerValidator extends AbstractAppValidator<VolunteerCommand> {

	@Override
	public void doExtraValidations(VolunteerCommand command, Errors errors) {
		Volunteer volunteer = command.getVolunteer();

		// for example: CPB

		// if (volunteer.getCurrentInactivationDate() != null
		// && DateTimeComparator.getInstance().compare(
		// volunteer.getCurrentActivationDate(),
		// volunteer.getCurrentInactivationDate()) > 0) {
		// errors.rejectValue("currentInactivationDate",
		// "currentInactivationDate.errors.invalidInactivationDate",
		// "The inactivation date must be later than the activation date");
		// }

		// or:

		// ValidationUtils.rejectIfEmpty(errors, field, errorCode, errorArgs,
		// defaultMessage);

		// (note that @NotBlank is Hibernate-specific API but achieves the same
		// as an annotation on the field)

		VtkState state = volunteer.getState();
		if (state == null) {
			errors.rejectValue("state", "volunteer.error.requiredState");
		}

		Integer mealsEligible = volunteer.getMealsEligible();
		if (mealsEligible != null && mealsEligible == 0 && StringUtils.isBlank(volunteer.getMealRemarks())) {
			errors.rejectValue("volunteer.mealRemarks", "volunteer.error.requiredMealRemarks");
		}

		if (volunteer.isTerminatedWithCause()) {
			if (StringUtils.isBlank(volunteer.getTerminationRemarks()))
				errors.rejectValue("volunteer.terminationRemarks", "volunteer.error.requiredTerminationRemarks");
			if (volunteer.getTerminationDate() == null)
				errors.rejectValue("volunteer.terminationDate", "volunteer.error.requiredTerminationDate");
		}

		if (volunteer.getTerminationDate() != null) {
			if (volunteer.getTerminationDate().isBefore(volunteer.getEntryDate())) {
				errors.rejectValue("volunteer.terminationDate", "volunteer.error.terminationDateBeforeEntryDate");
			} else {
				LocalDate mostRecentWorkEntryDate = command.getTimeSummary().getMostRecentWorkEntryDate();
				if (mostRecentWorkEntryDate != null
						&& !volunteer.getTerminationDate().isAfter(mostRecentWorkEntryDate)) {
					errors.rejectValue("volunteer.terminationDate",
							"volunteer.error.terminationDateBeforeMostRecentWorkEntryDate",
							new Object[] { mostRecentWorkEntryDate.format(DateUtil.DATE_ONLY_FORMAT) }, "");
				}
			}
		}

	}

}
