<%@ include file="../shared/inc_header.jsp"%>

<%--
	Expects the following params:
	
	- A "uniqueVolunteerSearchPopupId" param which should be a string of letters that
		uniquely identifies this volunteerSearchPopup among other volunteerSearchPopups on the same page
	- A "mode" param set to either "search", "add" or "duplicateCheck":
			If "search", search fields are shown and the callback is called as normal when the person clicks a result
			If "add", an "Add Volunteer" button will appear after the user has searched once, and the scope will be
				hardcoded to national.
			If "duplicateCheck", an array of volunteer objects will be provided directly to the popup via the
				popupVolunteerSearch method.
				
		If "mode" is unspecified, it defaults to "search".
		
	- A "callbackMethod" param which specifies the name of a JavaScript method defined by the
		-including- page which the volunteer popup will activate when the user selects a volunteer. The
		required method signature looks like:
		function someMethodWithUniqueName(volunteerObj)
		
		The volunteerObj parameter sent to the callback method is a javascript object containing the
		properties
		{
			id : <the ID of the volunteer selected>,
			[... other attributes of the Volunteer class marked for JsonProperty]
		}
		
	To activate this popup, call the method popupVolunteerSearch(uid) and pass in the same
	uniqueVolunteerSearchPopupId String above.
--%>

<c:set var="uid">
	<c:out value="${param.uniqueVolunteerSearchPopupId}" />
</c:set>

<script type="text/javascript">
	$(function() {
		initVolunteerSearchPopup({
			uid : "${uid}",
			mode : "<c:out value="${param.mode}" default="search" />",
			maxResults : <fmt:message key="volunteerSearch.maxResults" />,
			callbackMethod : <c:out value="${param.resultCallbackMethod}" default="null" />,
			addButtonCallbackMethod : <c:out value="${param.addButtonCallbackMethod}" default="null" />,
			showDisclaimer : <c:out value="${not empty param.disclaimerText}" />
		})
	})
</script>

<div id="volunteerSearchDialog${uid}" style="display: none"
	title="Search for Volunteer">
	<div align="center" id="volSearchDisclaimer${uid}"
		style="display: none; font-weight: bold">
		<c:out value="${param.disclaimerText}" />
		<p />
	</div>

	<table align="center">
		<tr valign="top" class="volunteerSearchFields${uid}">
			<td>
				<table>
					<tr>
						<td align="right"><label for='volunteerSearchName${uid}'>Name:</label></td>
						<td><input type="text" id="volunteerSearchName${uid}"
							value="" title="Type a name in the format Last[, First Middle]"
							placeholder="Last[, First Middle]" /></td>
					</tr>
					<tr>
						<td align="right"><label for='volunteerSearchEmail${uid}'>Email:</label></td>
						<td><input type="text" id="volunteerSearchEmail${uid}"
							value="" title="Type all or part of an email" /></td>

					</tr>
					<tr>
						<td align="right"><label for='volunteerSearchName${uid}'>Identifying
								Code:</label></td>
						<td><input type="text" id="volunteerSearchCode${uid}"
							value="" title="Type an identifying code" /></td>
					</tr>
					<tr>
						<td align="right"><label for='volunteerSearchName${uid}'>Date
								of Birth:</label></td>
						<td><input id="volunteerSearchDOB${uid}" /></td>
					</tr>
				</table>
			</td>
			<td>
				<table>
					<tr>
						<td align="right"><label for='volunteerSearchScope${uid}'>Search:</label></td>
						<td><input type="radio" id="volunteerSearchScopeLocal${uid}"
							name="volunteerSearchScope${uid}" value="Local" checked="checked">Within
							Facility <input type="radio"
							id="volunteerSearchScopeNational${uid}"
							name="volunteerSearchScope${uid}" value="National"
							style="margin-left: 50px">All Facilities</td>
					</tr>
					<tr>
						<td></td>
						<td><div id="volunteerSearchFacilityWrapper${uid}">
								<select id="volunteerSearchScopeFacilityId${uid}">
									<c:if test="${not empty vtkInstitutionContext}">
										<option value="${vtkInstitutionContext.id}"
											selected="selected"><c:out
												value="${vtkInstitutionContext.displayName}" /></option>
									</c:if>
								</select>
							</div> <img src="${imgHome}/spacer.gif" height="1" width="400" /></td>
					</tr>
					<tr>
						<td align="right"><input type="checkbox"
							id="volunteerSearchIncludeInactive${uid}" value="true" /></td>
						<td>Include Inactive Volunteers</td>
					</tr>

				</table>
			</td>
		</tr>
		<tr align="center">
			<td colspan="2"><a style="margin-left: 20px"
				class="buttonAnchor volunteerSearchLink${uid} volunteerSearchFields${uid}"
				tabIndex="0">Search</a> <a style="display: none"
				class="buttonAnchor volunteerAddLink${uid}" tabIndex="1">Add New
					Volunteer</a></td>
		</tr>
	</table>
	<div id="volunteerSearchNoResults${uid}" style="display: none">
		<table align="center" cellpadding="10">
			<tr>
				<td>Sorry, no volunteers were found that matched the specified
					criteria.</td>
			</tr>
		</table>
	</div>
	<div id="volunteerSearchMaxResults${uid}"
		style="display: none; padding: 10px" align="center">Your search
		exceeded the maximum results. Please enter additional search criteria.
	</div>
	<div id="volunteerSearchResultsTable${uid}" style="display: none">
		<table id="volunteerSearchResultsList${uid}" class="stripe"
			summary="List of Volunteers">
			<thead>
				<tr id="volunteerSearch${uid}FilterRow">
					<td class="noborder">Filters:</td>
					<td class="noborder" title="Filter by Primary Facility"></td>
					<td class="noborder"></td>
					<td class="noborder" title="Filter by Gender"></td>
					<td class="noborder" title="Filter by Status"></td>
					<td class="noborder"></td>
					<%--
					<td class="noborder"></td>
					 --%>
				</tr>
				<tr>
					<th width="40%" id="nameHeaderCol${uid}">Name</th>
					<th width="10%" class="select-filter">Primary Facility</th>
					<th width="10%">Date of Birth</th>
					<th width="5%" class="select-filter">Gender</th>
					<th width="10%" class="select-filter" nowrap>Status</th>
					<th width="20%">Contact Info</th>
					<%--
					<th width="5%" nowrap>Identifying<br>Code</th>
					--%>
				</tr>
			</thead>
		</table>
	</div>
</div>
