package gov.va.vss.service.batch;

import java.io.Serializable;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import gov.va.shared.util.SecurityUtil;
import gov.va.vss.service.AbstractJMSTest;
import gov.va.vss.service.impl.BatchProcessServiceImpl;
import gov.va.vss.service.messaging.listeners.BatchProcessListener;
import gov.va.vss.util.DateUtil;
import gov.va.vss.web.batch.BatchController;

public class TestBatchProcessService extends AbstractJMSTest {

	@Autowired
	private BatchProcessService svc;
	@Autowired
	@Qualifier("jmsTemplateBatchProcessEvents")
	private JmsTemplate jmsTemplate;
	@Autowired
	private BatchProcessListener listener;

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Before
	public void createTeams() {
		new TransactionTemplate(tm).execute(new TransactionCallback() {
			public Object doInTransaction(TransactionStatus ts) {
				/* build local test fields here */
				return null;
			}
		});
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@After
	public void deleteTeams() {
		new TransactionTemplate(tm).execute(new TransactionCallback() {
			public Object doInTransaction(TransactionStatus status) {
				/* Delete any objects created in the test here */
				return null;
			}
		});
	}

	@Test
	public void testPerformBulkPatientOperations() throws Exception {
		String requesterActiveDirectoryName = SecurityUtil.getCurrentUserName();

		List<String> objectIDs = Arrays.asList("1", "2");

		Map<String, Serializable> opParamsMap = getParamsMap(ZonedDateTime.now(), StringUtils.join(objectIDs, ","));
		svc.performBulkOperations(opParamsMap, requesterActiveDirectoryName);

		go(listener);

		// Perform verification that bulk operation data changes were made here

		// e.g.
		// List<PatientAssignment> assignmentsForCriteria =
		// patientAssignmentDAO.findAssignmentsForCriteria(
		// DateCriteria.activeAsOfDate(ZonedDateTime.now()), team.getId(), null,
		// null, null, null, null, null,
		// null, null);
		//
		// Set<Long> newPatientIDs = new HashSet<>();
		// for (PatientAssignment p : assignmentsForCriteria)
		// newPatientIDs.add(p.getPatient().getId());
		//
		// Assert.assertEquals(newPatientIDs, patientIDs);
	}

	private Map<String, Serializable> getParamsMap(ZonedDateTime effectiveDate, String idsToProcessList) {
		String prefix = BatchProcessServiceImpl.BATCH_ACTIONS_REQUEST_PARAM_PREFIX
				+ BatchProcessServiceImpl.BATCH_ACTIONS_ACTION_PARAMS_PREFIX;

		Map<String, Serializable> results = new HashMap<>();
		results.put(BatchController.PARAM_OBJECT_IDS_SELECTED, idsToProcessList);
		if (effectiveDate != null)
			results.put(prefix + "effectiveDate", DateUtil.TWO_DIGIT_MILITARY_DATE_TIME_FORMAT.format(effectiveDate));

		return results;
	}

	@Override
	protected JmsTemplate getJmsTemplate() {
		return jmsTemplate;
	}

}
