package gov.va.shared.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.AttributeOverride;
import javax.persistence.AttributeOverrides;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.hibernate.annotations.BatchSize;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;
import org.springframework.security.core.GrantedAuthority;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.Permission.PermissionType;
import gov.va.shared.model.lookup.AbstractLookup;
import gov.va.shared.model.lookup.LookupType;

@Entity
@Table(name = "PERMISSION", schema = "CORE")
@AttributeOverrides({ @AttributeOverride(name = "id", column = @Column(name = "PERMISSION_ID")) })
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = NONE, isGetterVisibility = NONE)
@Immutable
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
public class Permission extends AbstractLookup<Permission, PermissionType> implements GrantedAuthority {
	private static final long serialVersionUID = -3175355751589521985L;

	// --------------------------------------------- Static Fields

	public static final String DONATION_CREATE = "Create Donation";
	public static final String DONATION_READ = "Read Donation";
	public static final String FACILITY_CREATE = "Create Facility";
	public static final String FACILITY_EDIT_CURRENT = "Edit Current Facility";
	public static final String FACILITY_EDIT_ALL = "Edit All Facilities";
	public static final String LOGIN_APPLICATION = "Login Staff Application";
	public static final String PERM_CODE_SERVICE_CREATE = "Create Voluntary Services";
	public static final String PERM_CODE_SERVICE_READ = "Read Voluntary Services";
	public static final String ORG_CODE_NATIONAL_CREATE = "Create National Organization Codes";
	public static final String ORG_CODE_NATIONAL_READ = "Read National Organization Codes";
	public static final String ORG_CODE_LOCAL_CREATE = "Create Local Organization Codes";
	public static final String ORG_CODE_LOCAL_READ = "Read Local Organization Codes";
	public static final String USER_MANAGER = "Manage Users";
	public static final String VOLUNTEER_CREATE = "Create Volunteer";
	public static final String VOLUNTEER_READ = "Read Volunteer";
	public static final String VOL_SVC_STAFF_DELETE = "Delete Voluntary Services Staff";
	public static final String VOLUNTEER_UNTERMINATE_BY_CAUSE = "Unterminate Volunteer By Cause";

	// --------------------------------------------- Fields

	private Set<RolePermission> roles;

	// --------------------------------------------- Business Methods

	public void initializeAll() {
		initialize(getRoles());
	}

	// --------------------------------------------- Accessor Methods

	@Transient
	@Override
	public String getAuthority() {
		return getName();
	}

	@OneToMany(mappedBy = "permission", fetch = FetchType.LAZY, cascade = CascadeType.ALL)
	@BatchSize(size = 500)
	public Set<RolePermission> getRoles() {
		if (roles == null)
			roles = new HashSet<>();
		return roles;
	}

	public void setRoles(Set<RolePermission> roles) {
		this.roles = roles;
	}

	public static enum PermissionType implements LookupType {
		LOGIN_APPLICATION(Permission.LOGIN_APPLICATION, 10), //
		ORG_CODE_NATIONAL_CREATE(Permission.ORG_CODE_NATIONAL_CREATE, 20), //
		ORG_CODE_NATIONAL_READ(Permission.ORG_CODE_NATIONAL_READ, 30), //
		ORG_CODE_LOCAL_CREATE(Permission.ORG_CODE_LOCAL_CREATE, 40), //
		ORG_CODE_LOCAL_READ(Permission.ORG_CODE_LOCAL_READ, 50), //
		PERM_CODE_SERVICE_CREATE(Permission.PERM_CODE_SERVICE_CREATE, 60), //
		PERM_CODE_SERVICE_READ(Permission.PERM_CODE_SERVICE_READ, 70), //
		USER_MANAGER(Permission.USER_MANAGER, 80), //
		VOLUNTEER_CREATE(Permission.VOLUNTEER_CREATE, 90), //
		VOLUNTEER_READ(Permission.VOLUNTEER_READ, 100), //
		VOL_SVC_STAFF_DELETE(Permission.VOL_SVC_STAFF_DELETE, 110), //
		DONATION_CREATE(Permission.DONATION_CREATE, 120), //
		DONATION_READ(Permission.DONATION_READ, 130), //
		VOLUNTEER_UNTERMINATE_BY_CAUSE(Permission.VOLUNTEER_UNTERMINATE_BY_CAUSE, 140),
		FACILITY_CREATE(Permission.FACILITY_CREATE, 150), //
		FACILITY_EDIT_CURRENT(Permission.FACILITY_EDIT_CURRENT, 160), //
		FACILITY_EDIT_ALL(Permission.FACILITY_EDIT_ALL, 170), //
		;

		private String name;
		private long id;

		private PermissionType(String name, long id) {
			this.name = name;
			this.id = id;
		}

		public String getName() {
			return name;
		}

		@Override
		public long getId() {
			return id;
		}
	}

}
