package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.vss.model.BenefitingService.BenefitingServiceView;
import gov.va.vss.model.FacilityAssignment.FacilityAssignmentView;
import gov.va.vss.model.VolunteerFacilityAssignment.VolunteerFacilityAssignmentView.SearchForVolunteerFacilityAssignments;
import gov.va.vss.model.VolunteerFacilityAssignment.VolunteerFacilityAssignmentView.SearchForVolunteers;

@Entity
@Table(name = "VssVolunteerAssignments", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = NONE, isGetterVisibility = NONE)
public class VolunteerFacilityAssignment extends AbstractAuditedVersionedPersistent<VolunteerFacilityAssignment> {
	private static final long serialVersionUID = 6904844123870655771L;

	/**
	 * See @JsonView documentation - CPB
	 */
	public static class VolunteerFacilityAssignmentView {
		public interface Basic {
		}

		public interface SearchForVolunteerFacilityAssignments
				extends Basic, FacilityAssignmentView.SearchForFacilityAssignments, BenefitingServiceView.Basic {
		}

		public interface SearchForVolunteers extends Basic {
		}
	}

	public static final class CompareByFacilityAssignment implements Comparator<VolunteerFacilityAssignment> {
		@Override
		public int compare(VolunteerFacilityAssignment o1, VolunteerFacilityAssignment o2) {
			return new CompareToBuilder().append(o1.getFacilityAssignment(), o2.getFacilityAssignment()).toComparison();
		}
	}

	// -------------------------------------- Fields

	private Volunteer volunteer;
	private FacilityAssignment facilityAssignment;

	private boolean inactive;

	private List<WorkEntry> workEntries;

	// -------------------------------------- Constructors

	public VolunteerFacilityAssignment() {
	}

	public VolunteerFacilityAssignment(Volunteer volunteer, FacilityAssignment facilityAssignment) {
		this.volunteer = volunteer;
		this.facilityAssignment = facilityAssignment;
	}

	// -------------------------------------- Business Methods

	@Transient
	@JsonView(VolunteerFacilityAssignmentView.Basic.class)
	public boolean isActive() {
		return !inactive;
	}

	// -------------------------------------- Common Methods

	@Override
	protected boolean requiredEquals(VolunteerFacilityAssignment oo) {
		return new EqualsBuilder().append(nullSafeGetId(getVolunteer()), oo.getVolunteer())
				.append(nullSafeGetId(getFacilityAssignment()), nullSafeGetId(oo.getFacilityAssignment())).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(nullSafeGetId(getVolunteer()))
				.append(nullSafeGetId(getFacilityAssignment())).toHashCode();
	}

	// -------------------------------------- Accessor Methods

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssVolunteersFK", nullable = false)
	@JsonView(SearchForVolunteers.class)
	public Volunteer getVolunteer() {
		return volunteer;
	}

	public void setVolunteer(Volunteer volunteer) {
		this.volunteer = volunteer;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "VssFacilityAssignmentsFK", nullable = false)
	@JsonView(SearchForVolunteerFacilityAssignments.class)
	public FacilityAssignment getFacilityAssignment() {
		return facilityAssignment;
	}

	public void setFacilityAssignment(FacilityAssignment facilityAssignment) {
		this.facilityAssignment = facilityAssignment;
	}

	@OneToMany(mappedBy = "volunteerAssignment", fetch = FetchType.LAZY)
	@BatchSize(size = 500)
	@JsonIgnore
	public List<WorkEntry> getWorkEntries() {
		if (workEntries == null)
			workEntries = new ArrayList<>();
		return workEntries;
	}

	public void setWorkEntries(List<WorkEntry> workEntries) {
		this.workEntries = workEntries;
	}

	@Column(name = "IsInactive", nullable = false)
	@JsonIgnore
	public boolean isInactive() {
		return inactive;
	}

	public void setInactive(boolean inactive) {
		this.inactive = inactive;
	}

}
