package gov.va.vss.model;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import java.time.ZonedDateTime;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.hibernate.annotations.BatchSize;
import org.springframework.format.annotation.DateTimeFormat;

import com.fasterxml.jackson.annotation.JsonAutoDetect;

import gov.va.shared.model.AppUser;
import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.shared.util.StringUtil;
import gov.va.vss.model.facility.Facility;
import gov.va.vss.util.DateUtil;

@Entity
@Table(name = "VtkUsers", schema = "dbo")
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class VtkUser extends AbstractAuditedVersionedPersistent<VtkUser>
		implements BasicUserFields, Comparable<VtkUser> {
	private static final long serialVersionUID = 8168397789547901768L;

	// ------------------------------------ Fields

	private String username;
	private String firstName;
	private String middleName;
	private String lastName;
	private String nameSuffix;
	private String nickName;
	private String sex;
	private String grade;
	// private String lowercaseNameForDisplay;
	// private String uppercaseNameForDisplay;
	// private String fullNameForReports;
	private String emailAddress;
	private boolean inactive = false;
	private String inactivator;
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	private ZonedDateTime inactivationDate;
	@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY)
	private ZonedDateTime lastAccess;
	private Facility primaryInstitution;
	private Facility defaultInstitution;
	private AppUser appUser;
	private VtkStaffTitle vtkStaffTitle;

	// ------------------------------------ Constructors

	public VtkUser() {
	}

	public VtkUser(AppUser u) {
		setFromAppUser(u);
	}

	// ------------------------------------ Business Methods

	@Override
	@Transient
	public String getFullDisplayName() {
		return appUser != null
				? StringUtil.getDisplayName(true, appUser.getFirstName(), appUser.getMiddleName(),
						appUser.getLastName(), nameSuffix)
				: StringUtil.getDisplayName(true, firstName, middleName, lastName, nameSuffix);
	}

	public void setFromAppUser(AppUser u) {
		if (u != null) {
			setAppUser(u);
			setUsername(u.getUsername());
			setFirstName(u.getFirstName());
			setMiddleName(u.getMiddleName());
			setLastName(u.getLastName());
			setEmailAddress(u.getEmail());
		}
	}

	@Override
	@Transient
	public String getPhone() {
		return "";
	}

	@Override
	@Transient
	public String getTitle() {
		return "";
	}

	@Override
	@Transient
	public String getOffice() {
		return "";
	}

	@Override
	@Transient
	public String getDepartment() {
		return "";
	}

	// ------------------------------------ Common Methods

	@Override
	protected boolean requiredEquals(VtkUser oo) {
		return new EqualsBuilder().append(getUsername(), oo.getUsername())
				.append(getInactivationDate(), oo.getInactivationDate()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getUsername()).append(getInactivationDate()).toHashCode();
	}

	public int compareTo(VtkUser u) {
		if (equals(u))
			return 0;
		return new CompareToBuilder().append(getUsername(), u.getUsername())
				.append(getInactivationDate(), u.getInactivationDate()).toComparison() > 0 ? 1 : -1;
	}

	// ------------------------------------ Accessor Methods

	@Override
	@Column(name = "NTUSERNAME")
	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	@Override
	@Column(name = "FIRSTNAME", nullable = false)
	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	@Override
	@Column(name = "MIDDLENAME")
	public String getMiddleName() {
		return middleName;
	}

	public void setMiddleName(String middleName) {
		this.middleName = middleName;
	}

	@Override
	@Column(name = "LASTNAME", nullable = false)
	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	@Column(name = "NAMESUFFIX")
	public String getNameSuffix() {
		return nameSuffix;
	}

	public void setNameSuffix(String nameSuffix) {
		this.nameSuffix = nameSuffix;
	}

	@Column(name = "NICKNAME")
	public String get    Name() {
		return nickName;
	}

	public void set    Name(String nickName) {
		this.nickName = nickName;
	}

	@Column(name = "SEX")
	public String getSex() {
		return sex;
	}

	public void setSex(String sex) {
		this.sex = sex;
	}

	@Column(name = "GRADE")
	public String getGrade() {
		return grade;
	}

	public void setGrade(String grade) {
		this.grade = grade;
	}

	@Override
	@Column(name = "EMAILADDRESS")
	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	@Column(name = "INACTIVE")
	public boolean isInactive() {
		return inactive;
	}

	public void setInactive(boolean inactive) {
		this.inactive = inactive;
	}

	@Column(name = "INACTIVATOR")
	public String getInactivator() {
		return inactivator;
	}

	public void setInactivator(String inactivator) {
		this.inactivator = inactivator;
	}

	@Column(name = "DateOfInactivation")
	public ZonedDateTime getInactivationDate() {
		return inactivationDate;
	}

	public void setInactivationDate(ZonedDateTime inactivationDate) {
		this.inactivationDate = inactivationDate;
	}

	@Column(name = "LASTACCESS")
	public ZonedDateTime getLastAccess() {
		return lastAccess;
	}

	public void setLastAccess(ZonedDateTime lastAccess) {
		this.lastAccess = lastAccess;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "PRIMARYINSTITUTIONFK", nullable = false)
	@BatchSize(size = 500)
	public Facility getPrimaryInstitution() {
		return primaryInstitution;
	}

	public void setPrimaryInstitution(Facility primaryInstitution) {
		this.primaryInstitution = primaryInstitution;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "DEFAULTINSTITUTIONFK")
	@BatchSize(size = 500)
	public Facility getDefaultInstitution() {
		return defaultInstitution;
	}

	public void setDefaultInstitution(Facility defaultInstitution) {
		this.defaultInstitution = defaultInstitution;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "APP_USER_IDFK")
	@BatchSize(size = 500)
	public AppUser getAppUser() {
		return appUser;
	}

	public void setAppUser(AppUser appUser) {
		this.appUser = appUser;
	}

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "JOBTITLEFK")
	@BatchSize(size = 500)
	public VtkStaffTitle getVtkStaffTitle() {
		return vtkStaffTitle;
	}

	public void setVtkStaffTitle(VtkStaffTitle vtkStaffTitle) {
		this.vtkStaffTitle = vtkStaffTitle;
	}

}
