package gov.va.vss.model.lookup;

import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.NONE;
import static com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility.PUBLIC_ONLY;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.text.WordUtils;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.Immutable;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.core.AbstractAuditedVersionedPersistent;
import gov.va.shared.model.lookup.LookupType;

/* Legacy "Ver" version column ignored - CPB */
@Entity
@Immutable
@Table(name = "VtkState", schema = "dbo")
@Cache(usage = CacheConcurrencyStrategy.READ_ONLY)
@JsonAutoDetect(fieldVisibility = NONE, getterVisibility = PUBLIC_ONLY, isGetterVisibility = PUBLIC_ONLY)
public class VtkState extends AbstractAuditedVersionedPersistent<VtkState> implements Comparable<VtkState> {
	private static final long serialVersionUID = -8678395783438462990L;

	/**
	 * See @JsonView documentation - CPB
	 */
	public static class VtkStateView {
		public interface Basic {
		}

		public interface Extended extends Basic {
		}
	}

	private String abbreviation;
	private String vaCode;
	private String name;

	public VtkState() {
	}

	public VtkState(gov.va.shared.model.lookup.sds.State state) {
		this.abbreviation = state.getPostalName();
		this.vaCode = state.getFipsCode();
		this.name = state.getName();
	}

	@Transient
	@JsonView(VtkStateView.Basic.class)
	public String getDisplayName() {
		return WordUtils.capitalizeFully(getName());
	}

	@Override
	protected boolean requiredEquals(VtkState oo) {
		return new EqualsBuilder().append(getVaCode(), oo.getVaCode()).isEquals();
	}

	@Override
	protected int requiredHashCode() {
		return new HashCodeBuilder().append(getVaCode()).toHashCode();
	}

	@Override
	public int compareTo(VtkState o) {
		if (equals(o))
			return 0;

		return new CompareToBuilder().append(StringUtils.lowerCase(name), StringUtils.lowerCase(o.getName()))
				.toComparison() > 0 ? 1 : -1;
	}

	@Column(name = "StateName", length = 50)
	@JsonIgnore
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Column(name = "Abbreviation", length = 2)
	@JsonView(VtkStateView.Basic.class)
	public String getAbbreviation() {
		return abbreviation;
	}

	public void setAbbreviation(String abbreviation) {
		this.abbreviation = abbreviation;
	}

	@Column(name = "VACode", length = 2)
	@JsonView(VtkStateView.Extended.class)
	public String getVaCode() {
		return vaCode;
	}

	public void setVaCode(String vaCode) {
		this.vaCode = vaCode;
	}

	public static enum VtkStateType implements LookupType {
		COLORADO(13);

		private long id;

		private VtkStateType(long id) {
			this.id = id;
		}

		public long getId() {
			return id;
		}

	}

}
