package gov.va.vss.persistence.dao.jobExecution;

import java.time.ZonedDateTime;
import java.util.List;
import java.util.Set;

import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.model.lookup.JobExecutionType;
import gov.va.vss.model.lookup.JobExecutionType.JobExecutionTypeValue;
import gov.va.vss.persistence.dao.CustomizableSortedDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

/**
 * @author barrycon
 * 
 */
public interface JobExecutionResultDAO extends CustomizableSortedDAO<JobExecutionResult> {

	/**
	 * Finds a List of JobExecutionResults given the specified search criteria.
	 * 
	 * @param jobExecutionId
	 *            The ID of a specific JobExecutionResult (ignored if null)
	 * @param validStationIds
	 *            A set of station numbers for the Institutions that the
	 *            JobExecutionResults may be associated with. If this set is
	 *            valid but contains null, then JobExecutionResults not
	 *            associated with any VAFacility will be included (and
	 *            vice-versa, if the set is valid but does not contain null,
	 *            then JobExecutionResults not associated with any VAFacility
	 *            will be excluded). If this set itself is null then no
	 *            restriction is made one way or another on the linked
	 *            Institutions for each JobExecutionResult.
	 * 
	 * @param jobExecutionType
	 *            If specified, all returned results will have a type matching
	 *            this parameter.
	 * @param startDate
	 *            If specified, all returned results will have a start date on
	 *            or after this time.
	 * @param endDate
	 *            If specified, all returned results will have an end date on or
	 *            before this time.
	 * @param customization
	 * @return See description
	 */
	List<JobExecutionResult> findByCriteria(Long jobExecutionId, Set<Long> validStationIds,
			JobExecutionType jobExecutionType, ZonedDateTime startDate, ZonedDateTime endDate,
			QueryCustomization... customization);

	JobExecutionResult findRequiredByJobExecutionId(long jobExecutionId, QueryCustomization... customization);

	int deleteByCriteria(Long jobExecutionResultId, ZonedDateTime olderThanDate, JobExecutionTypeValue type,
			Long referenceJobExecutionId, String initiatorAppUserName);

}
