package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.BenefitingService;
import gov.va.vss.model.ScopeType;
import gov.va.vss.persistence.dao.BenefitingServiceDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class BenefitingServiceDAOImpl extends GenericHibernateSortedDAOImpl<BenefitingService>
		implements BenefitingServiceDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(BenefitingServiceDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<BenefitingService> findByCriteria(String code, String name, String subdivision, String abbreviation,
			boolean includeNationalScope, boolean includeLocalScope, Collection<Long> localFacilityIds,
			Collection<String> excludeCodesWithPrefixes, QueryCustomization... customization) {
		if (!includeLocalScope && !includeNationalScope)
			return new ArrayList<>();

		if (includeLocalScope && localFacilityIds == null)
			throw new IllegalArgumentException("Parameter 'localFacilityIds' is required if includeLocalScope is true");

		StringBuilder sb = new StringBuilder("select o from ").append(BenefitingService.class.getName()).append(" o");
		sb.append(" left join o.facility i");

		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "o");

		if (cust.getOrderBy() == null)
			cust.setOrderBy("o.scope desc, f.stationNumber, o.name");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (StringUtils.isNotBlank(name)) {
			whereClauseItems.add("LOWER(o.name) like :name1");
			params.put("name1", "%" + name.toLowerCase() + "%");
		}

		List<ScopeType> validScopeTypes = new ArrayList<>();
		if (includeLocalScope) {
			validScopeTypes.add(ScopeType.LOCAL);

			whereClauseItems.add("o.scope <> :localScope or COALESCE(i.id, -1) in ( :localFacilityIds )");
			params.put("localScope", ScopeType.LOCAL);
			params.put("localFacilityIds", localFacilityIds);
		}

		if (includeNationalScope)
			validScopeTypes.add(ScopeType.NATIONAL);

		whereClauseItems.add("o.scope in (:validScopes)");
		params.put("validScopes", validScopeTypes);

		if (excludeCodesWithPrefixes != null) {
			int i = 0;
			for (String c : excludeCodesWithPrefixes) {
				whereClauseItems.add("lower(o.code) not like :excludeCode" + i);
				params.put("excludeCode" + i, c.toLowerCase() + '%');
			}
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		return q.getResultList();
	}

}
