package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.BenefitingServiceRole;
import gov.va.vss.model.ScopeType;
import gov.va.vss.persistence.dao.BenefitingServiceRoleDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class BenefitingServiceRoleDAOImpl extends GenericHibernateSortedDAOImpl<BenefitingServiceRole>
		implements BenefitingServiceRoleDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(BenefitingServiceRoleDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<BenefitingServiceRole> findByCriteria(String name, boolean includeNationalScope,
			boolean includeLocalScope, Collection<Long> facilityIds, QueryCustomization... customization) {
		if (!includeLocalScope && !includeNationalScope)
			return new ArrayList<>();

		if (includeLocalScope && facilityIds == null)
			throw new IllegalArgumentException("Parameter 'facilityIds' is required if includeLocalScope is true");

		StringBuilder sb = new StringBuilder("select o from ").append(BenefitingServiceRole.class.getName())
				.append(" o");
		sb.append(" left join o.facility i");

		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "o");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (StringUtils.isNotBlank(name)) {
			whereClauseItems.add("LOWER(o.name) like :name1");
			params.put("name1", "%" + name.toLowerCase() + "%");
		}

		if (includeLocalScope && includeNationalScope) {
			if (CollectionUtils.isNotEmpty(facilityIds)) {
				whereClauseItems.add("COALESCE(i.id, -1) in ( :facilityIds ) or o.facility is null");
				params.put("facilityIds", facilityIds);
			}
		} else if (includeLocalScope && CollectionUtils.isNotEmpty(facilityIds)) {
			whereClauseItems.add("i.id in ( :facilityIds )");
			params.put("facilityIds", facilityIds);
		} else if (includeNationalScope) {
			whereClauseItems.add("o.facility is null");
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		return q.getResultList();
	}

}
