package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.FacilityAssignment;
import gov.va.vss.persistence.dao.FacilityAssignmentDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class FacilityAssignmentDAOImpl extends GenericHibernateSortedDAOImpl<FacilityAssignment>
		implements FacilityAssignmentDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(FacilityAssignmentDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<FacilityAssignment> findByCriteria(Collection<Long> benefitingServiceIds,
			Boolean facilityAssignmentActiveStatus, Long facilityId, QueryCustomization... customization) {

		StringBuilder sb = new StringBuilder("select v from ").append(FacilityAssignment.class.getName()).append(" v");

		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "v");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (CollectionUtils.isNotEmpty(benefitingServiceIds)) {
			whereClauseItems.add("v.benefitingService.id in (:benefitingServiceIds)");
			params.put("benefitingServiceIds", benefitingServiceIds);
		}

		if (facilityAssignmentActiveStatus != null) {
			whereClauseItems.add("v.inactive = :inactive");
			params.put("inactive", !facilityAssignmentActiveStatus);
		}

		if (facilityId != null) {
			whereClauseItems.add("v.facility.id = :facilityId");
			params.put("facilityId", facilityId);
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		int maxResults = -1;
		if (cust.getRowLimitation() != null)
			maxResults = cust.getRowLimitation();

		if (maxResults > 0)
			q.setMaxResults(maxResults);

		List<FacilityAssignment> resultList = q.getResultList();
		return resultList;
	}
}
