package gov.va.vss.persistence.impl;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.VoluntaryServiceStaff;
import gov.va.vss.persistence.dao.VoluntaryServiceStaffDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class VoluntaryServiceStaffDAOImpl extends GenericHibernateDAOImpl<VoluntaryServiceStaff>
		implements VoluntaryServiceStaffDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(VoluntaryServiceStaffDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<VoluntaryServiceStaff> findByCriteria(String stationNum, String appUserName,
			LocalDate activeAsOfDate) {
		StringBuilder sb = new StringBuilder("select s from ").append(VoluntaryServiceStaff.class.getName())
				.append(" s");
		sb.append(" left join fetch s.appUser a");
		sb.append(" left join fetch s.facility v");

		QueryCustomization cust = new QueryCustomization();

		if (cust.getOrderBy() == null)
			cust.setOrderBy("a.lastName, a.firstName");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (StringUtils.isNotBlank(stationNum)) {
			whereClauseItems.add("lower(v.stationNumber) = :stationNum");
			params.put("stationNum", stationNum);
		}

		if (StringUtils.isNotBlank(appUserName)) {
			whereClauseItems.add("a.username = :username");
			params.put("username", appUserName);
		}
		
		//Show all staff no matter 
		/*if (activeAsOfDate != null) {
			whereClauseItems.add("COALESCE(s.vavsStartDate, '1900-01-01') <= :activeAsOfDate");
			whereClauseItems.add("COALESCE(s.vavsEndDate, '2199-01-01') > :activeAsOfDate");
			params.put("activeAsOfDate", activeAsOfDate);
		} */

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		return q.getResultList();
	}
	
}
