package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.VolunteerFacilityAssignment;
import gov.va.vss.persistence.dao.VolunteerFacilityAssignmentDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class VolunteerFacilityAssignmentDAOImpl extends GenericHibernateDAOImpl<VolunteerFacilityAssignment>
		implements VolunteerFacilityAssignmentDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(VolunteerFacilityAssignmentDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<VolunteerFacilityAssignment> findByCriteria(Long volunteerId, Long facilityAssignmentId,
			Boolean facilityAssignmentActiveStatus, QueryCustomization... customization) {

		StringBuilder sb = new StringBuilder("select v from ").append(VolunteerFacilityAssignment.class.getName())
				.append(" v");

		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "v");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (volunteerId != null) {
			whereClauseItems.add("v.volunteer.id = :volunteerId");
			params.put("volunteerId", volunteerId);
		}

		if (facilityAssignmentId != null) {
			whereClauseItems.add("v.facilityAssignment.id = :facilityAssignmentId");
			params.put("facilityAssignmentId", facilityAssignmentId);
		}

		if (facilityAssignmentActiveStatus != null) {
			whereClauseItems.add("v.facilityAssignment.inactive = :inactive");
			params.put("inactive", !facilityAssignmentActiveStatus);
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		int maxResults = -1;

		if (cust.getRowLimitation() != null)
			maxResults = cust.getRowLimitation();

		if (maxResults > 0)
			q.setMaxResults(maxResults);

		List<VolunteerFacilityAssignment> resultList = q.getResultList();
		return resultList;
	}
}
