package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.VolunteerOrganization;
import gov.va.vss.persistence.dao.VolunteerOrganizationDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class VolunteerOrganizationDAOImpl extends GenericHibernateDAOImpl<VolunteerOrganization>
		implements VolunteerOrganizationDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(VolunteerOrganizationDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<VolunteerOrganization> findByCriteria(Long volunteerId, Long organizationId,
			QueryCustomization... customization) {
		if (volunteerId == null && organizationId == null)
			throw new IllegalArgumentException("No criteria specified");

		StringBuilder sb = new StringBuilder("select v from ").append(VolunteerOrganization.class.getName())
				.append(" v");

		/* Don't bother with this yet - CPB */
		QueryCustomization cust = new QueryCustomization();

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (volunteerId != null) {
			whereClauseItems.add("v.volunteer.id = :volunteerId");
			params.put("volunteerId", volunteerId);
		}

		if (organizationId != null) {
			whereClauseItems.add("v.organization.id = :organizationId");
			params.put("organizationId", organizationId);
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		List<VolunteerOrganization> resultList = q.getResultList();
		return resultList;
	}

}
