package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import javax.persistence.Query;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import com.google.common.collect.Iterables;

import gov.va.shared.util.PersistenceUtil;
import gov.va.vss.model.VtkUser;
import gov.va.vss.persistence.dao.VtkUserDAO;

@Repository
public class VtkUserDAOImpl extends GenericHibernateSortedDAOImpl<VtkUser> implements VtkUserDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(VtkUserDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<VtkUser> findByCriteria(String usernameCaseInsensitive) {
		StringBuilder sb = new StringBuilder("select t from ").append(VtkUser.class.getName()).append(" t");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (StringUtils.isNotBlank(usernameCaseInsensitive)) {
			whereClauseItems.add("lower(t.username) = :username");
			params.put("username", usernameCaseInsensitive.toLowerCase());
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null);
		List<VtkUser> u = q.getResultList();
		return u;
	}

	@Override
	public VtkUser findByAppUser(String username) {
		Collection<VtkUser> users = findByCriteria(username);
		if (users.isEmpty())
			return null;
		if (users.size() > 1)
			throw new IllegalStateException("More than one user was found with the username '" + username + "': IDs "
					+ StringUtils.join(PersistenceUtil.translateObjectsToIds(users), ","));
		VtkUser vtkUser = users.iterator().next();
		return vtkUser;
	}

	@Override
	public int deleteByAppUserId(long appUserId) {
		return query("delete from " + VtkUser.class.getName() + " u where u.appUser.id = :userId")
				.setParameter("userId", appUserId).executeUpdate();

	}
}
