package gov.va.vss.persistence.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import gov.va.vss.model.WorkEntry;
import gov.va.vss.persistence.dao.WorkEntryDAO;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class WorkEntryDAOImpl extends GenericHibernateSortedDAOImpl<WorkEntry> implements WorkEntryDAO {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(WorkEntryDAOImpl.class);

	@SuppressWarnings("unchecked")
	@Override
	public List<WorkEntry> findByCriteria(Long volunteerId, Long organizationId, QueryCustomization... customization) {
		StringBuilder sb = new StringBuilder("select o from ").append(WorkEntry.class.getName()).append(" o");

		QueryCustomization cust = ArrayUtils.isEmpty(customization) ? new QueryCustomization() : customization[0];
		cust.appendRemainingJoins(sb, "o");

		if (cust.getOrderBy() == null)
			cust.setOrderBy("o.dateWorked");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (volunteerId != null) {
			whereClauseItems.add("o.volunteerAssignment.volunteer.id = :volunteerId");
			params.put("volunteerId", volunteerId);
		}
		
		if (organizationId != null) {
			whereClauseItems.add("o.organization.id = :organizationId");
			params.put("organizationId", organizationId);
		}

		Query q = constructQuery(em, sb, whereClauseItems, params, null, cust);

		return q.getResultList();
	}

	@Override
	public int getNumYearsWorked(long volunteerId) {
		Number n = (Number) query(
				"select count(distinct h.fiscalYear) + min(v.grandfatheredYearsWorked) as YearsWorked from "
						+ WorkEntry.class.getName()
						+ " h join h.volunteerAssignment.volunteer v where v.id = :volunteerId")
								.setParameter("volunteerId", volunteerId).getSingleResult();
		return n == null ? 0 : n.intValue();
	}

}
