package gov.va.vss.persistence.impl.jobExecution;

import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import javax.persistence.Query;

import org.apache.commons.lang3.ArrayUtils;
import org.springframework.stereotype.Repository;

import gov.va.shared.model.AppUser;
import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.model.jobExecution.JobExecutionResultItem;
import gov.va.vss.model.lookup.JobExecutionType.JobExecutionTypeValue;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionResultItemDAO;
import gov.va.vss.persistence.impl.GenericHibernateSortedDAOImpl;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;

@Repository
public class JobExecutionResultItemDAOImpl extends GenericHibernateSortedDAOImpl<JobExecutionResultItem>
		implements JobExecutionResultItemDAO {

	@SuppressWarnings("unchecked")
	@Override
	public List<JobExecutionResult> findByCriteria(Long referencePositionId, QueryCustomization... customization) {
		QueryCustomization c = ArrayUtils.isNotEmpty(customization) ? customization[0] : new QueryCustomization();

		StringBuilder sb = new StringBuilder("select t from ").append(JobExecutionResultItem.class.getName())
				.append(" t");

		c.appendRemainingJoins(sb, "t");

		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> params = new HashMap<>();

		if (referencePositionId != null) {
			whereClauseItems.add("t.referencePosition.id = :referencePositionId");
			params.put("referencePositionId", referencePositionId);
		}

		if (c.getOrderBy() == null)
			c.setOrderBy("t.startTime desc");

		Query q = constructQuery(em, sb, whereClauseItems, params, null, c);

		return q.getResultList();
	}

	@Override
	public int bulkUpdateForPositionDeletion(long positionId, AppUser currentUser) {
		int numRowsUpdated = query("update " //
				+ JobExecutionResultItem.class.getName() //
				+ " set referencePosition = null" //
				+ " where referencePosition.id = :referencePositionId").setParameter("referencePositionId", positionId)
						.executeUpdate();
		flushIfDebug();
		return numRowsUpdated;

	}

	@Override
	public int deleteByCriteria(Long jobExecutionResultId, Long referencePatientId, Long referencePositionId,
			Long referenceTeamId, ZonedDateTime parentOlderThanDate, Long parentReferenceJobExecutionId,
			JobExecutionTypeValue parentJobExecutionTypeValue) {
		List<String> whereClauseItems = new ArrayList<>();
		Map<String, Object> paramMap = new HashMap<>();

		if (jobExecutionResultId != null) {
			whereClauseItems.add("a.jobExecutionResult.id = :jobExecutionResultId");
			paramMap.put("jobExecutionResultId", jobExecutionResultId);
		}

		if (referencePatientId != null) {
			whereClauseItems.add("a.referencePatient.id = :referencePatientId");
			paramMap.put("referencePatientId", referencePatientId);
		}

		if (referencePositionId != null) {
			whereClauseItems.add("a.referencePosition.id = :referencePositionId");
			paramMap.put("referencePositionId", referencePositionId);
		}

		if (referenceTeamId != null) {
			whereClauseItems.add("a.referenceTeam.id = :referenceTeamId");
			paramMap.put("referenceTeamId", referenceTeamId);
		}

		if (parentOlderThanDate != null) {
			whereClauseItems.add("a.jobExecutionResult.startTime < :cutoff");
			paramMap.put("cutoff", parentOlderThanDate);
		}

		if (parentReferenceJobExecutionId != null) {
			whereClauseItems.add("a.jobExecutionResult.referenceJobExecution.id" + " = :parentReferenceJobExecutionId");
			paramMap.put("parentReferenceJobExecutionId", parentReferenceJobExecutionId);
		}

		if (parentJobExecutionTypeValue != null) {
			whereClauseItems.add("a.jobExecutionResult.type.id" + " = :parentJobExecutionTypeId");
			paramMap.put("parentJobExecutionTypeId", parentJobExecutionTypeValue.getId());
		}

		StringBuilder whereClause = new StringBuilder();
		for (ListIterator<String> it = whereClauseItems.listIterator(); it.hasNext();) {
			whereClause.append(it.hasPrevious() ? " and " : " where ").append(it.next());
		}

		String[] queries = new String[] {
				/*
				 * Normally we'd delete the children of JobExecutionResultItem
				 * here, the only "child" / dependent object is the
				 * statusException which has a database-level cascade delete, so
				 * we can skip that one. CPB
				 */
				"delete from " + JobExecutionResultItem.class.getName() + " where id in (select a.id from "
						+ JobExecutionResultItem.class.getName() + " a" + whereClause.toString() + ")" };

		int numResults = 0;
		for (int queryIndex = 0; queryIndex < queries.length; queryIndex++) {
			Query q = query(queries[queryIndex]);
			for (Map.Entry<String, Object> paramEntry : paramMap.entrySet())
				q.setParameter(paramEntry.getKey(), paramEntry.getValue());
			/*
			 * Return the number of JobExecutionResultItems deleted, we don't
			 * care about the children count
			 */
			int currResults = q.executeUpdate();
			flushIfDebug();
			if (queryIndex == queries.length - 1)
				numResults = currResults;
		}

		return numResults;
	}

}
