package gov.va.vss.service.batch.tasklet;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.core.JobExecution;
import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.batch.core.scope.context.StepContext;
import org.springframework.batch.core.step.tasklet.Tasklet;
import org.springframework.batch.item.ExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.MessageSource;
import org.springframework.transaction.PlatformTransactionManager;

import gov.va.shared.model.AppUser;
import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.model.lookup.JobExecutionType;
import gov.va.vss.model.lookup.JobExecutionType.JobExecutionTypeValue;
import gov.va.vss.persistence.dao.BinaryObjectDAO;
import gov.va.vss.persistence.dao.jobExecution.JobExecutionResultDAO;
import gov.va.vss.persistence.dao.lookup.ActiveStatusDAO;
import gov.va.vss.persistence.dao.lookup.JobExecutionTypeDAO;
import gov.va.vss.service.BinaryObjectService;
import gov.va.vss.service.batch.JobExecutionResultItemService;
import gov.va.vss.service.batch.JobExecutionResultService;

public abstract class AbstractTasklet implements Tasklet {
	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(AbstractTasklet.class);

	public static final String JOB_CONTEXT_ATTR_JOB_RESULT_ID = "jobResultID";

	@Autowired
	protected ActiveStatusDAO activeStatusDAO;
	@Autowired
	protected BinaryObjectDAO binaryObjectDAO;
	@Autowired
	protected BinaryObjectService binaryObjectService;
	@Autowired
	protected JobExecutionResultDAO jobExecutionResultDAO;
	@Autowired
	protected JobExecutionResultService jobExecutionResultService;
	@Autowired
	protected JobExecutionResultItemService jobExecutionResultItemService;
	@Autowired
	protected JobExecutionTypeDAO jobExecutionTypeDAO;
	@Autowired
	protected MessageSource messageSource;
	@Autowired
	@Qualifier("transactionManager")
	protected PlatformTransactionManager tm;
	
	@Autowired
	protected JobExplorer jobExplorer;
	
	protected JobExecutionResult getOrCreateJobExecutionResult(StepContext stepContext, JobExecutionTypeValue type,
			AppUser initiator) {
		ExecutionContext stepExecutionContext = stepContext.getStepExecution().getExecutionContext();
		Map<String, Object> jobExecutionContext = stepContext.getJobExecutionContext();
		Long resultId = (Long) stepExecutionContext.get(JOB_CONTEXT_ATTR_JOB_RESULT_ID);
		if (resultId == null)
			resultId = (Long) jobExecutionContext.get(JOB_CONTEXT_ATTR_JOB_RESULT_ID);

		JobExecutionResult result = null;
		if (resultId == null) {
			Long jobExecutionId = stepContext.getStepExecution().getJobExecutionId();
			JobExecution jobExecution = jobExplorer.getJobExecution(jobExecutionId);
			JobExecutionType jobExecutionType = jobExecutionTypeDAO.findByLookup(type);
			result = new JobExecutionResult(jobExecution, jobExecutionType, initiator);
			result = jobExecutionResultService.saveOrUpdate(result);
			stepExecutionContext.putLong(JOB_CONTEXT_ATTR_JOB_RESULT_ID, result.getId());
		} else {
			result = jobExecutionResultDAO.findRequiredByPrimaryKey(resultId);
		}
		return result;
	}
}
