package gov.va.vss.service.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import gov.va.vss.model.Donor;
import gov.va.vss.model.Volunteer;
import gov.va.vss.model.lookup.VtkDonorType;
import gov.va.vss.model.organization.AbstractBasicOrganization;
import gov.va.vss.persistence.dao.lookup.VtkDonorTypeDAO;
import gov.va.vss.service.DonorService;
import gov.va.vss.service.validation.ServiceValidationException;
import gov.va.vss.web.ValidationException;

@Service
public class DonorServiceImpl extends AbstractServiceImpl implements DonorService {
	private static final Logger log = LoggerFactory.getLogger(DonorServiceImpl.class);

	@Autowired
	VtkDonorTypeDAO vtkDonorTypeDAO;
	
	@Override
	public Donor saveOrUpdate(Donor donor)
			throws ServiceValidationException {

		/* Business-level validations */

		donor = donorDAO.saveOrUpdate(donor);
		return donor;
	}

	
	@Override
	public Donor linkVolunteer(Long donorId, Long volunteerId) throws ValidationException {
		
		Donor donor = null;
		
		if (donorId == null) {
			// new donor that should be created
			donor = new Donor();
			donor.setVtkDonorType(vtkDonorTypeDAO.findByLookup(VtkDonorType.VtkDonorTypeValue.INDIVIDUAL));
		} else {		
			donor = donorDAO.findRequiredByPrimaryKey(donorId);
			if (donor == null) {
				throw new IllegalArgumentException("Donor not found for donor id = " + donorId);
			}
		}
	
		if (volunteerId == null) {
			throw new IllegalArgumentException("Volunteer Id cannot be null");
		}

		// donor record should no longer retain name/address info
		clearDonorData(donor);

		Volunteer v = volunteerDAO.findRequiredByPrimaryKey(volunteerId);
		donor.setVolunteer(v);

		try {
			donor = saveOrUpdate(donor);
		} catch (ServiceValidationException e) {
			throw new ValidationException(e);
		}
		
		return donor;
	
	}
	
	@Override
	public Donor linkOrganization(Long donorId,Long orgId) throws ValidationException {
		Donor donor = null;
		if (donorId == null) {
			// new donor that should be created
			donor = new Donor();
			donor.setVtkDonorType(vtkDonorTypeDAO.findByLookup(VtkDonorType.VtkDonorTypeValue.ORGANIZATION));
		} else {
			donor = donorDAO.findRequiredByPrimaryKey(donorId);

			if (donor == null) {
				throw new IllegalArgumentException("Donor not found for donor id = " + donorId);
			}
		}
	
		if (orgId == null) {
			throw new IllegalArgumentException("Organization Id cannot be null");
		}
		
		// donor record should no longer retain name/address info
		clearDonorData(donor);

		AbstractBasicOrganization o = organizationDAO.findRequiredByPrimaryKey(orgId);
		donor.setOrganization(o);

		try {
			donor = saveOrUpdate(donor);
		} catch (ServiceValidationException e) {
			throw new ValidationException(e);
		}
		
		return donor;
	
	}
	
	private Donor clearDonorData(Donor donor) {
		donor.setAddressLine1(null);
		donor.setAddressLine2(null);
		donor.setCity(null);
		donor.setDonorAffiliation(null);
		donor.setEmail(null);
		donor.setFirstName(null);
		donor.setLastName(null);
		donor.setMiddleName(null);
		donor.setPhone(null);
		donor.setPrefix(null);
		donor.setState(null);
		donor.setSuffix(null);
		donor.setZip(null);
		return donor;
	}
	
}
