package gov.va.vss.service.impl;

import java.time.ZonedDateTime;

import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import gov.va.vss.model.jobExecution.JobExecutionResult;
import gov.va.vss.model.jobExecution.JobExecutionResultItem;
import gov.va.vss.model.lookup.JobExecutionType.JobExecutionTypeValue;
import gov.va.vss.service.batch.JobExecutionResultItemService;

@Service
@Profile({ "attended" })
public class JobExecutionResultItemServiceImpl extends AbstractServiceImpl implements JobExecutionResultItemService {

	@Override
	public JobExecutionResultItem saveOrUpdate(JobExecutionResultItem result) {
		return jobExecutionResultItemDAO.saveOrUpdate(result);
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRES_NEW)
	public JobExecutionResultItem saveOrUpdateInNewTransaction(JobExecutionResultItem result) {
		return jobExecutionResultItemDAO.saveOrUpdate(result);
	}

	@Override
	public int deleteByCriteria(Long jobExecutionResultId, ZonedDateTime olderThanDate, JobExecutionTypeValue type,
			Long referenceJobExecutionId) {
		int numDeleted = jobExecutionResultItemDAO.deleteByCriteria(jobExecutionResultId, null, null, null,
				olderThanDate, referenceJobExecutionId, type);
		return numDeleted;
	}

	public JobExecutionResultItem saveItemResult(JobExecutionResult result, ZonedDateTime start, ZonedDateTime end,
			boolean successful, String message, Exception ex) {
		JobExecutionResultItem resultItem = new JobExecutionResultItem(start);

		resultItem.setEndTime(end);
		resultItem.setItemSuccessful(successful);
		resultItem.setItemStatusMessageStr(message);
		if (ex != null)
			resultItem.setStatusException(ex);

		resultItem.setJobExecutionResult(result);
		resultItem = saveOrUpdate(resultItem);
		return resultItem;
	}

	@Transactional(propagation = Propagation.REQUIRES_NEW)
	public JobExecutionResultItem saveItemResultInNewTransaction(JobExecutionResult result, ZonedDateTime start,
			ZonedDateTime end, boolean successful, String message, Exception ex) {
		return saveItemResult(result, start, end, successful, message, ex);
	}

}
