package gov.va.vss.service.impl;

import java.time.ZonedDateTime;
import java.util.Collection;
import java.util.List;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import gov.va.shared.util.StationsUtil;
import gov.va.vss.model.VistaStationData;
import gov.va.vss.service.VistaStationDataService;

@Service
public class VistaStationDataServiceImpl extends AbstractServiceImpl implements VistaStationDataService {
	private static final Logger log = LoggerFactory.getLogger(VistaStationDataServiceImpl.class);

	@Resource(name = "vistaStationDataServiceImpl")
	private VistaStationDataServiceImpl selfProxy;

	@Override
	public VistaStationData getOrCreateByStation(String stationNumber) {
		stationNumber = StationsUtil.getThreeDigitStationNumber(stationNumber);
		VistaStationData d = vistaStationDataDAO.findByStationNumber(stationNumber);
		if (d != null)
			return d;

		VistaStationData data = selfProxy.getStationNumberInNewTransaction(stationNumber);
		return data;
	}

	@Transactional(propagation = Propagation.REQUIRES_NEW)
	public VistaStationData getStationNumberInNewTransaction(String stationNumber) {
		VistaStationData d = vistaStationDataDAO.findByStationNumber(stationNumber);
		if (d != null)
			return d;

		d = new VistaStationData();
		d.setStationNumber(stationNumber);
		d = vistaStationDataDAO.saveOrUpdate(d);
		vistaStationDataDAO.clearCache();

		return d;
	}

	@Override
	@Transactional(propagation = Propagation.REQUIRES_NEW)
	public int saveOrUpdateWithFieldsInNewTransaction(String stationNumber,
			ZonedDateTime cprsHeaderSyncLastExecutionDate, ZonedDateTime patientInactivationLastExecutionDate,
			ZonedDateTime vistaCleanupProcessLastExecutionDate, ZonedDateTime staffUpdateLastExecutionDate) {
		int rowsUpdated = vistaStationDataDAO.updateWithFields(stationNumber, null, cprsHeaderSyncLastExecutionDate,
				null, patientInactivationLastExecutionDate, null, staffUpdateLastExecutionDate, null,
				vistaCleanupProcessLastExecutionDate, null);
		if (rowsUpdated == 0) {
			getOrCreateByStation(stationNumber);
			rowsUpdated = vistaStationDataDAO.updateWithFields(stationNumber, null, cprsHeaderSyncLastExecutionDate,
					null, patientInactivationLastExecutionDate, null, staffUpdateLastExecutionDate, null,
					vistaCleanupProcessLastExecutionDate, null);
		}
		return rowsUpdated;
	}

	public int updateCPRSHeaderSyncLastExecutionDatesForEnabledStations(Collection<String> stationNumbers,
			ZonedDateTime executionDate) {
		return vistaStationDataDAO.updateCPRSHeaderSyncLastExecutionDatesForEnabledStations(stationNumbers,
				executionDate);
	}

	@Override
	public int setProcessesEnabled(String stationNumber, Boolean cprsHeaderSyncProcessEnabled,
			Boolean patientInactivationProcessEnabled, Boolean vistaSyncProcessEnabled,
			Boolean vistaCleanupProcessEnabled, Boolean staffUpdateProcessEnabled) {
		int rowsUpdated = vistaStationDataDAO.updateWithFields(stationNumber, cprsHeaderSyncProcessEnabled, null,
				patientInactivationProcessEnabled, null, staffUpdateProcessEnabled, null, vistaCleanupProcessEnabled,
				null, vistaSyncProcessEnabled);
		if (rowsUpdated == 0) {
			getOrCreateByStation(stationNumber);
			rowsUpdated = vistaStationDataDAO.updateWithFields(stationNumber, cprsHeaderSyncProcessEnabled, null,
					patientInactivationProcessEnabled, null, staffUpdateProcessEnabled, null,
					vistaCleanupProcessEnabled, null, vistaSyncProcessEnabled);
		}
		return rowsUpdated;
	}

	@Override
	public List<VistaStationData> getStationsNeedingAutoInactivationProcessing() {
		List<VistaStationData> map = vistaStationDataDAO.findByCriteria(null, null, true);
		return map;
	}

}
