package gov.va.vss.web.donation;

import java.time.LocalDate;

import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;

import gov.va.vss.model.DonGenPostFund;
import gov.va.vss.model.DonationDetail;
import gov.va.vss.model.DonationSummary;
import gov.va.vss.model.lookup.VtkDonationType;
import gov.va.vss.model.lookup.VtkDonationType.VtkDonationTypeValue;
import gov.va.vss.web.AbstractAppValidator;

@Component
public class DonationValidator extends AbstractAppValidator<DonationCommand> {

	@Override
	public void doExtraValidations(DonationCommand command, Errors errors) {
		DonationSummary donationSummary = command.getDonationSummary();

		// for example: CPB

		// No need to check null for donation date since it has been set to
		// default if it is null in database
		if (command.getCurrentFiscalYearStartDate().isAfter(command.getDonationSummary().getDonationDate())
				|| command.getCurrentFiscalYearEndDate().isBefore(command.getDonationSummary().getDonationDate())) {
			errors.rejectValue("donationSummary.donationDate", "donationSummary.error.donationDateNotWhthinFiscalYear");

		}

		if (donationSummary.getDonationDate().isAfter(LocalDate.now()))
			errors.rejectValue("donationSummary.donationDate", "donationSummary.error.donationDate");

		if (donationSummary.getAcknowledgementDate() != null
				&& donationSummary.getDonationDate().isAfter(donationSummary.getAcknowledgementDate()))
			errors.rejectValue("donationSummary.acknowledgementDate", "donationSummary.error.acknowledgementDate");

		VtkDonationType donationType = donationSummary.getVtkDonationType();
		if (donationType != null) {
			if (donationType.getId() == VtkDonationTypeValue.ACTIVITY.getId()
					|| donationType.getId() == VtkDonationTypeValue.ITEM.getId()) {
				// search for a non-none GPF
				boolean hasNotNoneGPF = false;
				for (DonationDetail dd : donationSummary.getDonationDetails()) {
					DonGenPostFund donGenPostFund = dd.getDonGenPostFund();
					if (donGenPostFund != null
							&& !DonGenPostFund.GPF_NONE_VALUE.equals(donGenPostFund.getGeneralPostFund())) {
						hasNotNoneGPF = true;
						break;
					}
				}
				if (hasNotNoneGPF) {
					errors.rejectValue("donationSummary.type", "donationSummary.error.acknowledgementDate");
				}
			}
		}
	}

}
