package gov.va.vss.web.reports;

import static gov.va.vss.persistence.dao.BenefitingServiceDAO.BENEFITING_SERVICE_PREFIX_GAMES;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSession;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.HttpClientConnectionManager;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.conn.BasicHttpClientConnectionManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.ssl.SSLContexts;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.fasterxml.jackson.annotation.JsonView;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;

import gov.va.shared.model.AppUserFacility;
import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.util.PersistenceUtil;
import gov.va.vss.model.BenefitingService;
import gov.va.vss.model.BenefitingService.BenefitingServiceView;
import gov.va.vss.model.Volunteer;
import gov.va.vss.model.Volunteer.VolunteerView;
import gov.va.vss.model.facility.VtkAdministrativeUnit;
import gov.va.vss.model.organization.AbstractBasicOrganization.OrganizationView;
import gov.va.vss.model.organization.BasicOrganization;
import gov.va.vss.model.views.CombinedFacility;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.interceptor.BreadcrumbsInterceptor;

@Controller
public class ReportParametersController extends AbstractAppController {
	private static final Logger log = LoggerFactory.getLogger(ReportParametersController.class);

	private static final String PROP_REPORT_PREFIX = "reports.";

	@Value("${disableHostnameVerification}")
	private boolean disableHostnameVerification;
	@Value("${ssrs.baseURL}")
	private String baseURL;
	@Value("${ssrs.viewReport.baseURL}")
	private String baseViewURL;

	@RequestMapping("/printReport.htm")
	public void testReport(@RequestParam String reportName, HttpServletRequest req, HttpServletResponse servletResponse)
			throws IOException {
		servletResponse.setHeader("X-Frame-Options", "SAMEORIGIN");
		CloseableHttpClient httpclient = createHttpClient();

		HttpPost httpPost = new HttpPost(baseURL + "?/" + reportName);
		List<NameValuePair> nvps = new ArrayList<>();

		Set<String> paramsToIgnore = new HashSet<>(Arrays.asList("reportName", "_csrf"));

		Map<String, String[]> params = req.getParameterMap();
		for (String name : params.keySet()) {
			if (paramsToIgnore.contains(name))
				continue;

			for (String val : params.get(name)) {
				System.out.println("Adding " + name + " = " + val);
				nvps.add(new BasicNameValuePair(name, val));
			}
		}

		httpPost.setEntity(new UrlEncodedFormEntity(nvps));
		try (CloseableHttpResponse ssrsResponse = httpclient.execute(httpPost);) {
			copyAndDecoratePDFEntity(ssrsResponse, servletResponse);
		}
	}

	public CloseableHttpClient createHttpClient() {
		HttpClientBuilder b = HttpClientBuilder.create();

		try {
			if (disableHostnameVerification) {
				HostnameVerifier hostnameVerifier = new HostnameVerifier() {
					@Override
					public boolean verify(String hostname, SSLSession session) {
						return true;
					}
				};
				HttpsURLConnection.setDefaultHostnameVerifier(hostnameVerifier);

				SSLConnectionSocketFactory sslConnectionFactory = new SSLConnectionSocketFactory(
						SSLContexts.createSystemDefault(), hostnameVerifier);
				b.setSSLSocketFactory(sslConnectionFactory);
				Registry<ConnectionSocketFactory> registry = RegistryBuilder.<ConnectionSocketFactory> create()
						.register("https", sslConnectionFactory)
						// .register("http", new PlainConnectionSocketFactory())
						.build();
				HttpClientConnectionManager ccm = new BasicHttpClientConnectionManager(registry);
				b.setConnectionManager(ccm);
			}

			return b.build();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	@RequestMapping("/displayReportParameters")
	public String displayReportParameters(@RequestParam(required = true) String reportCode, ModelMap model,
			HttpServletRequest request) {
		model.put("reportBaseURL", baseURL);
		model.put("reportViewURL", baseViewURL);
		model.put("allStates", stateDAO.findAllSorted());

		String displayName = env.getRequiredProperty(PROP_REPORT_PREFIX + reportCode + ".displayName");
		model.put("pageTitle", "Specify Parameters for the " + displayName + " Report");
		model.put("pageDescription",
				"This screen allows the user to specify parameters for the " + displayName + " Report");
		model.put("ssrsReportBasename",
				env.getRequiredProperty(PROP_REPORT_PREFIX + reportCode + ".ssrsReportBasename"));
		model.put("disclaimerText", env.getProperty(PROP_REPORT_PREFIX + reportCode + ".disclaimerText"));
		BreadcrumbsInterceptor.setRequestBreadcrumb(request, "Run Report \"" + displayName + "\"");

		return env.getRequiredProperty(PROP_REPORT_PREFIX + reportCode + ".tilesView");
	}

	@RequestMapping(value = "/assignedVAFacilities", method = RequestMethod.GET)
	public @ResponseBody Collection<VAFacility> getAssignedVAFacilities() {
		SortedSet<VAFacility> allLinkedFacilities = facilityDAO.findVAFacilitiesWithLinkToFacility();
		if (getCurrentUser().isNationalAdmin())
			return allLinkedFacilities;

		List<VAFacility> assignedFacilities = new ArrayList<VAFacility>();
		for (AppUserFacility auFac : getCurrentUser().getFacilities()) {
			VAFacility facility = auFac.getFacility();
			if (allLinkedFacilities.contains(facility))
				assignedFacilities.add(facility);
		}

		return assignedFacilities;
	}

	@RequestMapping(value = "/assignedFacilities", method = RequestMethod.GET)
	public @ResponseBody Collection<CombinedFacility> getAssignedFacilities(@RequestParam boolean showAllFacilities) {
		SortedSet<CombinedFacility> linkedFacilities = combinedInstitutionDAO.findWithLinkToVAFacility();
		if (getCurrentUser().isNationalAdmin() || showAllFacilities)
			return linkedFacilities;

		Set<Long> vaFacilityIds = PersistenceUtil.translateObjectsToIds(getCurrentUser().getBasicFacilities());

		for (Iterator<CombinedFacility> it = linkedFacilities.iterator(); it.hasNext();) {
			CombinedFacility vi = it.next();
			if (!vaFacilityIds.contains(vi.getVaFacility().getId()))
				it.remove();
		}

		return linkedFacilities;
	}

	/**
	 * Powers the jQuery organization selection table - CPB
	 * 
	 * @param draw
	 * @param start
	 * @param length
	 * @param searchValue
	 * @param searchIsRegex
	 * @param allParams
	 * @return
	 */
	@RequestMapping("/organization/quickSearch")
	public @ResponseBody Map<String, Object> findOrganizationsByName(@RequestParam int draw, @RequestParam int start,
			@RequestParam int length, @RequestParam(name = "search[value]") String searchValue,
			@RequestParam(name = "search[regex]") boolean searchIsRegex) {
		Map<String, Object> resultMap = new HashMap<>();

		SortedSet<OrgQuickSearchResult> results = null;
		if (StringUtils.isNotBlank(searchValue)) {
			results = new TreeSet<>(organizationDAO.quickSearch(searchValue, length));
		} else {
			results = new TreeSet<>();
		}
		resultMap.put("data", results);
		resultMap.put("draw", draw);
		return resultMap;
	}

	@RequestMapping("/organization")
	@JsonView(OrganizationView.Basic.class)
	public @ResponseBody List<BasicOrganization> findOrganizations(@RequestParam boolean includeLocal,
			@RequestParam boolean includeNational, @RequestParam(required = false) Long stationId) {
		return organizationDAO.findByCriteria(null, includeNational, includeLocal,
				stationId != null ? Arrays.asList(stationId) : null, true, null, null);
	}

	@RequestMapping("/benefitingService")
	@JsonView(BenefitingServiceView.Basic.class)
	public @ResponseBody SortedSet<BenefitingService> findBenefitingServices(@RequestParam boolean includeLocal,
			@RequestParam boolean includeNational, @RequestParam boolean excludeGames, @RequestParam long stationId) {
		return new TreeSet<>(benefitingServiceDAO.findByCriteria(null, null, null, null, includeNational, includeLocal,
				Arrays.asList(stationId), excludeGames ? Arrays.asList(BENEFITING_SERVICE_PREFIX_GAMES) : null));
	}

	@RequestMapping(value = "/assignedVISNs", method = RequestMethod.GET)
	public @ResponseBody Collection<VtkAdministrativeUnit> getAssignedVISNs() {
		// if (getCurrentUser().isNationalAdmin()) {
		return vtkAdministrativeUnitDAO.findAllSorted();
		// } else {
		// List<VAFacility> assignedFacilities = new ArrayList<VAFacility>();
		// for (AppUserFacility auFac : getCurrentUser().getFacilities()) {
		// assignedFacilities.add(auFac.getFacility());
		// }
		// return assignedFacilities;
		// }
	}

	@RequestMapping(value = "/volunteersAtFacility", method = RequestMethod.GET)
	@JsonView(VolunteerView.Basic.class)
	public @ResponseBody SortedSet<Volunteer> getVolunteersAtFacility(@RequestParam long stationId) {
		List<Volunteer> volunteerOrgs = volunteerDAO.findByCriteria(null, null, null, null, null, null, null, null,
				null, null, null, true, null, null, Arrays.asList(stationId));
		return new TreeSet<>(volunteerOrgs);
	}

	@RequestMapping(value = "/volunteerZipCodesAtFacility", method = RequestMethod.GET)
	public @ResponseBody SortedSet<String> getVolunteerZipCodesAtFacility(@RequestParam long stationId) {
		List<String> zipCodes = volunteerDAO.findZipCodesAtFacilities(Arrays.asList(stationId), true, true);
		return new TreeSet<>(zipCodes);
	}

	@RequestMapping(value = "/monthsOfYear", method = RequestMethod.GET)
	public @ResponseBody SortedSet<MultiSelectGenericCommand<Integer, String>> getMonthsOfYear() {
		SortedSet<MultiSelectGenericCommand<Integer, String>> months = new TreeSet<MultiSelectGenericCommand<Integer, String>>();
		Calendar cal = Calendar.getInstance();
		Map<String, Integer> monthMap = cal.getDisplayNames(Calendar.MONTH, Calendar.LONG, Locale.ENGLISH);
		for (Map.Entry<String, Integer> entry : monthMap.entrySet()) {
			String monthName = entry.getKey();
			Integer monthInt = entry.getValue() + 1;
			months.add(new MultiSelectGenericCommand<Integer, String>(monthInt, monthName));
		}

		return months;
	}

	public static void copyAndDecoratePDFEntity(HttpResponse proxyResponse, HttpServletResponse servletResponse)
			throws IOException {
		OutputStream servletOutputStream = servletResponse.getOutputStream();

		HttpEntity entity = proxyResponse.getEntity();
		if (entity == null)
			return;

		byte[] inputPdf = EntityUtils.toByteArray(entity);

		try {
			PdfReader reader = new PdfReader(inputPdf);
			ByteArrayOutputStream baos = new ByteArrayOutputStream(inputPdf.length + 2000);
			PdfStamper stamper = new PdfStamper(reader, baos);
			stamper.addJavaScript("print();");
			stamper.close();

			servletResponse.setContentType("application/pdf");
			servletResponse.setContentLength(baos.size());

			baos.writeTo(servletOutputStream);
		} catch (Exception e) {
			log.error("Error stamping PDF with additional content", e);
			servletOutputStream.write(inputPdf);
		}
	}

}