package gov.va.vss.web.volunteer;

import static gov.va.shared.model.Permission.PermissionType.VOLUNTEER_CREATE;
import static gov.va.shared.util.SecurityUtil.hasAllPermissionsAtCurrentFacility;
import static gov.va.vss.persistence.queryCustomization.fieldTypes.FacilityAssignmentFieldType.BENEFITING_SERVICE;
import static gov.va.vss.persistence.queryCustomization.fieldTypes.FacilityAssignmentFieldType.BENEFITING_SERVICE_ROLE;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;

import com.fasterxml.jackson.annotation.JsonView;

import gov.va.shared.model.Permission.PermissionType;
import gov.va.shared.model.lookup.sds.State;
import gov.va.shared.model.lookup.sds.VAFacility;
import gov.va.shared.util.SecurityUtil;
import gov.va.shared.util.StringUtil;
import gov.va.vss.model.FacilityAssignment;
import gov.va.vss.model.FacilityAssignment.FacilityAssignmentView.SearchForFacilityAssignments;
import gov.va.vss.model.ParkingSticker;
import gov.va.vss.model.ParkingSticker.ParkingStickerView;
import gov.va.vss.model.ShirtSize;
import gov.va.vss.model.Uniform;
import gov.va.vss.model.Uniform.UniformView;
import gov.va.vss.model.Volunteer;
import gov.va.vss.model.Volunteer.VolunteerView;
import gov.va.vss.model.VolunteerFacilityAssignment;
import gov.va.vss.model.VolunteerFacilityAssignment.CompareByFacilityAssignment;
import gov.va.vss.model.VolunteerFacilityAssignment.VolunteerFacilityAssignmentView.SearchForVolunteerFacilityAssignments;
import gov.va.vss.model.VolunteerOrganization;
import gov.va.vss.model.VolunteerOrganization.CompareByOrganization;
import gov.va.vss.model.VolunteerOrganization.VolunteerOrganizationView;
import gov.va.vss.model.facility.Facility;
import gov.va.vss.persistence.dao.VolunteerDAO.TimeSummary;
import gov.va.vss.persistence.queryCustomization.QueryCustomization;
import gov.va.vss.persistence.queryCustomization.fieldTypes.VolunteerFacilityAssignmentFieldType;
import gov.va.vss.service.validation.ServiceValidationException;
import gov.va.vss.util.DateUtil;
import gov.va.vss.web.AbstractAppController;
import gov.va.vss.web.ValidationException;
import gov.va.vss.web.breadcrumbs.Breadcrumb;

@Controller
@SessionAttributes(value = { AbstractAppController.DEFAULT_COMMAND_NAME })
public class VolunteerController extends AbstractAppController {

	@Autowired
	private VolunteerValidator volunteerValidator;

	@RequestMapping("/getVolunteerSearchStations")
	public @ResponseBody SortedSet<Facility> getStationsForVolunteerSearch() {
		// Set<VAFacility> validFacilities =
		// SecurityUtil.getAllFacilitiesWhereUserHasPermissions(VOLUNTEER_READ);
		// return new TreeSet<>(validFacilities);
		return facilityDAO.findAllSorted(); // findVAFacilitiesWithLinkToFacility();
	}

	@RequestMapping("/volunteerOrganizations")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	@JsonView(VolunteerOrganizationView.SearchForOrganizations.class)
	public @ResponseBody Map<String, Object> findOrgsForVolunteer(@RequestParam long volunteerId) {
		Map<String, Object> results = new HashMap<>();
		Volunteer v = volunteerDAO.findRequiredByPrimaryKey(volunteerId);

		SortedSet<VolunteerOrganization> r = new TreeSet<>(new CompareByOrganization());
		r.addAll(v.getVolunteerOrganizations());
		results.put("organizations", r);

		results.put("primaryOrganization", v.getPrimaryOrganization());
		return results;
	}

	@RequestMapping(value = "/volunteer/setPrimaryOrganization", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean primaryOrganizationSet(@RequestParam long volunteerId,
			@RequestParam long organizationId) {
		volunteerService.setPrimaryOrganization(volunteerId, organizationId);
		return true;
	}

	@RequestMapping(value = "/volunteer/deleteParkingSticker", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean parkingStickerDelete(@RequestParam long parkingStickerId) {
		parkingStickerService.delete(parkingStickerId);
		return true;
	}

	@RequestMapping(value = "/volunteer/deleteUniform", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean uniformDelete(@RequestParam long uniformId) {
		uniformService.delete(uniformId);
		return true;
	}

	@RequestMapping(value = "/volunteer/deleteFacilityAssignment", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean facilityAssignmentDelete(@RequestParam long volunteerFacilityAssignmentId) {
		boolean reportPrimaryInactivated = volunteerService.deleteAssignment(volunteerFacilityAssignmentId);
		return reportPrimaryInactivated;
	}

	@RequestMapping(value = "/volunteer/inactivateOrganization", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean organizationDeleteOrInactivate(@RequestParam long volunteerOrganizationId) {
		boolean reportPrimaryInactivated = volunteerService.deleteOrInactivateOrganization(volunteerOrganizationId);
		return reportPrimaryInactivated;
	}

	@RequestMapping(value = "/volunteer/addOrganization", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean organizationAdd(@RequestParam long volunteerId, @RequestParam long organizationId)
			throws ServiceValidationException {
		volunteerService.addOrganization(volunteerId, organizationId);
		return true;
	}

	@RequestMapping(value = "/volunteer/addFacilityAssignment", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean facilityAssignmentAdd(@RequestParam long volunteerId,
			@RequestParam long facilityAssignmentId) {
		boolean assignmentAdded = volunteerService.addFacilityAssignment(volunteerId, facilityAssignmentId);
		return assignmentAdded;
	}

	@RequestMapping(value = "/volunteer/parkingSticker/createOrUpdate", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean parkingStickerCreateOrUpdate(@RequestParam long facilityId,
			@RequestParam(required = false) Long parkingStickerId, @RequestParam long volunteerId,
			@RequestParam String number, @RequestParam(required = false) State state,
			@RequestParam(required = false) String licensePlate) {
		ParkingSticker ps;
		if (parkingStickerId != null) {
			ps = parkingStickerDAO.findRequiredByPrimaryKey(parkingStickerId);
		} else {
			ps = new ParkingSticker();
			Volunteer v = volunteerDAO.findRequiredByPrimaryKey(volunteerId);
			ps.setVolunteer(v);
		}

		ps.setStickerNumber(number);
		ps.setLicensePlate(licensePlate);
		ps.setState(state);

		Facility i = facilityDAO.findRequiredByPrimaryKey(facilityId);
		ps.setFacility(i);

		ps = parkingStickerService.saveOrUpdate(ps);
		return true;
	}

	@RequestMapping(value = "/volunteer/setPrimaryFacility", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean primaryFacilitySet(@RequestParam long volunteerId, @RequestParam long facilityId) {
		volunteerService.setPrimaryFacility(volunteerId, facilityId);
		return true;
	}

	@RequestMapping("/findFacilityAssignments")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	@JsonView(SearchForFacilityAssignments.class)
	public @ResponseBody SortedSet<FacilityAssignment> facilityAssignmentFindForFacility(
			@RequestParam long facilityId) {
		List<FacilityAssignment> results = facilityAssignmentDAO.findByCriteria(null, true, facilityId,
				new QueryCustomization(BENEFITING_SERVICE_ROLE, BENEFITING_SERVICE));
		return new TreeSet<>(results);
	}

	@RequestMapping("/volunteerAssignments")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	@JsonView(SearchForVolunteerFacilityAssignments.class)
	public @ResponseBody Map<String, Object> findAssignmentsForVolunteer(@RequestParam long volunteerId) {
		Map<String, Object> results = new HashMap<>();
		Volunteer v = volunteerDAO.findRequiredByPrimaryKey(volunteerId);

		List<VolunteerFacilityAssignment> resultList = volunteerFacilityAssignmentDAO
				.findByCriteria(volunteerId, null, null,
						new QueryCustomization()
								.prefetchField(VolunteerFacilityAssignmentFieldType.FACILITY_ASSIGNMENT, "fa")
								.addExtraJoin(" left join fetch fa.benefitingService"));
		SortedSet<VolunteerFacilityAssignment> r = new TreeSet<>(new CompareByFacilityAssignment());
		r.addAll(resultList);
		results.put("assignments", r);

		results.put("primaryFacility", v.getPrimaryFacility());
		return results;
	}

	@RequestMapping("/volunteerParkingStickers")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	@JsonView(ParkingStickerView.Extended.class)
	public @ResponseBody SortedSet<ParkingSticker> findParkingStickersForVolunteer(@RequestParam long volunteerId) {
		Volunteer v = volunteerDAO.findRequiredByPrimaryKey(volunteerId);
		SortedSet<ParkingSticker> r = new TreeSet<>();
		r.addAll(v.getParkingStickers());
		return r;
	}

	@RequestMapping("/volunteerUniforms")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	@JsonView(UniformView.Extended.class)
	public @ResponseBody SortedSet<Uniform> findUniformsForVolunteer(@RequestParam long volunteerId) {
		Volunteer v = volunteerDAO.findRequiredByPrimaryKey(volunteerId);
		SortedSet<Uniform> r = new TreeSet<>();
		r.addAll(v.getUniforms());
		return r;
	}

	@RequestMapping("/findVolunteers")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	@JsonView(VolunteerView.Search.class)
	public @ResponseBody SortedSet<Volunteer> findVolunteers(@RequestParam(required = false) String name,
			@RequestParam(required = false) String code, @RequestParam(required = false) String email,
			@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY) @RequestParam(required = false) LocalDate dob,
			@RequestParam String scope, @RequestParam(required = false) Long facilityId,
			@RequestParam boolean includeInactive) {
		if (StringUtil.allBlank(name, code, email) && dob == null)
			throw new IllegalArgumentException("Please specify at least one piece of search criteria");

		if ("National".equals(scope))
			facilityId = null;

		String firstName = null;
		String lastName = null;
		String middleName = null;
		if (StringUtils.isNotBlank(name)) {
			String[] nameComponents = StringUtil.parseNameComponents(name);
			lastName = nameComponents[0];
			firstName = nameComponents[1];
			middleName = nameComponents[2];
		}

		SortedSet<Volunteer> volunteers = new TreeSet<>(volunteerDAO.findByCriteria(firstName, middleName, lastName,
				code, dob, null, null, null, null, null, email, includeInactive ? null : true, null, null,
				facilityId != null ? Arrays.asList(facilityId) : null));

		/*
		 * Populate a map if tertiary information (outside of the Volunteer
		 * object itself) needs to be included in the response; otherwise, just
		 * return the Collection<Volunteer> above directly and Jackson will
		 * serialize it into JSON (per the annotations in the Volunteer class).
		 * CPB
		 */
		// List<Map<String, Object>> results = new ArrayList<>();
		//
		// for (Volunteer volunteer : volunteers) {
		// Map<String, Object> result = new HashMap<>();
		// result.put("volunteerId", volunteer.getId());
		// result.put("displayName", volunteer.getDisplayName());
		// result.put("contactInfo",
		// volunteer.getStationProfileForSta3n(sta3n).getContactInfo());
		// results.add(result);
		// }
		//
		// return results;

		return volunteers;
	}

	@RequestMapping("/volunteerDuplicateCheck")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	@JsonView(VolunteerView.Search.class)
	public @ResponseBody SortedSet<Volunteer> findDuplicateVolunteers(@RequestParam String firstName,
			@RequestParam String lastName,
			@DateTimeFormat(pattern = DateUtil.TWO_DIGIT_DATE_ONLY) @RequestParam LocalDate dob) {
		if (StringUtils.isBlank(lastName) || StringUtils.isBlank(firstName))
			throw new IllegalArgumentException("Both last name and first name are required.");

		SortedSet<Volunteer> volunteers = new TreeSet<>();
		List<Volunteer> firstNameMatches = volunteerDAO.findByCriteria(firstName, null, null, null, dob, null, null,
				null, null, null, null, null, null, null, null);
		volunteers.addAll(firstNameMatches);

		List<Volunteer> lastNameMatches = volunteerDAO.findByCriteria(null, null, lastName, null, dob, null, null, null,
				null, null, null, null, null, null, null);
		volunteers.addAll(lastNameMatches);

		return volunteers;
	}

	@RequestMapping("/volunteerCreate.htm")
	// Don't want a Breadcrumb here since we want to force them to search first
	// every time - CPB
	// @Breadcrumb("Create Volunteer")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_EDIT + "')")
	public String volunteerCreate(ModelMap model, HttpServletRequest request) {
		Volunteer volunteer = new Volunteer();
		volunteer.setMealsEligible(1);

		VolunteerCommand command = new VolunteerCommand(volunteer, null);
		model.addAttribute(DEFAULT_COMMAND_NAME, command);
		createReferenceData(model);
		return "createVolunteer";
	}

	@RequestMapping("/volunteerEdit.htm")
	@Breadcrumb("Edit Volunteer")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_EDIT + "')")
	public String volunteerEdit(@RequestParam long id, ModelMap model, HttpServletRequest request) {
		Volunteer volunteer = volunteerDAO.findRequiredByPrimaryKey(id);

		TimeSummary summary = volunteerDAO.getTimeSummary(id);
		VolunteerCommand command = new VolunteerCommand(volunteer, summary);
		model.addAttribute(DEFAULT_COMMAND_NAME, command);

		// TODO query DB once we figure out how to join - CPB
		model.addAttribute("backgroundStatus", "No Issue Reported");
		model.addAttribute("allVolunteerFacilities", volunteerDAO.findFacilitiesForVolunteer(id));
		model.addAttribute("yearsVolunteering", workEntryDAO.getNumYearsWorked(id));

		createReferenceData(model);

		setFormAsReadOnly(model, !hasAllPermissionsAtCurrentFacility(VOLUNTEER_CREATE));

		return "editVolunteer";
	}

	private void createReferenceData(ModelMap model) {
		model.put("allGenders", genderDAO.findAllSorted());
		model.put("allLanguages", vtkLanguageDAO.findAllSorted());
		model.put("allTransportationMethods", transportationMethodDAO.findAllSorted());
		model.put("allAwards", awardDAO.findAllSorted());
		model.put("allShirtSizes", uniformDAO.findAllShirtSizes());

		Set<VAFacility> editSites = SecurityUtil
				.getFacilitiesWhereUserHasAllPermissions(PermissionType.VOLUNTEER_CREATE);
		Map<Long, Facility> i = facilityDAO.findByVAFacilities(editSites);
		List<Long> facilityIds = i.values().stream().filter(p -> !p.isCentralOffice()).map(p -> p.getId())
				.collect(Collectors.toList());
		model.addAttribute("volunteerEditSiteIds", StringUtils.join(facilityIds, ','));
	}

	@RequestMapping("/volunteerSubmit.htm")
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_EDIT + "')")
	public String volunteerSubmit(@ModelAttribute(DEFAULT_COMMAND_NAME) VolunteerCommand command, BindingResult result,
			SessionStatus status, ModelMap model, HttpServletRequest request) throws ValidationException {
		Volunteer volunteer = command.getVolunteer();

		boolean isEdit = volunteer.isPersistent();

		volunteerValidator.validate(command, result, false, "volunteer");
		boolean hasErrors = result.hasErrors();

		if (!hasErrors) {
			try {
				volunteer = volunteerService.saveOrUpdate(volunteer, getCurrentUser());
				userNotifier.notifyUserOnceWithMessage(request,
						getMessage(isEdit ? "volunteer.update.success" : "volunteer.create.success"));
			} catch (ServiceValidationException e) {
				webValidationService.handle(e, result);
				hasErrors = true;
			}
		}

		if (hasErrors) {
			createReferenceData(model);
			return isEdit ? "editVolunteer" : "createVolunteer";
		} else {
			status.setComplete();
			return "redirect:/volunteerEdit.htm?id=" + volunteer.getId();
		}
	}

	@RequestMapping(value = "/volunteer/uniform/createOrUpdate", method = RequestMethod.POST)
	// @PreAuthorize("hasAuthority('" + Permission.VOLUNTEER_VIEW + "')")
	public @ResponseBody boolean uniformCreateOrUpdate(@RequestParam long facilityId,
			@RequestParam(required = false) Long uniformId, @RequestParam long volunteerId,
			@RequestParam(required = false) ShirtSize size, @RequestParam(required = false) Integer count) {
		Uniform u;
		if (uniformId != null) {
			u = uniformDAO.findRequiredByPrimaryKey(uniformId);
		} else {
			u = new Uniform();
			Volunteer v = volunteerDAO.findRequiredByPrimaryKey(volunteerId);
			u.setVolunteer(v);
		}

		u.setShirtSize(size);
		u.setNumberOfShirts(count);

		Facility i = facilityDAO.findRequiredByPrimaryKey(facilityId);
		u.setFacility(i);

		u = uniformService.saveOrUpdate(u);
		return true;
	}

}
