var facilityId = null
var canEditAllFacilities = null

function onPageLoad(facilitySelected, createNew, editFacilityId, vaFacilitySet, editAllFacilities) {
	facilityId = editFacilityId
	canEditAllFacilities = editAllFacilities
	
	if (canEditAllFacilities) {
		$("#facilityHierarchyDiv").show()
		buildFacilityListTable()
		refreshFacilityListTable()
		filterFacilityListTable()
	} else {
		$("#facilityName").prop('disabled', true)
		$("#facilityParentSelect").prop('disabled', true)
		$("#facilityVISN").prop('disabled', true)
		$("#facilityType").prop('disabled', true)
		$("#facilityActive").prop('disabled', true)
	}
	
	buildLocationListTable()
	if (!createNew)
		refreshLocations()
	
	buildVAFacilityPopup()
	buildLocationPopup()
	
	$("#facilityParentSelect").multiselect({
		selectedText : function(numChecked, numTotal, checkedItems) {
			return $(checkedItems[0]).next().text() // abbreviate($(checkedItems[0]).next().text())
		},
		height: 300,
		minWidth: 250,
		multiple : false
	}).multiselectfilter()
	
	$(".vaFacilityControlledInput").toggle(!vaFacilitySet)
	$(".vaFacilityControlledValue").toggle(vaFacilitySet)
}

function buildVAFacilityPopup() {
	$("#sdsFacilityTable").DataTable({
        "ajax": ajaxHomePath + "/facility/quickSearch",
        "columns": [
            { "data": "name",
            	"render": function (data, type, full, meta) {
	    	      return '<a href=\'javascript:linkSDSFacility(' + full.id + ')\'>' + full.displayName + '</a>'
	    	    }
            }
        ],
        "dom" : '<"top"f>rt<"bottom">',
        "language": {
            zeroRecords: "", // Please search above by name or station
								// number.",
            search: "", // Search",
            searchPlaceholder: "Search by name or station number..."
        },
        "paging" : false,
        "processing": false,
        "scrolling" : false,
        "serverSide": true
    })
   
	 $("#sdsFacilityTableWrapper").dialog({
		autoOpen : false,
		modal : false,
		width : 550,
		height : 350,
		closeOnEscape : true,
		draggable : true,
		resizable : true,
		buttons : {
			'Cancel' : function() {
				$(this).dialog('close')
			}
		}
	})
}

function buildFacilityListTable() {
	$('#facilityList').DataTable({
		buttons : [ {
			extend : 'collection',
			text : 'Export',
			buttons : [ 'excel', {
				extend : 'pdfHtml5',
				orientation : 'portrait'
			}, 'print' ]
		} ],
		columnDefs : [
				{
					"targets" : 0,
					"data" : function(row, type, val, meta) {
						/* No link for VISNs*/
						if (row.depth == 0)
							return '<b>' + escapeHTML(row.item.displayName) + '</b>'
						
						if (type === 'display') {
							var activeClass = (facilityId == row.item.id) ? 'activeFacilityLink' : ''
							var t = '<a class="facilityLink ' + activeClass + '" id="facilityLink' + row.item.id + '" style="margin-left:' + (row.depth * 25)
									+ 'px" href="facilityEdit.htm?id=' + row.item.id + '">'
									+ escapeHTML(row.item.displayName) + '</a>'
							if (!row.item.active)
								t = '<span class="inactiveFacility">' + t + ' (Inactive)</span>'
							
							return t
						}
						// used for 'filter', 'sort', 'type' and undefined
						return row.item.displayName
					}
				},
				{
					"targets" : 1,
					"data" : function(row, type, val, meta) {
						return row.item.active ? 'ActiveNode' : ''
					},
					"visible" : false
				}],
		dom : '<"top"fB>rt<"bottom"pl><"clear">',
		language: {
			emptyTable: "Please wait...",
            zeroRecords: "", // Please search above by name or station
								// number.",
            search: "", // Search",
            searchPlaceholder: "Name or station number..."
        },
		order : [],
		paging : false,
		scrollY : "500px",
		scrollCollapse : true
	})
	
	$("#showInactiveFacilities").change(filterFacilityListTable)
}

function filterFacilityListTable() {
	var scrollPos = $("#facilityHierarchyDiv .dataTables_scrollBody").scrollTop()
	var isChecked = $("#showInactiveFacilities").is(':checked')
	var table = $('#facilityList').DataTable()
    table.columns(1).search(isChecked ? '' : 'ActiveNode').draw()
    $("#facilityHierarchyDiv .dataTables_scrollBody").scrollTop(scrollPos)
}

function refreshFacilityListTable() {
	var flatten = function(node, depth, result) {
		var facilityChildren = node.facilityChildren
		/* Let's save some space - CPB */
		delete node['facilityChildren']
		
		result.push({
			item : node,
			depth : depth
		})
		for (var i = 0; facilityChildren && i < facilityChildren.length; i++) {
			flatten(facilityChildren[i], depth + 1, result)
		}
	}

	$.ajax({
		url : ajaxHomePath + '/findFacilitiesForHierarchyDisplay',
		dataType : 'json',
		data : {},
		global: false,     // this makes sure ajaxStart is not triggered
		error : commonAjaxErrorHandler,
		success : function(r) {
			var flattenedResults = []
			for (var i = 0; i < r.length; i++) {
				flatten(r[i], 0, flattenedResults)
			}

			var table = $('#facilityList').DataTable()
			table.clear()
			table.rows.add(flattenedResults)
			rebuildTableFilters('facilityList')
			table.draw()
			
			if (facilityId)
				$('#facilityHierarchyDiv .dataTables_scrollBody').scrollTo($('a#facilityLink' + facilityId), 0, {
					offset: -25
				})
		}
	})
}

function buildLocationListTable() {
	var locationListCols = [{
		"targets" : 0,
		"data" : function(row, type, val, meta) {
			return row.name
		}
	}, {
		"targets" : 1,
		"data" : function(row, type, val, meta) {
			var addressHtml = getLocationAddressDashedBoxEl(row).outerHtml()
			return addressHtml
		}
	}, {
		"targets" : 2,
		"data" : function(row, type, val, meta) {
			var contactInfoHtml = getLocationContactDashedBoxEl(row).outerHtml()
			return contactInfoHtml
		}
	}, {
		"targets" : 3,
		"data" : function(row, type, val, meta) {
			return row.active ? 'Active' : 'Inactive'
		}
	}]
	
	// if we want to make screen read-only, change this - CPB
	if (true) {
		locationListCols[locationListCols.length] = {
			"targets" : 4,
			"data" : function(row, type, val, meta) {
				var actions = '<div style="margin:0 auto; text-align:center">'
				actions += '<a href="javascript:popupLocationEdit(' + row.id + ')"><img src="' + imgHome
					+ '/edit-small.gif" border="0" hspace="5" align="center"/></a>'
				actions += '<a href="javascript:deleteLocation('
						+ row.id + ')"><img src="' + imgHome
						+ '/delete.gif" border="0" hspace="5" align="center"/></a>'
						
				actions += '</div>'
				return actions;
			}
		}
	}
	    			
	$('#locationList').DataTable({
		"columnDefs" : locationListCols,
		"dom" : '<"top">rt<"bottom"pl><"clear">',
		"order" : [],
		"paging" : false,
		"scrollY" : "200px",
		"scrollCollapse" : true
	})
}

function buildLocationPopup() {
	$("#locationFieldsWrapper").dialog({
		autoOpen : false,
		modal : false,
		width : 700,
		height : 250,
		closeOnEscape : true,
		draggable : true,
		resizable : true,
		buttons : {
			'Submit' : function() {
				var locationId = $("#locationFieldsWrapper").data('locationId')
				
				var name = $("#locationName").val()
				var addressLine1 = $("#locationAddressLine1").val()
				var city = $("#locationAddressCity").val()
				var state = $("#locationAddressState").val()
				var zip = $("#locationAddressZip").val()
					
				
				if ($.trim(name) == '') {
					displayAttentionDialog('Please enter the name.')
					return
				}
				if ($.trim(addressLine1) == '') {
					displayAttentionDialog('Please enter the address line 1.')
					return
				}
				if ($.trim(city) == '') {
					displayAttentionDialog('Please enter the city.')
					return
				}
				if ($.trim(state) == '') {
					displayAttentionDialog('Please select the state.')
					return
				}
				if ($.trim(zip) == '') {
					displayAttentionDialog('Please enter the zip code.')
					return
				}
				
				var contactEmail = $("#locationContactEmail").val()
				if (!validateEmail(contactEmail)) {
					displayAttentionDialog("Please enter a valid contact email in the format 'user@domain.tld'.")
					return
				}
				
				$.ajax({
					url : ajaxHomePath + '/facility/location/saveOrUpdate',
					dataType : 'json',
					data : {
						facilityId : facilityId,
						locationId : locationId,
						name : name,
						addressLine1 : addressLine1,
						addressLine2 : $("#locationAddressLine2").val(),
						city : city,
						state : state,
						zip : zip,
						contactName : $("#locationContactName").val(),
						contactRole : $("#locationContactRole").val(),
						contactPhone : $("#locationContactPhone").val(),
						contactEmail : contactEmail,
						active : $("#locationActive").is(':checked')
					},
					error : commonAjaxErrorHandler,
					success : function() {
						$("#locationFieldsWrapper").dialog('close')
						refreshLocations()
					}
				})
			},
			'Cancel' : function() {
				$(this).dialog('close')
			}
		}
	})
}

function linkSDSFacility(vaFacilityId) {
	$.ajax({
		url : ajaxHomePath + '/linkSDSFacilityToFacility',
		dataType : 'json',
		data : {
			facilityId : facilityId,
			vaFacilityId : vaFacilityId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			document.location.href = "facilityEdit.htm?id=" + facilityId
			showSpinner()
		}
	})
}

function popupSDSFacilitySelection() {
	$("#sdsFacilityTableWrapper").dialog('open')
}

var allLocations = {}
function refreshLocations() {
	var table = $('#locationList').DataTable()
	table.clear()
	
	$.ajax({
		url : ajaxHomePath + '/facility/location',
		dataType : 'json',
		data : {
			facilityId : facilityId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			allLocations = {}
			for (var i = 0; i < r.length; i++)
				allLocations['' + r[i].id] = r[i]
				
			table.rows.add(r)
			rebuildTableFilters('locationList')
			table.draw()
			
			$("#locationList_filter3").val('Active')
			$("#locationList_filter3").change()
		}
	})
}

function popupLocationEdit(locationId) {
	var location = null
	if (locationId && allLocations[locationId])
		location = allLocations[locationId]
	$("#locationFieldsWrapper").data('locationId', locationId ? locationId : '')
	
	$("#locationName").val(location ? location.name : '')
	$("#locationAddressLine1").val(location ? location.addressLine1 : '')
	$("#locationAddressLine2").val(location ? location.addressLine2 : '')
	$("#locationAddressCity").val(location ? location.city : '')
	$("#locationAddressState").val(location && location.state ? location.state.id : '')
	$("#locationAddressZip").val(location ? location.zip : '')
	$("#locationContactName").val(location ? location.contactName : '')
	$("#locationContactRole").val(location ? location.contactRole : '')
	$("#locationContactPhone").val(location ? location.contactPhone : '')
	$("#locationContactEmail").val(location ? location.contactEmail : '')
	$("#locationActive").prop('checked', !location || location.active)
	$("#locationFieldsWrapper").dialog('open')
}

function deleteLocation(locationId) {
	$.ajax({
		url : ajaxHomePath + '/facility/location/delete',
		dataType : 'json',
		data : {
			locationId : locationId
		},
		error : commonAjaxErrorHandler,
		success : refreshLocations
	})
}

function unlinkSDSFacility() {
	$.ajax({
		url : ajaxHomePath + '/unlinkSDSFacilityFromFacility',
		dataType : 'json',
		data : {
			facilityId : facilityId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			document.location.href = "facilityEdit.htm?id=" + facilityId
			showSpinner()
		}
	})
}

function submitForm(isPersistent) {
	return true
}

function getLocationAddressDashedBoxEl(location) {
	var theHtml = '<table width="100%" class="addressBox">'
	theHtml += '<tr valign="top">'
	var addressHtml = ""
	if (location.addressMultilineDisplay)
		addressHtml = escapeHTML(location.addressMultilineDisplay)
	theHtml += '<td width="1%" nowrap>' + convertLinefeedToBR(addressHtml)
			+ '</td></tr></table>'
	return getBoxEl(theHtml, false)
}

function getLocationContactDashedBoxEl(location) {
	var theHtml = '<table width="100%" class="addressBox">'
	theHtml += '<tr valign="top"><td nowrap width="99%">'
	
	if (location) {
		var nameHtml = ""
		if (location.contactName)
			nameHtml += escapeHTML(location.contactName) + '<br>'
		var roleHtml = ""
		if (location.contactRole)
			roleHtml += '<i>' + escapeHTML(location.contactRole) + '</i><br>'
		var phoneHtml = ""
		if (location.contactPhone)
			phoneHtml = escapeHTML(location.contactPhone) + '<br>'
		var emailHtml = ""
		if (location.contactEmail)
			emailHtml = escapeHTML(location.contactEmail) + '<a href="mailto:'
					+ escapeHTML(location.contactEmail)
					+ '"><img alt="Click to email '
					+ escapeHTML(location.contactEmail) + '"' + 'src="' + imgHome
					+ '/envelope.jpg" height="14"'
					+ ' width="18" border="0" align="absmiddle"'
					+ ' style="padding-left: 4px; padding-right: 4px" /></a>'

		theHtml += nameHtml + roleHtml + phoneHtml + emailHtml
	}
	theHtml += '</td></tr></table>'
	return getBoxEl(theHtml, false)
}