function onPageLoad(isNew, isReadOnly, disableTerminationFields, anyTerminationFieldsSet) {
	if (isReadOnly)
		setPageTitle('View Volunteer')
	
	$('.dateInput').each(function() {
		$(this).enableDatePicker({
			showOn : "button",
			buttonImage : imgHome + "/calendar.gif",
			buttonImageOnly : true
		})
		$(this).mask(twoDigitDateMask)
	})
	
	var defaultParms = {
		"paging" : false,
		"processing" : true,
		"scrollCollapse" : true,
    	"scrollY" : "110px"
	}
	
	if (isNew) {
		editName()
	} else {
		buildOrganizationsTable(defaultParms, isReadOnly)
		refreshOrganizations()
		
		buildAssignmentsTable(defaultParms, isReadOnly)
		refreshAssignments()
		
		buildParkingStickersTable(defaultParms, isReadOnly)
		refreshParkingStickers()
		
		buildUniformsTable(defaultParms, isReadOnly)
		refreshUniforms()
		
		$("#primaryOrganization").change(function() {
			setPrimaryOrganization($(this).val())
		})
		
		$("#primaryFacility").change(function() {
			setPrimaryFacility($(this).val())
		})
	}
	
	// ----------- Meals fields
	
	var mealRemarksUpdated = function() {
		var theVal = $("#mealsEligibleInput").val()
		var show = $.trim(theVal) == '0'
		var theTextarea = $("#mealRemarks")
		theTextarea.attr('placeholder', show ? '*[Meal remarks - required]' : '[Meal remarks]')
		theTextarea.removeClass("placeholderRequired")
		if (show)
			theTextarea.addClass("placeholderRequired")
		
	}
	$("#mealsEligibleInput").change(mealRemarksUpdated)
	mealRemarksUpdated()
	
	// ----------- Terminated fields
	
	$('.terminatedFields').toggle(anyTerminationFieldsSet)
	$('.notTerminatedFields').toggle(!anyTerminationFieldsSet)
	
	var updateTerminationFields = function() {
		var terminatedWithCause = $("#terminatedWithCauseCheckbox").is(":checked")
		
		$("#terminationDateRequired").toggle(terminatedWithCause)
		
		var terminationRemarks = $("#terminationRemarks")
		terminationRemarks.attr('placeholder', terminatedWithCause ? '*[Termination remarks - required]' : '[Termination remarks]')
		terminationRemarks.removeClass("placeholderRequired")
		if (terminatedWithCause)
			terminationRemarks.addClass("placeholderRequired")
		
		if (disableTerminationFields) {
			$("#terminatedWithCauseCheckbox").attr('disabled', true)
			terminationRemarks.attr('disabled', true)
			$("#terminationDate").attr('disabled', true)
		}
	}
	$("#terminatedWithCauseCheckbox").change(updateTerminationFields)
	updateTerminationFields()
	
	// ----------- Awards fields
	
	$("#awardSelect").change(function() {
		$("#lastAwardHours").val(awardHoursMap[$(this).val()])
		if ($(this).val() == '')
			$("#dateOfLastAwardInput").val('')
	})
}

function buildAssignmentsTable(defaultParms, isReadOnly) {
	var assignmentCols = [ {
		"targets" : 0,
		"data" : function(row, type, val, meta) {
			if (type === 'filter') {
				return abbreviate(row.facilityAssignment.benefitingService.displayName, 25)
			}
			return row.facilityAssignment.benefitingService.displayName
		}
	}, {
		"targets" : 1,
		"data" : function(row, type, val, meta) {
			if (type === 'filter') {
				return abbreviate(row.facilityAssignment.facility.displayName, 25)
			}
			return row.facilityAssignment.facility.displayName
		}
	}, {
		"targets" : 2,
		"data" : function(row, type, val, meta) {
			return row.active ? 'Active' : 'Inactive'
		}
	}]
	
	if (!isReadOnly) {
		assignmentCols[assignmentCols.length] = {
			"targets" : 3,
			"data" : function(row, type, val, meta) {
				var actions = '<div style="margin:0 auto; text-align:center">'
				if (row.facilityAssignment.facility && volunteerEditSites.search(row.facilityAssignment.facility.id) != -1) {
					if (row.active)
						actions += '<a href="javascript:deleteAssignment('
								+ row.id + ')"><img src="' + imgHome
								+ '/delete.gif" border="0" hspace="5" align="center"/></a>'
					if (row.active === false)
						actions += '<a title="Reactivate Facility Assignment" href="javascript:reactivateFacilityAssignment('
							+ row.facilityAssignment.id + ')"><img src="' + imgHome
							+ '/revert.png" border="0" hspace="5" align="center"/></a>'
				}
				actions += '</div>'
				return actions;
			}
		}
	}
	
	$('#assignmentsTable').DataTable($.extend({}, defaultParms, {
		"columnDefs" : assignmentCols,
    	"dom": '<"top">rt<"bottom"l>',
    	"language" : {
			zeroRecords : "No Assignments",
			infoEmpty: "No assignments available",
			searchPlaceholder : "Search by name..."
		}
	}))
}

function buildOrganizationsTable(defaultParms, isReadOnly) {
	var orgCols = [{
		"targets" : 0,
		"data" : function(row, type, val, meta) {
			return row.organization.displayName
		}
	},{
		"targets" : 1,
		"data" : function(row, type, val, meta) {
			return row.organization.facility ? row.organization.facility.displayName : 'National'
		}
	},{
		"targets" : 2,
		"data" : function(row, type, val, meta) {
			return row.active ? 'Active' : 'Inactive'
		}
	}]
	
	if (!isReadOnly) {
		orgCols[orgCols.length] = {
			"targets" : 3,
			"data" : function(row, type, val, meta) {
				var actions = '<div style="margin:0 auto; text-align:center">'
				
				if (row.organization.facility && volunteerEditSites.search(row.organization.facility.id) != -1) {
					if (row.active)
						actions += '<a href="javascript:inactivateVolunteerOrganization('
								+ row.id + ')"><img src="'+ imgHome
								+ '/delete.gif" border="0" hspace="5" align="center"/></a>'
					if (row.active === false)
						actions += '<a title="Reactivate Organization" href="javascript:reactivateOrganization('
							+ row.organization.id + ')"><img src="'+ imgHome
							+ '/revert.png" border="0" hspace="5" align="center"/></a>'
				}
				actions += '</div>'
				return actions;
			}
		}
	}
	
	$('#organizationsTable').DataTable($.extend({}, defaultParms, {
		"columnDefs" : orgCols,
    	"dom": '<"top">rt<"bottom"l>',
    	"language" : {
			zeroRecords : "No Organizations",
			searchPlaceholder : "Search by name..."
		}
	}))
}

function buildParkingStickersTable(defaultParms, isReadOnly) {
	var parkingCols = [ { // facility, sticker, license, expires
		"targets" : 0,
		"data" : function(row, type, val, meta) {
			return row.facility ? row.facility.displayName : ''
		}
	}, {
		"targets" : 1,
		"data" : function(row, type, val, meta) {
			return row.stickerNumber
		}
	}, {
		"targets" : 2,
		"data" : function(row, type, val, meta) {
			if (row.state == null && row.licensePlate == null) return "(unknown)"
			var stateVal = row.state ? row.state.postalName : ""
			return stateVal + " ["  + (row.licensePlate ? row.licensePlate : '(unknown)') + "]"
			
		}
	}]
	
	if (!isReadOnly) {
		parkingCols[parkingCols.length] = {
			"targets" : 3,
			"data" : function(row, type, val, meta) {
				var actions = '<div style="margin:0 auto; text-align:center">'
				
				if (row.facility && volunteerEditSites.search(row.facility.id) != -1) {
						actions += '<a href="javascript:showParkingStickerDetailsPopup('
							+ row.id + ')"><img src="' + imgHome
							+ '/edit-small.gif" border="0" hspace="5" align="center"/></a>'
					
						actions += '<a href="javascript:deleteParkingSticker('
								+ row.id + ')"><img src="' + imgHome
								+ '/delete.gif" border="0" hspace="5" align="center"/></a>'
				}
				actions += '</div>'
				return actions
			}
		}
	}
	
	$('#parkingStickersTable').DataTable($.extend({}, defaultParms, {
		"columnDefs" : parkingCols,
    	"dom": '<"top">rt<"bottom"l>',
    	"language" : {
			zeroRecords : "No parking stickers available",
			infoEmpty: "No parking stickers available",
			searchPlaceholder : "Search by name..."
		}
	}))
}

function buildUniformsTable(defaultParms, isReadOnly) {
	var uniformCols = [{ // shirt size, uniform, number of shirts
		"targets" : 0,
		"data" : function(row, type, val, meta) {
			return row.facility ? row.facility.displayName : ''
		}
	}, {
		"targets" : 1,
		"data" : function(row, type, val, meta) {
			return row.shirtSize ? row.shirtSize.name : ''
		}
	}, {
		"targets" : 2,
		"data" : function(row, type, val, meta) {
			return row.numberOfShirts
		}
	}]
	
	if (!isReadOnly) {
		uniformCols[uniformCols.length] = {
			"targets" : 3,
			"data" : function(row, type, val, meta) {
				var actions = '<div style="margin:0 auto; text-align:center">'
				
				if (row.facility && volunteerEditSites.search(row.facility.id) != -1) {
					actions += '<a href="javascript:showUniformDetailsPopup('
						+ row.id + ')"><img src="' + imgHome
						+ '/edit-small.gif" border="0" hspace="5" align="center"/></a>'
					actions += '<a href="javascript:deleteUniform('
							+ row.id + ')"><img src="' + imgHome
							+ '/delete.gif" border="0" hspace="5" align="center"/></a>'
				}
				
				actions += '</div>'
				return actions
			}
		}
	}
	
	$('#uniformsTable').DataTable($.extend({}, defaultParms, {
		"columnDefs" : uniformCols,
    	"dom": '<"top">rt<"bottom"l>',
    	"language" : {
			zeroRecords : "No Uniforms",
			infoEmpty: "No uniforms available",
			searchPlaceholder : "Search..."
		}
	}))
}

function editName() {
	$(".fixedNameFields").hide()
	$(".nameInputs").show()
}

function terminateVolunteer() {
	$('.terminatedFields').show()
	$('.notTerminatedFields').hide()
}

function refreshOrganizations() {
	$.ajax({
		url : ajaxHomePath + '/volunteerOrganizations',
		dataType : 'json',
		data : {
			volunteerId: volunteerId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			redrawWithRows('organizationsTable', r.organizations)
			$("#organizationsTable_filter2").val('Active')
			$("#organizationsTable_filter2").change()
			
			var uniqueOrganizations = new SortedArray([])
			var organizations = []
			$(r.organizations).each(function(index, item) {
				if (uniqueOrganizations.search(item.organization.id) != -1)
					return
				if (item.active) {
					organizations.push({
						id: item.organization.id,
						name: item.organization.displayName
					})
					uniqueOrganizations.insert(item.organization.id)
				}
			})
			if (r.primaryOrganization && uniqueOrganizations.search(r.primaryOrganization.id) == -1) {
				organizations.push({
					id: r.primaryOrganization.id,
					name: r.primaryOrganization.displayName
				})
			}
			
			var orgSelect = $('#primaryOrganization')
			var orgsAvail = organizations.length > 0
			orgSelect.toggle(orgsAvail)
			$('#noOrganizationsAvailable').toggle(!orgsAvail)
			orgSelect.empty()
			
			if (!r.primaryOrganization) {
				orgSelect.append($('<option>', {
						value : '',
						text : '(Unknown)',
						selected : true
					}))
			}
			
			$(organizations).each(function(index, item) {
				orgSelect.append($('<option>', {
					value : item.id,
					text : item.name,
					selected : r.primaryOrganization && item.id == r.primaryOrganization.id
				}))
			})
		}
	})
}

function deleteParkingSticker(parkingStickerId) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/deleteParkingSticker",
		type : "POST",
		dataType : 'json',
		data : {
			parkingStickerId : parkingStickerId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			refreshParkingStickers()
		}
	})
}

function deleteUniform(uniformId) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/deleteUniform",
		type : "POST",
		dataType : 'json',
		data : {
			uniformId : uniformId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			refreshUniforms()
		}
	})
}

function deleteAssignment(volunteerFacilityAssignmentId) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/deleteFacilityAssignment",
		type : "POST",
		dataType : 'json',
		data : {
			volunteerFacilityAssignmentId : volunteerFacilityAssignmentId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			if (r) {
				displayAttentionDialog('The primary facility for this volunteer was inactivated; please choose another primary facility.')
			}
			refreshAssignments()
		}
	})
}

function inactivateVolunteerOrganization(volunteerOrganizationId) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/inactivateOrganization",
		type : "POST",
		dataType : 'json',
		data : {
			volunteerOrganizationId : volunteerOrganizationId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			if (r) {
				displayAttentionDialog('The primary organization for this volunteer was inactivated; please choose another primary organization.')
			}
			refreshOrganizations()
		}
	})
}

function reactivateOrganization(theId) {
	addOrReactivateOrganization({ id : theId })
}

function addOrReactivateOrganization(organizationObj) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/addOrganization",
		type : "POST",
		dataType : 'json',
		data : {
			volunteerId : volunteerId,
			organizationId : organizationObj.id
		},
		error : commonAjaxErrorHandler,
		success : refreshOrganizations
	})
}

function reactivateFacilityAssignment(facilityAssignmentId) {
	for (var i = 0; i < allVolunteerAssignments.length; i++) {
		var item = allVolunteerAssignments[i]
		if (item.facilityAssignment.id == facilityAssignmentId) {
			addOrReactivateFacilityAssignment(item.facilityAssignment)
			return
		}
	}
}

function addOrReactivateFacilityAssignment(facilityAssignmentObj) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/addFacilityAssignment",
		type : "POST",
		dataType : 'json',
		data : {
			volunteerId : volunteerId,
			facilityAssignmentId : facilityAssignmentObj.id
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			if (!r) {
				displayAttentionDialog('This volunteer assignment cannot be activated. Please activate the facility assignment with benefiting service "'
						+ facilityAssignmentObj.benefitingService.displayName + '" at the facility "'
						+ facilityAssignmentObj.facility.displayName + '" first.')
				return
			} else {
				refreshAssignments()
			}
		}
	})
}

function setPrimaryOrganization(organizationId) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/setPrimaryOrganization",
		type : "POST",
		dataType : 'json',
		data : {
			volunteerId : volunteerId,
			organizationId : organizationId
		},
		error : commonAjaxErrorHandler,
		success : function(result) {
			displayAttentionDialog('The primary organization was updated successfully.')
			refreshOrganizations()
		}
	})
}

function setPrimaryFacility(facilityId) {
	$.ajax({
		url : ajaxHomePath + "/volunteer/setPrimaryFacility",
		type : "POST",
		dataType : 'json',
		data : {
			volunteerId : volunteerId,
			facilityId : facilityId
		},
		error : commonAjaxErrorHandler,
		success : function(result) {
			displayAttentionDialog('The primary facility was updated successfully.')
			refreshAssignments()
		}
	})
}

var allVolunteerAssignments = undefined
function refreshAssignments() {
	$.ajax({
		url : ajaxHomePath + '/volunteerAssignments',
		dataType : 'json',
		data : {
			volunteerId: volunteerId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			allVolunteerAssignments = r.assignments
			redrawWithRows('assignmentsTable', r.assignments)
			
			$("#assignmentsTable_filter2").val('Active')
			$("#assignmentsTable_filter2").change()
			
			var uniqueFacilities = new SortedArray([])
			var facilities = []
			if (r.primaryFacility) {
				facilities.push({
					id: r.primaryFacility.id,
					name: r.primaryFacility.displayName
				})
				uniqueFacilities.insert(r.primaryFacility.id)
			}
			$(r.assignments).each(function(index, item) {
				if (uniqueFacilities.search(item.facilityAssignment.facility.id) != -1)
					return
					
				facilities.push({
					id: item.facilityAssignment.facility.id,
					name: item.facilityAssignment.facility.displayName
				})
				uniqueFacilities.insert(item.facilityAssignment.facility.id)
			})
			
			var facSelect = $('#primaryFacility')
			var facilitiesAvail = facilities.length > 0
			facSelect.toggle(facilitiesAvail)
			$('#noFacilitiesAvailable').toggle(!facilitiesAvail)
			facSelect.empty()
			
			if (!r.primaryFacility) {
				facSelect.append($('<option>', {
					value : '',
					text : '(Unknown)',
					selected : true
				}))
			}
			
			$(facilities).each(function(index, item) {
				facSelect.append($('<option>', {
					value : item.id,
					text : item.name,
					selected : r.primaryFacility && item.id == r.primaryFacility.id
				}))
			})
		}
	})
}

function hasActiveAssignmentAtFacility(facilityId) {
	if (typeof allVolunteerAssignments == 'undefined') {
		alert('There was an error retrieving / processing assignments for this volunteer')
		return
	}
	
	for (var i = 0; i < allVolunteerAssignments.length; i++) {
		var item = allVolunteerAssignments[i]
		if (item.facilityAssignment.facility.id == facilityId && item.active && item.facilityAssignment.active)
			return true
	}
	return false
}

var parkingStickerData = {}
function refreshParkingStickers() {
	$.ajax({
		url : ajaxHomePath + '/volunteerParkingStickers',
		dataType : 'json',
		data : {
			volunteerId: volunteerId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			parkingStickerData = {}
			$(r).each(function(index, item) {
				parkingStickerData[item.id] = item
			})
			redrawWithRows('parkingStickersTable', r)
		}
	})
}

var uniformData = {}
function refreshUniforms() {
	$.ajax({
		url : ajaxHomePath + '/volunteerUniforms',
		dataType : 'json',
		data : {
			volunteerId: volunteerId
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			uniformData = {}
			$(r).each(function(index, item) {
				uniformData[item.id] = item
			})
			redrawWithRows('uniformsTable', r)
		}
	})
}

function redrawWithRows(tableId, rows) {
	var table = $('#' + tableId).DataTable()
	table.clear()
	table.rows.add(rows)
	rebuildTableFilters(tableId)
	table.draw()
}

function toggleFields(className, val) {
	$('.' + className).toggle(val)
	$('#show_' + className).toggle(!val)
	$('#hide_' + className).toggle(val)
}

function submitForm(isEdit) {
	doubleClickSafeguard($("#submitButton"))
	
	var firstName = $("#firstNameInput").val()
	var lastName = $("#lastNameInput").val()
	if ($.trim(firstName) == '' || $.trim(lastName) == '') {
		displayAttentionDialog('Please enter both first name and last name.')
		return false
	}
	
	var dobVal = $.trim($("#dateOfBirthInput").val())
	if (dobVal == '' || !validateDate(dobVal)) {
		displayAttentionDialog('Please enter a valid date of birth.')
		return false
	}
	
	if (isEdit) return true
	
	if (firstName == commandFirstName && lastName == commandLastName
			&& removeChars(dobVal, ['-', '/']) == getAsMMDDYYYY(commandDob))
		return true
	
	$("#submitButton").val('Checking duplicates...')
	$("#submitButton").prop('disabled', true)
	
	$.ajax({
		url : ajaxHomePath + '/volunteerDuplicateCheck',
		dataType : 'json',
		data : {
			firstName: $("#firstNameInput").val(),
			lastName: $("#lastNameInput").val(),
			dob: $("#dateOfBirthInput").val()
		},
		error : commonAjaxErrorHandler,
		success : function(r) {
			if (r.length == 0) {
				$("#submitButton").val('Submitting form...')
				finalSubmit()
				return
			}
			
			$("#submitButton").val('Create')
			$("#submitButton").prop('disabled', false)				
			popupVolunteerSearch('volProfCheckForDups', r)
		}
	})
	
	return false
}

function finalSubmit() {
	$("#volunteerForm")[0].submit()
}