package gov.va.vss.persistence;

import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import gov.va.vss.AbstractTransactionalWebDAOTest;
import gov.va.vss.model.ScopeType;
import gov.va.vss.model.organization.AbstractBasicOrganization;
import gov.va.vss.model.organization.BasicOrganization;
import gov.va.vss.model.organization.Organization;
import gov.va.vss.persistence.dao.CustomizableAppDAO;

public class TestOrganizationDAO extends AbstractTransactionalWebDAOTest<AbstractBasicOrganization> {

	@Override
	protected CustomizableAppDAO<AbstractBasicOrganization> getDAO() {
		return organizationDAO;
	}

	@Override
	protected Organization getInstanceToSave() {
		Organization o = new Organization();
		o.setName("UnitTestOrganization");
		o.setFullName(o.getName());
		o.setCode(99999);
		o.setScope(ScopeType.LOCAL);
		return o;
	}

	@Override
	protected boolean testDelete() {
		return true;
	}

	@Test
	public void testFindByCriteria() {
		List<Long> staList1 = new ArrayList<Long>();
		staList1.add(1234L);
		organizationDAO.findByCriteria("test", false, false, staList1, null, null, null);
		organizationDAO.findByCriteria("test", true, false, staList1, null, null, null);
		organizationDAO.findByCriteria("test", false, true, staList1, null, null, null);
		organizationDAO.findByCriteria("test", true, true, staList1, null, null, null);

		List<Long> staList2 = new ArrayList<Long>();
		staList2.add(1000098L);
		List<BasicOrganization> resultsFor442 = organizationDAO.findByCriteria(null, true, true, null, null, null,
				null);
		Assert.assertFalse("Failed - 442 didn't return any local or national results", resultsFor442.isEmpty());
		resultsFor442 = organizationDAO.findByCriteria(null, false, true, null, null, null, null);
		Assert.assertFalse("Failed - 442 didn't return any local results", resultsFor442.isEmpty());

	}

	@Test
	public void testFindNextAvailableCode() {
		organizationDAO.findNextAvailableCode(ScopeType.NATIONAL, null);
		organizationDAO.findNextAvailableCode(ScopeType.LOCAL, 1000098L);
		organizationDAO.findNextAvailableCode(ScopeType.LOCAL, -1L);

		// CCR 275208 Local code should be in the range 900-9999
		try {
			// test that the local code max out at 9999
			organizationDAO.findNextAvailableCode(ScopeType.LOCAL, 1000212L);
			Assert.assertFalse("Failed - facility 623 should not return any local results", true);
		} catch (AssertionError e) {
			// expecting exception here
		}

		// test that the local code starts from 900
		// station FK=281, 565GC, SDS ID=1001126
		int nextCode = organizationDAO.findNextAvailableCode(ScopeType.LOCAL, 1001126L);
		Assert.assertEquals(900, nextCode);
	}

}
